@@ -1,31 +1,40 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: pgConn.cpp,v 1.60 2004/11/21 14:19:41 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // pgConn.cpp - PostgreSQL Connection class
 //
-//////////////////////////////////////////////////////////////////////////
+/////////////////////////////////////////////////////////////////////////
+
+#include "pgAdmin3.h"
 
 // wxWindows headers
 #include <wx/wx.h>
 
 // PostgreSQL headers
 #include <libpq-fe.h>
+#include "pgfeatures.h"
 
 // Network  headers
 #ifdef __WXMSW__
-    #include <winsock.h>
+#include <winsock.h>
 #else
-    #include <arpa/inet.h>
-    #include <netdb.h>
-    #include <netinet/in.h>
+
+#include <arpa/inet.h>
+#include <netdb.h>
+#include <netinet/in.h>
+
+#ifndef INADDR_NONE
+#define INADDR_NONE (-1)
+#endif
+
 #endif
 
 // App headers
 #include "pgConn.h"
-#include "pgAdmin3.h"
 #include "misc.h"
 #include "pgSet.h"
 #include "sysLogger.h"
@@ -39,7 +48,7 @@ static void pgNoticeProcessor(void *arg, const char *message)
 }
 
 
-pgConn::pgConn(const wxString& server, const wxString& database, const wxString& username, const wxString& password, int port, int sslmode)
+pgConn::pgConn(const wxString& server, const wxString& database, const wxString& username, const wxString& password, int port, int sslmode, OID oid)
 {
     wxLogInfo(wxT("Creating pgConn object"));
     wxString msg, hostip;
@@ -53,6 +62,8 @@ pgConn::pgConn(const wxString& server, const wxString& database, const wxString&
     conn=0;
     majorVersion=0;
     noticeArg=0;
+    connStatus = PGCONN_BAD;
+    memset(features, 0, sizeof(features));
     
 #ifdef __WXMSW__
     struct in_addr ipaddr;
@@ -67,7 +78,7 @@ pgConn::pgConn(const wxString& server, const wxString& database, const wxString&
 		host = gethostbyname(server.ToAscii());
 		if (host == NULL)
 		{
-            resolvedIP = FALSE;
+            connStatus = PGCONN_DNSERR;
             wxLogError(__("Could not resolve hostname %s"), server.c_str());
 			return;
 		}
@@ -79,26 +90,25 @@ pgConn::pgConn(const wxString& server, const wxString& database, const wxString&
     else
         hostip = server;
 
-    resolvedIP = TRUE;
     wxLogInfo(wxT("Server name: %s (resolved to: %s)"), server.c_str(), hostip.c_str());
 
     // Create the connection string
     wxString connstr;
     if (!server.IsEmpty()) {
       connstr.Append(wxT(" hostaddr="));
-      connstr.Append(hostip);
+      connstr.Append(qtString(hostip));
     }
     if (!database.IsEmpty()) {
       connstr.Append(wxT(" dbname="));
-      connstr.Append(database);
+      connstr.Append(qtString(database));
     }
     if (!username.IsEmpty()) {
       connstr.Append(wxT(" user="));
-      connstr.Append(username);
+      connstr.Append(qtString(username));
     }
     if (!password.IsEmpty()) {
       connstr.Append(wxT(" password="));
-      connstr.Append(password);
+      connstr.Append(qtString(password));
     }
     if (port > 0) {
       connstr.Append(wxT(" port="));
@@ -128,38 +138,59 @@ pgConn::pgConn(const wxString& server, const wxString& database, const wxString&
     // Open the connection
     wxLogInfo(wxT("Opening connection with connection string: %s"), connstr.c_str());
 
+#if wxUSE_UNICODE
+    conn = PQconnectdb(connstr.mb_str(wxConvUTF8));
+    if (PQstatus(conn) != CONNECTION_OK)
+    {
+        PQfinish(conn);
+        conn = PQconnectdb(connstr.mb_str(wxConvLibc));
+    }
+#else
     conn = PQconnectdb(connstr.ToAscii());
+#endif
+
     dbHost = server;
 
     // Set client encoding to Unicode/Ascii
     if (PQstatus(conn) == CONNECTION_OK)
     {
-
+        connStatus = PGCONN_OK;
         PQsetNoticeProcessor(conn, pgNoticeProcessor, this);
 
-        pgSet *set=ExecuteSet(
-            wxT("SELECT pg_encoding_to_char(encoding) AS encoding, datlastsysoid\n")
-            wxT("  FROM pg_database WHERE datname=") + qtString(database));
+
+        wxString sql=wxT("SELECT oid, pg_encoding_to_char(encoding) AS encoding, datlastsysoid\n")
+                      wxT("  FROM pg_database WHERE ");
+        if (oid)
+            sql += wxT("oid = ") + NumToStr(oid);
+        else
+            sql += wxT("datname=") + qtString(database);
+
+        pgSet *set = ExecuteSet(sql);
+
+
         if (set)
         {
             if (set->ColNumber(wxT("\"datlastsysoid\"")) >= 0)
                 needColQuoting = true;
 
-            lastSystemOID = set->GetLong(wxT("datlastsysoid"));
+            lastSystemOID = set->GetOid(wxT("datlastsysoid"));
+            dbOid = set->GetOid(wxT("oid"));
             wxString encoding = set->GetVal(wxT("encoding"));
 
 #if wxUSE_UNICODE
-            if (encoding != wxT("SQL_ASCII"))
+            if (encoding != wxT("SQL_ASCII") && encoding != wxT("MULE_INTERNAL"))
             {
                 encoding = wxT("UNICODE");
                 conv = &wxConvUTF8;
             }
+            else
+                conv = &wxConvLibc;
 #endif
 
             wxLogInfo(wxT("Setting client_encoding to '%s'"), encoding.c_str());
             if (PQsetClientEncoding(conn, encoding.ToAscii()))
-                wxLogError(wxT("%s"), PQerrorMessage(conn));
-            
+				wxLogError(wxT("%s"), GetLastError().c_str());
+
             delete set;
         }
     }
@@ -169,8 +200,17 @@ pgConn::pgConn(const wxString& server, const wxString& database, const wxString&
 pgConn::~pgConn()
 {
     wxLogInfo(wxT("Destroying pgConn object"));
+    Close();
+}
+
+
+
+void pgConn::Close()
+{
     if (conn)
         PQfinish(conn);
+    conn=0;
+    connStatus=PGCONN_BAD;
 }
 
 
@@ -183,7 +223,7 @@ extern void *PQgetssl(PGconn *conn);
 
 bool pgConn::IsSSLconnected()
 {
-    return (PQstatus(conn) == CONNECTION_OK && PQgetssl(conn) != NULL);
+    return (conn && PQstatus(conn) == CONNECTION_OK && PQgetssl(conn) != NULL);
 }
 #endif
 
@@ -195,6 +235,12 @@ void pgConn::RegisterNoticeProcessor(PQnoticeProcessor proc, void *arg)
 }
 
 
+wxString pgConn::SystemNamespaceRestriction(const wxString &nsp)
+{
+    return wxT("(") + nsp + wxT(" NOT LIKE 'pg\\_%' AND ") + nsp + wxT(" NOT LIKE 'information_schema')");
+}
+
+
 void pgConn::Notice(const char *msg)
 {
     wxString str(msg, *conv);
@@ -211,23 +257,27 @@ void pgConn::Notice(const char *msg)
 
 bool pgConn::ExecuteVoid(const wxString& sql)
 {
+    if (GetStatus() != PGCONN_OK)
+        return false;
+
     // Execute the query and get the status.
     PGresult *qryRes;
 
     wxLogSql(wxT("Void query (%s:%d): %s"), this->GetHost().c_str(), this->GetPort(), sql.c_str());
     qryRes = PQexec(conn, sql.mb_str(*conv));
-    int res = PQresultStatus(qryRes);
+    lastResultStatus = PQresultStatus(qryRes);
 
     // Check for errors
-    if (res != PGRES_TUPLES_OK &&
-        res != PGRES_COMMAND_OK)
+    if (lastResultStatus != PGRES_TUPLES_OK &&
+        lastResultStatus != PGRES_COMMAND_OK)
     {
-        wxLogError(wxT("%s"), wxString(PQerrorMessage(conn), *conv).c_str());
+        LogError();
+        return false;
     }
 
     // Cleanup & exit
     PQclear(qryRes);
-    return res == PGRES_TUPLES_OK || res == PGRES_COMMAND_OK;
+    return  true;
 }
 
 
@@ -244,63 +294,71 @@ bool pgConn::HasPrivilege(const wxString &objTyp, const wxString &objName, const
 
 wxString pgConn::ExecuteScalar(const wxString& sql)
 {
-    // Execute the query and get the status.
-    PGresult *qryRes;
-    wxLogSql(wxT("Scalar query (%s:%d): %s"), this->GetHost().c_str(), this->GetPort(), sql.c_str());
-    qryRes = PQexec(conn, sql.mb_str(*conv));
-        
-    // Check for errors
-    if (PQresultStatus(qryRes) != PGRES_TUPLES_OK)
+    wxString result;
+
+    if (GetStatus() == PGCONN_OK)
     {
-        wxLogError(wxT("%s"), wxString(PQerrorMessage(conn), *conv).c_str());
+        // Execute the query and get the status.
+        PGresult *qryRes;
+        wxLogSql(wxT("Scalar query (%s:%d): %s"), this->GetHost().c_str(), this->GetPort(), sql.c_str());
+        qryRes = PQexec(conn, sql.mb_str(*conv));
+        lastResultStatus = PQresultStatus(qryRes);
+        
+        // Check for errors
+        if (lastResultStatus != PGRES_TUPLES_OK)
+        {
+            LogError();
+            PQclear(qryRes);
+            return wxEmptyString;
+        }
+
+	    // Check for a returned row
+        if (PQntuples(qryRes) < 1)
+        {
+		    wxLogInfo(wxT("Query returned no tuples"));
+            PQclear(qryRes);
+            return wxEmptyString;
+	    }
+	    
+	    // Retrieve the query result and return it.
+        result=wxString(PQgetvalue(qryRes, 0, 0), *conv);
+
+        wxLogSql(wxT("Query result: %s"), result.c_str());
+
+        // Cleanup & exit
         PQclear(qryRes);
-        return wxEmptyString;
     }
 
-	// Check for a returned row
-    if (PQntuples(qryRes) < 1)
-    {
-		wxLogInfo(wxT("Query returned no tuples"));
-        PQclear(qryRes);
-        return wxEmptyString;
-	}
-	
-	// Retrieve the query result and return it.
-    wxString result;
-    result=wxString(PQgetvalue(qryRes, 0, 0), *conv);
-
-    wxLogSql(wxT("Query result: %s"), result.c_str());
-
-    // Cleanup & exit
-    PQclear(qryRes);
     return result;
 }
 
 pgSet *pgConn::ExecuteSet(const wxString& sql)
 {
     // Execute the query and get the status.
-    PGresult *qryRes;
-    wxLogSql(wxT("Set query (%s:%d): %s"), this->GetHost().c_str(), this->GetPort(), sql.c_str());
-    qryRes = PQexec(conn, sql.mb_str(*conv));
-
-    int status= PQresultStatus(qryRes);
-
-    if (status == PGRES_TUPLES_OK || status == PGRES_COMMAND_OK)
+    if (GetStatus() == PGCONN_OK)
     {
-        pgSet *set = new pgSet(qryRes, conn, *conv, needColQuoting);
-        if (!set)
+        PGresult *qryRes;
+        wxLogSql(wxT("Set query (%s:%d): %s"), this->GetHost().c_str(), this->GetPort(), sql.c_str());
+        qryRes = PQexec(conn, sql.mb_str(*conv));
+
+        lastResultStatus= PQresultStatus(qryRes);
+
+        if (lastResultStatus == PGRES_TUPLES_OK || lastResultStatus == PGRES_COMMAND_OK)
         {
-            wxLogError(__("Couldn't create a pgSet object!"));
+            pgSet *set = new pgSet(qryRes, this, *conv, needColQuoting);
+            if (!set)
+            {
+                wxLogError(__("Couldn't create a pgSet object!"));
+                PQclear(qryRes);
+            }
+    	    return set;
+        }
+        else
+        {
+            LogError();
             PQclear(qryRes);
         }
-    	return set;
     }
-    else
-    {
-        wxLogError(wxT("%s"), wxString(PQerrorMessage(conn), *conv).c_str());
-        PQclear(qryRes);
-    }
-
     return 0;
 }
 
@@ -308,13 +366,73 @@ pgSet *pgConn::ExecuteSet(const wxString& sql)
 // Info
 //////////////////////////////////////////////////////////////////////////
 
+wxString pgConn::GetLastError() const
+{ 
+    wxString errmsg;
+	char *pqErr;
+    if (conn && (pqErr = PQerrorMessage(conn)) != 0)
+        errmsg=wxString(pqErr, wxConvUTF8);
+    else
+    {
+        if (connStatus == PGCONN_BROKEN)
+            errmsg = _("Connection to database broken.");
+        else
+            errmsg = _("No connection to database.");
+    }
+    return errmsg;
+}
+
+
+
+void pgConn::LogError()
+{
+    if (conn)
+    {
+        wxLogError(wxT("%s"), GetLastError().c_str());
+
+        IsAlive();
+#if 0
+        ConnStatusType status = PQstatus(conn);
+        if (status == CONNECTION_BAD)
+        {
+            PQfinish(conn);
+            conn=0;
+            connStatus = PGCONN_BROKEN;
+        }
+#endif
+    }
+}
+
+
+
+bool pgConn::IsAlive()
+{
+    if (GetStatus() != PGCONN_OK)
+        return false;
+
+    PGresult *qryRes = PQexec(conn, "SELECT 1;");
+    lastResultStatus = PQresultStatus(qryRes);
+    PQclear(qryRes);
+
+    // Check for errors
+    if (lastResultStatus != PGRES_TUPLES_OK)
+    {
+        PQfinish(conn);
+        conn=0;
+        connStatus = PGCONN_BROKEN;
+        return false;
+    }
+
+    return true;
+}
+
+
 int pgConn::GetStatus() const
 {
-    if(resolvedIP) {
-	    return PQstatus(conn);
-    } else {
-        return PGCONN_DNSERR;
-    }
+    if (conn)
+        ((pgConn*)this)->connStatus = PQstatus(conn);
+
+    return connStatus;
 }
 
 
@@ -332,3 +450,48 @@ bool pgConn::BackendMinimumVersion(int major, int minor)
     }
 	return majorVersion > major || (majorVersion == major && minorVersion >= minor);
 }
+
+
+bool pgConn::HasFeature(int featureNo)
+{
+    if (!features[FEATURE_INITIALIZED])
+    {
+        features[FEATURE_INITIALIZED] = true;
+
+        pgSet *set=ExecuteSet(
+            wxT("SELECT proname, pronargs, proargtypes[0] AS arg0, proargtypes[1] AS arg1, proargtypes[2] AS arg2\n")
+            wxT("  FROM pg_proc\n")
+            wxT(" WHERE proname IN ('pg_tablespace_size', 'pg_file_read', 'pg_rotate_log',")
+            wxT(                  " 'pg_postmaster_starttime', 'pg_terminate_backend', 'pg_reload_conf')"));
+
+        if (set)
+        {
+            while (!set->Eof())
+            {
+                wxString proname=set->GetVal(wxT("proname"));
+                long pronargs = set->GetLong(wxT("pronargs"));
+
+                if (proname == wxT("pg_tablespace_size") && pronargs == 1 && set->GetLong(wxT("arg0")) == 26)
+                    features[FEATURE_SIZE]= true;
+                else if (proname == wxT("pg_file_read") && pronargs == 3 && set->GetLong(wxT("arg0")) == 25
+                    && set->GetLong(wxT("arg1")) == 20 && set->GetLong(wxT("arg2")) == 20)
+                    features[FEATURE_FILEREAD] = true;
+                else if (proname == wxT("pg_rotate_log") && pronargs == 0)
+                    features[FEATURE_ROTATELOG] = true;
+                else if (proname == wxT("pg_postmaster_starttime") && pronargs == 0)
+                    features[FEATURE_POSTMASTER_STARTTIME] = true;
+                else if (proname == wxT("pg_terminate_backend") && pronargs == 1 && set->GetLong(wxT("arg0")) == 23)
+                    features[FEATURE_TERMINATE_BACKEND] = true;
+                else if (proname == wxT("pg_reload_conf") && pronargs == 0)
+                    features[FEATURE_RELOAD_CONF] = true;
+
+                set->MoveNext();
+            }
+            delete set;
+        }
+    }
+
+    if (featureNo <= FEATURE_INITIALIZED || featureNo >= FEATURE_LAST)
+        return false;
+    return features[featureNo];
+}
