@@ -1,13 +1,15 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: pgSet.cpp,v 1.48 2004/10/15 10:43:45 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // pgSet.cpp - PostgreSQL ResultSet class
 //
 //////////////////////////////////////////////////////////////////////////
 
+#include "pgAdmin3.h"
 // wxWindows headers
 #include <wx/wx.h>
 
@@ -18,11 +20,10 @@
 // App headers
 #include "pgSet.h"
 #include "pgConn.h"
-#include "pgAdmin3.h"
 #include "sysLogger.h"
 #include "pgDefs.h"
 
-pgSet::pgSet(PGresult *newRes, PGconn *newConn, wxMBConv &cnv, bool needColQt)
+pgSet::pgSet(PGresult *newRes, pgConn *newConn, wxMBConv &cnv, bool needColQt)
 : conv(cnv)
 {
     needColQuoting = needColQt;
@@ -59,6 +60,16 @@ OID pgSet::ColTypeOid(int col) const
 }
 
 
+long pgSet::GetInsertedCount() const
+{
+    char *cnt=PQcmdTuples(res);
+    if (!*cnt)
+        return -1;
+    else
+        return atol(cnt);
+}
+
+
 pgTypClass pgSet::ColTypClass(int col) const
 {
     wxString typoid=ExecuteScalar(
@@ -135,9 +146,22 @@ int pgSet::ColNumber(const wxString &colname) const
 }
 
 
+
+char *pgSet::GetCharPtr(const int col) const
+{
+    return PQgetvalue(res, pos -1, col);
+}
+
+
+char *pgSet::GetCharPtr(const wxString &col) const
+{
+    return PQgetvalue(res, pos -1, ColNumber(col));
+}
+
+
 wxString pgSet::GetVal(const int col) const
 {
-    return wxString(PQgetvalue(res, pos -1, col), conv);
+    return wxString(GetCharPtr(col), conv);
 }
 
 
@@ -189,6 +213,8 @@ wxDateTime pgSet::GetDateTime(const int col) const
 {
     wxDateTime dt;
     wxString str=GetVal(col);
+    /* This hasn't just been used. ( Is not infinity ) */
+    if ( !str.IsEmpty() )
     dt.ParseDateTime(str);
     return dt;
 }
@@ -202,11 +228,7 @@ wxDateTime pgSet::GetDateTime(const wxString &col) const
 
 double pgSet::GetDouble(const int col) const
 {
-    char *c=PQgetvalue(res, pos-1, col);
-    if (c)
-        return strtod(c, 0);
-    else
-        return 0;
+    return StrToDouble(GetVal(col));
 }
 
 
@@ -233,7 +255,11 @@ wxULongLong pgSet::GetLongLong(const wxString &col) const
 
 OID pgSet::GetOid(const int col) const
 {
-    return (OID)GetLong(col);
+    char *c=PQgetvalue(res, pos-1, col);
+    if (c)
+        return (OID)strtoul(c, 0, 10);
+    else
+        return 0;
 }
 
 
@@ -245,26 +271,11 @@ OID pgSet::GetOid(const wxString &col) const
 
 wxString pgSet::ExecuteScalar(const wxString& sql) const
 {
-    // Execute the query and get the status.
-    PGresult *qryRes;
-
-    wxLogSql(wxT("Set sub-query: %s"), sql.c_str());
-
-    qryRes = PQexec(conn, sql.mb_str(conv));
-    if (PQresultStatus(qryRes) != PGRES_TUPLES_OK) {
-        return wxEmptyString;
-    }
-
-    // Retrieve the query result and return it.
-    wxString result=wxString(PQgetvalue(qryRes, 0, 0), conv);
-    wxLogInfo(wxT("Query result: %s"), result.c_str());
-
-    // Cleanup & exit
-    PQclear(qryRes);
-    return result;
+    return conn->ExecuteScalar(sql);
 }
 
 
+
 static void pgNoticeProcessor(void *arg, const char *message)
 {
     wxString str(message, wxConvUTF8);
@@ -283,16 +294,18 @@ pgQueryThread::pgQueryThread(pgConn *_conn, const wxString &qry, int _resultToRe
     result=0;
     resultToRetrieve=_resultToRetrieve;
     rc=-1;
+    insertedOid = (OID)-1;
     conn->RegisterNoticeProcessor(pgNoticeProcessor, this);
-    PQsetnonblocking(conn->conn, 1);
+    if (conn->conn)
+        PQsetnonblocking(conn->conn, 1);
 }
 
 
 pgQueryThread::~pgQueryThread()
 {
+    conn->RegisterNoticeProcessor(0, 0);
     if (dataSet)
         delete dataSet;
-    conn->RegisterNoticeProcessor(0, 0);
 }
 
 
@@ -313,20 +326,25 @@ wxString pgQueryThread::GetMessagesAndClear()
 void pgQueryThread::appendMessage(const wxString &str)
 {
     wxCriticalSectionLocker cs(criticalSection);
-    messages += str;
+    messages.Append(str);
 }
 
 
 int pgQueryThread::execute()
 {
+    rowsInserted = -1L;
     wxLongLong startTime=wxGetLocalTimeMillis();
 
     wxLogSql(wxT("Thread Query %s"), query.c_str());
 
-
-    if (!PQsendQuery(conn->conn, query.mb_str(*conn->conv)))
+    if (!conn->conn)
         return(0);
 
+    if (!PQsendQuery(conn->conn, query.mb_str(*conn->conv)))
+    {
+        conn->IsAlive();
+        return(0);
+    }
     int resultsRetrieved=0;
     PGresult *lastResult=0;
     while (true)
@@ -346,7 +364,7 @@ int pgQueryThread::execute()
         if (PQisBusy(conn->conn))
         {
             Yield();
-            wxUsleep(10);
+            wxMilliSleep(10);
             continue;
         }
 
@@ -363,12 +381,17 @@ int pgQueryThread::execute()
         if (resultsRetrieved == resultToRetrieve)
         {
             result=res;
-            appendMessage(wxString::Format(_("Query result with %d rows will be returned.\n"), PQntuples(result)));
+            insertedOid=PQoidValue(res);
+            if (insertedOid && insertedOid != (OID)-1)
+                appendMessage(wxString::Format(_("Query result with %d rows will be returned.\n"), PQntuples(result)));
+            else
+                appendMessage(wxString::Format(_("Query inserted one rows with OID %d.\n"), insertedOid));
             continue;
         }
         if (lastResult)
         {
-            appendMessage(wxString::Format(_("Query result with %d rows discarded.\n"), PQntuples(lastResult)));
+            if (PQntuples(lastResult))
+                appendMessage(wxString::Format(_("Query result with %d rows discarded.\n"), PQntuples(lastResult)));
             PQclear(lastResult);
         }
         lastResult=res;
@@ -379,13 +402,22 @@ int pgQueryThread::execute()
 
     appendMessage(wxT("\n"));
     rc=PQresultStatus(result);
+    insertedOid=PQoidValue(result);
+    if (insertedOid == (OID)-1)
+        insertedOid=0;
 
     if (rc == PGRES_TUPLES_OK)
     {
-        dataSet = new pgSet(result, conn->conn, *conn->conv, conn->needColQuoting);
+        dataSet = new pgSet(result, conn, *conn->conv, conn->needColQuoting);
         dataSet->MoveFirst();
         dataSet->GetVal(0);
     }
+    else if (rc == PGRES_COMMAND_OK)
+    {
+        char *s=PQcmdTuples(result);
+        if (*s)
+            rowsInserted = atol(s);
+    }
     return(1);
 }
 
