@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: ctlSQLBox.h,v 1.10 2004/10/05 20:32:02 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // ctlSQLBox.h - SQL syntax highlighting textbox
@@ -23,10 +24,14 @@
 // Class declarations
 class ctlSQLBox : public wxStyledTextCtrl
 {
+    static wxString sqlKeywords;
+
 public:
     ctlSQLBox(wxWindow *parent, wxWindowID id = -1, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = 0);
+    ctlSQLBox();
     ~ctlSQLBox();
 
+    void Create(wxWindow *parent, wxWindowID id = -1, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize, long style = 0);
     wxFindReplaceData m_findData;
     wxFindReplaceDialog* m_dlgFind;
 
@@ -34,7 +39,8 @@ public:
     void OnFind(wxCommandEvent& event);
     void OnFindDialog(wxFindDialogEvent& event);
     
-    DECLARE_EVENT_TABLE();
+    DECLARE_DYNAMIC_CLASS(ctlSQLBox)
+    DECLARE_EVENT_TABLE()
 };
 
 
