@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: ctlSQLResult.h,v 1.13 2004/06/17 17:47:02 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // ctlSQLResult.h - SQL Query result window
@@ -31,7 +32,9 @@ public:
 
 
     int Execute(const wxString &query, int resultToDisplay=0); // > 0: resultset to display, <=0: last result
-    long NumRows();
+    long NumRows() const;
+    long InsertedCount() const;
+    OID  InsertedOid() const;
 
     int Retrieve(long chunk=-1);
     int RetrieveOne();
@@ -40,6 +43,9 @@ public:
     bool Export();
     bool CanExport() { return rowsRetrieved>0 && colNames.GetCount() > 0; }
 
+    wxString GetItemText(int row, int col=-1);
+    wxString GetExportLine(int row);
+
     int RunStatus();
     wxString GetMessagesAndClear();
     wxString GetErrorMessage();
@@ -48,6 +54,9 @@ public:
     wxArrayString colTypes;
     wxArrayLong colTypClasses;
 
+    wxArrayInt  colSizes;
+    wxArrayString colHeaders;
+
 private:
     pgQueryThread *thread;
     pgConn *conn;
