@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: dlgAggregate.h,v 1.6 2004/07/20 12:38:04 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // dlgAggregate.h - Aggregate property 
@@ -23,6 +24,7 @@ public:
     dlgAggregate(frmMain *frame, pgAggregate *agg, pgSchema *sch);
     int Go(bool modal);
 
+    void CheckChange();
     wxString GetSql();
     pgObject *CreateObject(pgCollection *collection);
     pgObject *GetObject();
@@ -31,8 +33,9 @@ private:
     pgSchema *schema;
     pgAggregate *aggregate;
 
-    void OnChange(wxNotifyEvent &ev);
-    void OnChangeType(wxNotifyEvent &ev);
+    void OnChangeType(wxCommandEvent &ev);
+    void OnChangeTypeBase(wxCommandEvent &ev);
+    void OnChangeTypeState(wxCommandEvent &ev);
     
     wxArrayString procedures;
 
