@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: dlgCast.h,v 1.7 2004/07/20 12:38:04 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // dlgCast.h - Cast property 
@@ -22,13 +23,15 @@ public:
     dlgCast(frmMain *frame, pgCast *ca);
     int Go(bool modal);
 
+    void CheckChange();
     wxString GetSql();
     pgObject *CreateObject(pgCollection *collection);
     pgObject *GetObject();
 
 private:
-    void OnChange(wxNotifyEvent &ev);
-    void OnChangeType(wxNotifyEvent &ev);
+    void OnChangeType(wxCommandEvent &ev);
+    void OnChangeTypeSrc(wxCommandEvent &ev);
+    void OnChangeTypeTrg(wxCommandEvent &ev);
 
     pgCast *cast;
     wxArrayString functions;
