@@ -1,7 +1,7 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // dlgCheck.h - Check property 
@@ -22,18 +22,20 @@ class dlgCheck : public dlgProperty
 {
 public:
     dlgCheck(frmMain *frame, pgCheck *node=0, pgTable *parentNode=0);
+
+    void CheckChange();
     wxString GetSql();
     wxString GetDefinition();
     pgObject *CreateObject(pgCollection *collection);
     pgObject *GetObject();
+    wxString GetHelpPage() const { return wxT("sql-altertable"); }
 
     int Go(bool modal);
 
 private:
     pgCheck *check;
     pgTable *table;
-    void OnChange(wxNotifyEvent &ev);
-    void OnCheckDeferrable(wxNotifyEvent &ev);
+    void OnCheckDeferrable(wxCommandEvent &ev);
 
     DECLARE_EVENT_TABLE();
 };
