@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: dlgColumn.h,v 1.10 2004/07/20 12:38:04 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // dlgColumn.h - Column property 
@@ -22,6 +23,7 @@ class dlgColumn : public dlgTypeProperty
 public:
     dlgColumn(frmMain *frame, pgColumn *column, pgTable *parentNode);
 
+    void CheckChange();
     wxString GetSql();
     pgObject *CreateObject(pgCollection *collection);
     pgObject *GetObject();
@@ -34,10 +36,10 @@ public:
 private:
     pgColumn *column;
     pgTable *table;
-    void OnChange(wxNotifyEvent &ev);
-    void OnSelChangeTyp(wxNotifyEvent &ev);
+    void OnSelChangeTyp(wxCommandEvent &ev);
  
     wxString previousDefinition;
+    wxArrayString sequences;
 
     DECLARE_EVENT_TABLE();
 };
