@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: dlgDatabase.h,v 1.9 2004/07/20 12:38:04 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // dlgDatabase.h - Database property 
@@ -22,23 +23,24 @@ public:
     dlgDatabase(frmMain *frame, pgDatabase *db);
     int Go(bool modal);
 
+    void CheckChange();
     wxString GetSql();
     pgObject *CreateObject(pgCollection *collection);
     pgObject *GetObject();
+    wxString GetHelpPage() const;
 
 private:
     pgDatabase *database;
     wxArrayString varInfo;
 
-    void OnChange(wxNotifyEvent &ev);
-    void OnGroupAdd(wxNotifyEvent &ev);
-    void OnGroupRemove(wxNotifyEvent &ev);
+    void OnGroupAdd(wxCommandEvent &ev);
+    void OnGroupRemove(wxCommandEvent &ev);
 
-    void OnVarAdd(wxNotifyEvent &ev);
-    void OnVarRemove(wxNotifyEvent &ev);
+    void OnVarAdd(wxCommandEvent &ev);
+    void OnVarRemove(wxCommandEvent &ev);
     void OnVarSelChange(wxListEvent &ev);
 
-    void OnVarnameSelChange(wxNotifyEvent &ev);
+    void OnVarnameSelChange(wxCommandEvent &ev);
 
     DECLARE_EVENT_TABLE();
 };
