@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: dlgDomain.h,v 1.9 2004/07/20 12:38:04 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // dlgDomain.h - Domain property 
@@ -23,13 +24,13 @@ public:
     dlgDomain(frmMain *frame, pgDomain *dom, pgSchema *sch);
     int Go(bool modal);
 
+    void CheckChange();
     wxString GetSql();
     pgObject *CreateObject(pgCollection *collection);
     pgObject *GetObject();
 
 private:
-    void OnChange(wxNotifyEvent &ev);
-    void OnSelChangeTyp(wxNotifyEvent &ev);
+    void OnSelChangeTyp(wxCommandEvent &ev);
 
     pgSchema *schema;
     pgDomain *domain;
