@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: dlgForeignKey.h,v 1.8 2004/07/20 12:38:04 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // dlgForeignKey.h - ForeignKey property 
@@ -22,24 +23,29 @@ class dlgForeignKey : public dlgCollistProperty
 {
 public:
     dlgForeignKey(frmMain *frame, pgForeignKey *node, pgTable *parentNode);
-    dlgForeignKey(frmMain *frame, wxListCtrl *colList);
+    dlgForeignKey(frmMain *frame, ctlListView *colList);
+
+    void CheckChange();
     wxString GetSql();
     wxString GetDefinition();
     pgObject *CreateObject(pgCollection *collection);
     pgObject *GetObject();
+    wxString GetHelpPage() const { return wxT("sql-altertable"); }
 
     int Go(bool modal);
 
 private:
     pgForeignKey *foreignKey;
+    wxString savedIndexName, savedFKName;
 
-    void OnChange(wxNotifyEvent &ev);
-    void OnCheckDeferrable(wxNotifyEvent &ev);
-    void OnSelChangeCol(wxNotifyEvent &ev);
-    void OnSelChangeRef(wxNotifyEvent &ev);
-    void OnSelChangeRefCol(wxNotifyEvent &ev);
-    void OnAddRef(wxNotifyEvent &ev);
-    void OnRemoveRef(wxNotifyEvent &ev);
+    wxString DefaultIndexName(const wxString &name);
+
+    void OnCheckDeferrable(wxCommandEvent &ev);
+    void OnSelChangeCol(wxListEvent &ev);
+    void OnSelChangeRef(wxCommandEvent &ev);
+    void OnSelChangeRefCol(wxCommandEvent &ev);
+    void OnAddRef(wxCommandEvent &ev);
+    void OnRemoveRef(wxCommandEvent &ev);
 
     DECLARE_EVENT_TABLE();
 };
