@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: dlgFunction.h,v 1.12 2004/07/20 12:38:04 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // dlgFunction.h - Function property 
@@ -16,7 +17,6 @@
 
 class pgSchema;
 class pgFunction;
-class ctlSQLBox;
 
 class dlgFunction : public dlgSecurityProperty
 {
@@ -24,30 +24,34 @@ public:
     dlgFunction(frmMain *frame, pgFunction *func, pgSchema *sch);
     int Go(bool modal);
 
+    void CheckChange();
     wxString GetSql();
     pgObject *CreateObject(pgCollection *collection);
     pgObject *GetObject();
+    void SetObject(pgObject *obj) { function = (pgFunction*)obj; }
 
 private:
     pgSchema *schema;
     pgFunction *function;
 
-    void OnChange(wxNotifyEvent &ev);
-    void OnSelChangeLanguage(wxNotifyEvent &ev);
-    void OnSelChangeArg(wxNotifyEvent &ev);
-    void OnSelChangeType(wxNotifyEvent &ev);
-    void OnAddArg(wxNotifyEvent &ev);
-    void OnRemoveArg(wxNotifyEvent &ev);
+    void OnChangeArgName(wxCommandEvent &ev);
+    void OnChangeReturn(wxCommandEvent &ev);
+    void OnSelChangeLanguage(wxCommandEvent &ev);
+    void OnSelChangeArg(wxListEvent &ev);
+    void OnSelChangeType(wxCommandEvent &ev);
+    void OnAddArg(wxCommandEvent &ev);
+    void OnChangeArg(wxCommandEvent &ev);
+    void OnRemoveArg(wxCommandEvent &ev);
 
-    wxString GetArgs(bool quoted=false);
+    wxString GetArgs(bool withNames=true, bool quoted=false);
     void ReplaceSizer(wxWindow *w, bool isC, int border);
-    wxSizer *libSizer;
 
-    ctlSQLBox *sqlBox;
     wxArrayString typOids;
     wxArrayString types;
     wxArrayString argOids;
 
+    wxSizer *libcSizer;
+    int typeColNo;
 
     DECLARE_EVENT_TABLE();
 };
