@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: dlgGroup.h,v 1.6 2004/07/20 12:38:04 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // dlgGroup.h - Group property 
@@ -24,13 +25,14 @@ public:
     pgObject *CreateObject(pgCollection *collection);
     pgObject *GetObject();
 
+    void CheckChange();
     int Go(bool modal);
 
 private:
     pgGroup *group;
-    void OnChange(wxNotifyEvent &ev);
-    void OnUserAdd(wxNotifyEvent &ev);
-    void OnUserRemove(wxNotifyEvent &ev);
+
+    void OnUserAdd(wxCommandEvent &ev);
+    void OnUserRemove(wxCommandEvent &ev);
 
     wxArrayString usersIn;
 
