@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: dlgIndex.h,v 1.8 2004/07/20 12:38:04 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // dlgIndex.h - Index property 
@@ -21,18 +22,19 @@ class dlgIndexBase : public dlgCollistProperty
 {
 public:
     dlgIndexBase(frmMain *frame, const wxString &resName, pgIndex *index, pgTable *parentNode);
-    dlgIndexBase(frmMain *frame, const wxString &resName, wxListCtrl *colList);
+    dlgIndexBase(frmMain *frame, const wxString &resName, ctlListView *colList);
+
+    void CheckChange();
     pgObject *GetObject();
     wxString GetColumns();
     int Go(bool modal);
 
 protected:
     pgIndex *index;
-    void OnChange(wxNotifyEvent &ev);
 
 private:
-    void OnAddCol(wxNotifyEvent &ev);
-    void OnRemoveCol(wxNotifyEvent &ev);
+    void OnAddCol(wxCommandEvent &ev);
+    void OnRemoveCol(wxCommandEvent &ev);
 
     DECLARE_EVENT_TABLE();
 };
@@ -44,11 +46,11 @@ public:
     dlgIndex(frmMain *frame, pgIndex *index, pgTable *parentNode);
 
     int Go(bool modal);
+    void CheckChange();
     wxString GetSql();
     pgObject *CreateObject(pgCollection *collection);
 
 private:
-    void OnChange(wxNotifyEvent &ev);
     DECLARE_EVENT_TABLE();
 };
 
