@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: dlgIndexConstraint.h,v 1.9 2004/03/16 20:24:37 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // dlgIndexConstraint.h - IndexConstraint property 
@@ -20,18 +21,19 @@ class dlgIndexConstraint : public dlgIndexBase
 {
 protected:
     dlgIndexConstraint(frmMain *frame, const wxString &resName, pgIndex *index, pgTable *parentNode);
-    dlgIndexConstraint(frmMain *frame, const wxString &resName, wxListCtrl *colList);
+    dlgIndexConstraint(frmMain *frame, const wxString &resName, ctlListView *colList);
 
 public:
-    wxListCtrl *columns;
+    ctlListView *columns;
 
 
     int Go(bool modal);
     wxString GetDefinition();
     wxString GetSql();
+    wxString GetHelpPage() const { return wxT("sql-altertable"); }
 
 private:
-    void OnCheckDeferrable(wxNotifyEvent &ev);
+    void OnCheckDeferrable(wxCommandEvent &ev);
     DECLARE_EVENT_TABLE();
 };
 
@@ -40,7 +42,7 @@ class dlgPrimaryKey : public dlgIndexConstraint
 {
 public:
     dlgPrimaryKey(frmMain *frame, pgIndex *index, pgTable *parentNode);
-    dlgPrimaryKey(frmMain *frame, wxListCtrl *colList);
+    dlgPrimaryKey(frmMain *frame, ctlListView *colList);
 
     pgObject *CreateObject(pgCollection *collection);
 };
@@ -50,7 +52,7 @@ class dlgUnique : public dlgIndexConstraint
 {
 public:
     dlgUnique(frmMain *frame, pgIndex *index, pgTable *parentNode);
-    dlgUnique(frmMain *frame, wxListCtrl *colList);
+    dlgUnique(frmMain *frame, ctlListView *colList);
 
     pgObject *CreateObject(pgCollection *collection);
 };
