@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: dlgOperator.h,v 1.9 2004/10/22 14:25:16 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // dlgOperator.h - Operator property 
@@ -23,16 +24,18 @@ public:
     dlgOperator(frmMain *frame, pgOperator *op, pgSchema *sch);
     int Go(bool modal);
 
+    void CheckChange();
     wxString GetSql();
     pgObject *CreateObject(pgCollection *collection);
     pgObject *GetObject();
 
 private:
-    void OnChange(wxNotifyEvent &ev);
-    void OnChangeType(wxNotifyEvent &ev);
-    void OnChangeJoin(wxNotifyEvent &ev);
+    void CheckChangeType();
+    void OnChangeTypeLeft(wxCommandEvent &ev);
+    void OnChangeTypeRight(wxCommandEvent &ev);
+    void OnChangeJoin(wxCommandEvent &ev);
 
-    void AppendFilledOperator(wxString &sql, wxChar *txt, wxComboBox *cb);
+    void AppendFilledOperator(wxString &sql, wxChar *txt, wxComboBoxFix *cb);
 
     pgSchema *schema;
     pgOperator *oper;
