@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: dlgProperty.h,v 1.41 2004/11/02 16:12:06 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // dlgProperty.h - common property dialog class
@@ -16,80 +17,100 @@
 #include <wx/notebook.h>
 #include "pgObject.h"
 #include "pgConn.h"
-#include <wx/xrc/xmlres.h>
 
 class pgSchema;
 class pgTable;
+class frmMain;
+class ctlSecurityPanel;
 
-#define txtName         CTRL("txtName",         wxTextCtrl)
-#define txtOID          CTRL("txtOID",          wxTextCtrl)
-#define txtComment      CTRL("txtComment",      wxTextCtrl)
-#define lstColumns      CTRL("lstColumns",      wxListCtrl)
-#define cbColumns       CTRL("cbColumns",       wxComboBox)
-#define btnOK           CTRL("btnOK",           wxButton)
-#define btnCancel       CTRL("btnCancel",       wxButton)
-
+#define stComment       CTRL_STATIC("stComment")
+#define lstColumns      CTRL_LISTVIEW("lstColumns")
+#define cbColumns       CTRL_COMBOBOX("cbColumns")
 
 class dlgProperty : public DialogWithHelp
 {
 public:
-    static void CreateObjectDialog(frmMain *frame, wxListCtrl *properties, pgObject *node, int type);
-    static void EditObjectDialog(frmMain *frame, wxListCtrl *properties, wxListCtrl *statistics, ctlSQLBox *sqlbox, pgObject *node);
+    static bool CreateObjectDialog(frmMain *frame, pgObject *node, int type);
+    static bool EditObjectDialog(frmMain *frame, ctlSQLBox *sqlbox, pgObject *node);
 
     wxString GetName();
 
     virtual wxString GetSql() =0;
     virtual pgObject *CreateObject(pgCollection *collection) =0;
     virtual pgObject *GetObject() =0;
+    virtual void SetObject(pgObject *obj) {} // only necessary if apply is implemented
 
     virtual void CreateAdditionalPages();
     virtual wxString GetHelpPage() const;
     void SetConnection(pgConn *conn) { connection=conn; }
+    void SetDatabase(pgDatabase *db);
     virtual int Go(bool modal=false);
+    virtual void CheckChange() =0;
 
 protected:
     dlgProperty(frmMain *frame, const wxString &resName);
+    ~dlgProperty();
 
     void EnableOK(bool enable);
-
-    void CreateListColumns(wxListCtrl *list, const wxString &left, const wxString &right, int leftSize=100);
-    int AppendListItem(wxListCtrl *list, const wxString& str1, const wxString& str2, int icon);
+    void ShowObject();
 
     void CheckValid(bool &enable, const bool condition, const wxString &msg);
     static dlgProperty *CreateDlg(frmMain *frame, pgObject *node, bool asNew, int type=-1);
+    void AppendNameChange(wxString &sql);
+    void AppendOwnerChange(wxString &sql, const wxString &objName=wxEmptyString);
+    void AppendOwnerNew(wxString &sql, const wxString &objname);
     void AppendComment(wxString &sql, const wxString &objType, pgSchema *schema, pgObject *obj);
     void AppendComment(wxString &sql, const wxString &objName, pgObject *obj);
     void AppendQuoted(wxString &sql, const wxString &name);
 
-    void OnPageSelect(wxNotebookEvent& event);
-    void OnOK(wxNotifyEvent &ev);
-    void OnCancel(wxNotifyEvent &ev);
-    void OnClose(wxCloseEvent &ev);
 
+#if __GNUC__ >= 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4)
+    // ANSI spec 11.5 is quite brain dead about pointers of protected members: In order to access 
+    // them using the base class name, they can't be protected.
+    // apparently, only gcc 3.4 knows that; other compilers take protected as protected.
+public:
+
+#endif
+
+    void OnPageSelect(wxNotebookEvent& event);
+    void OnOK(wxCommandEvent &ev);
+    void OnApply(wxCommandEvent &ev);
+    void OnChange(wxCommandEvent &ev);
+    void OnChangeOwner(wxCommandEvent &ev);
+    void OnChangeStc(wxStyledTextEvent& event);
+
+protected:
+    void AddUsers(wxComboBox *cb1, wxComboBox *cb2=0);
+    void FillCombobox(const wxString &query, wxComboBox *cb1, wxComboBox *cb2=0);
+    void PrepareTablespace(wxComboBox *cb, const wxChar *current=0);
 
     pgConn *connection;
+    pgDatabase *database;
 
+    frmMain *mainForm;
     ctlSQLBox *sqlPane;
-    wxListCtrl *properties, *statistics;
-    ctlSQLBox *sqlFormPane;
 
     wxTextValidator numericValidator;
 
-    wxTextCtrl *statusBox;
     wxNotebook *nbNotebook;
+    wxTextCtrl *txtName, *txtOid, *txtComment;
+    ctlComboBox *cbOwner;
+
     int width, height;
     wxTreeItemId item;
     int objectType;
     bool readOnly;
+    bool processing;
 
 private:
     bool tryUpdate(wxTreeItemId collectionItem);
+    bool apply(const wxString &sql);
 
     DECLARE_EVENT_TABLE();
 };
 
 
-#define cbDatatype      CTRL("cbDatatype",      wxComboBox)
+#define cbDatatype      CTRL_COMBOBOX2("cbDatatype")
 
 
 class dlgTypeProperty : public dlgProperty
@@ -106,8 +127,8 @@ public:
 protected:
     dlgTypeProperty(frmMain *frame, const wxString &resName);
     void CheckLenEnable();
-    void FillDatatype(wxComboBox *cb, bool withDomains=true);
-    void FillDatatype(wxComboBox *cb, wxComboBox *cb2, bool withDomains=true);
+    void FillDatatype(ctlComboBox *cb, bool withDomains=true);
+    void FillDatatype(ctlComboBox *cb, ctlComboBox *cb2, bool withDomains=true);
 
     bool isVarLen, isVarPrec;
     long minVarLen, maxVarLen;
@@ -125,9 +146,9 @@ public:
 
 protected:
     dlgCollistProperty(frmMain *frame, const wxString &resName, pgTable *table);
-    dlgCollistProperty(frmMain *frame, const wxString &resName, wxListCtrl *colList);
+    dlgCollistProperty(frmMain *frame, const wxString &resName, ctlListView *colList);
 
-    wxListCtrl *columns;
+    ctlListView *columns;
     pgTable *table;
 };
 
@@ -137,36 +158,44 @@ class dlgSecurityProperty : public dlgProperty
 protected:
     dlgSecurityProperty(frmMain *frame, pgObject *obj, const wxString &resName, const wxString& privilegeList, char *privilegeChar);
     ~dlgSecurityProperty();
-    void AddGroups(wxComboBox *comboBox=0);
-    void AddUsers(wxComboBox *comboBox=0);
+    void AddGroups(ctlComboBox *comboBox=0);
+    void AddUsers(ctlComboBox *comboBox=0);
 
     wxString GetGrant(const wxString &allPattern, const wxString &grantObject);
     void EnableOK(bool enable);
     virtual wxString GetHelpPage() const;
+    virtual int Go(bool modal=false);
 
 private:
-    void OnPrivSelChange(wxListEvent &ev);
-    void OnAddPriv(wxNotifyEvent& ev);
-    void OnDelPriv(wxNotifyEvent& ev);
-    void OnPrivCheck(wxCommandEvent& ev);
-    void OnPrivCheckAll(wxCommandEvent& ev);
-    void OnPrivCheckAllGrant(wxCommandEvent& ev);
-
-    void ExecPrivCheck(int index);
-    bool GrantAllowed() const;
 
+    void OnAddPriv(wxCommandEvent& ev);
+    void OnDelPriv(wxCommandEvent& ev);
     bool securityChanged;
+
+    ctlSecurityPanel *securityPage;
     wxArrayString currentAcl;
-    wxListView *lbPrivileges;
-    wxComboBox *cbGroups;
-    wxStaticText *stGroup;
-    wxButton *btnAddPriv, *btnDelPriv;
-    int privilegeCount;
-    char *privilegeChars;
-    wxCheckBox **privCheckboxes;
-    wxCheckBox *allPrivileges, *allPrivilegesGrant;
+
     DECLARE_EVENT_TABLE();
 };
 
 
+
+class dlgOidProperty : public dlgProperty
+{
+public:
+
+protected:
+    dlgOidProperty(frmMain *frame, const wxString &resName);
+    void OnOK(wxCommandEvent &ev);
+    bool executeSql();
+    virtual wxString GetInsertSql() =0;
+    virtual wxString GetUpdateSql() =0;
+    wxString GetSql();
+
+    DECLARE_EVENT_TABLE();
+
+    OID oid;
+};
+
+
 #endif
