@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: dlgRule.h,v 1.8 2004/07/20 12:38:04 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // dlgRule.h - Rule property 
@@ -24,6 +25,7 @@ public:
     dlgRule(frmMain *frame, pgRule *r, pgTable *tab);
     int Go(bool modal);
 
+    void CheckChange();
     wxString GetSql();
     pgObject *CreateObject(pgCollection *collection);
     pgObject *GetObject();
@@ -33,12 +35,8 @@ private:
     pgRule *rule;
     wxString oldDefinition;
 
-    void OnChange(wxNotifyEvent &ev);
-
     bool didChange();
 
-    ctlSQLBox *sqlBox;
-
     DECLARE_EVENT_TABLE();
 };
 
