@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: dlgSchema.h,v 1.6 2004/07/20 12:38:04 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // dlgSchema.h - Schemaproperty 
@@ -22,13 +23,13 @@ public:
     dlgSchema(frmMain *frame, pgSchema *db);
     int Go(bool modal);
 
+    void CheckChange();
     wxString GetSql();
     pgObject *CreateObject(pgCollection *collection);
     pgObject *GetObject();
 
 private:
     pgSchema *schema;
-    void OnChange(wxNotifyEvent &ev);
 
     DECLARE_EVENT_TABLE();
 };
