@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: dlgSequence.h,v 1.7 2004/07/20 12:38:04 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // dlgSequence.h - Sequence property 
@@ -23,6 +24,7 @@ public:
     dlgSequence(frmMain *frame, pgSequence *seq, pgSchema *sch);
     int Go(bool modal);
 
+    void CheckChange();
     wxString GetSql();
     pgObject *CreateObject(pgCollection *collection);
     pgObject *GetObject();
@@ -31,8 +33,6 @@ private:
     pgSchema *schema;
     pgSequence *sequence;
 
-    void OnChange(wxNotifyEvent &ev);
-
     DECLARE_EVENT_TABLE();
 };
 
