@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: dlgTrigger.h,v 1.8 2004/07/20 12:38:04 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // dlgTrigger.h - Trigger property 
@@ -23,6 +24,7 @@ public:
     dlgTrigger(frmMain *frame, pgTrigger *trg, pgTable *sch);
     int Go(bool modal);
 
+    void CheckChange();
     wxString GetSql();
     pgObject *CreateObject(pgCollection *collection);
     pgObject *GetObject();
@@ -31,7 +33,7 @@ private:
     pgTable *table;
     pgTrigger *trigger;
 
-    void OnChange(wxNotifyEvent &ev);
+    void OnChangeFunc(wxCommandEvent &ev);
 
     DECLARE_EVENT_TABLE();
 };
