@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: dlgType.h,v 1.9 2004/07/20 12:38:05 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // dlgType.h - Typeproperty 
@@ -20,6 +21,8 @@ class dlgType : public dlgTypeProperty
 {
 public:
     dlgType(frmMain *frame, pgType *node, pgSchema *schema);
+
+    void CheckChange();
     wxString GetSql();
     pgObject *CreateObject(pgCollection *collection);
     pgObject *GetObject();
@@ -29,18 +32,17 @@ public:
 private:
     pgSchema *schema;
     pgType *type;
-    void OnChange(wxNotifyEvent &ev);
-    void OnTypeChange(wxNotifyEvent &ev);
-    void OnVarAdd(wxNotifyEvent &ev);
-    void OnVarRemove(wxNotifyEvent &ev);
+    void OnTypeChange(wxCommandEvent &ev);
+    void OnVarAdd(wxCommandEvent &ev);
+    void OnVarRemove(wxCommandEvent &ev);
     void OnVarSelChange(wxListEvent &ev);
-    void OnSelChangeTyp(wxNotifyEvent &ev);
-    void OnChangeMember(wxNotifyEvent &ev);
+    void OnSelChangeTyp(wxCommandEvent &ev);
+    void OnSelChangeTypOrLen(wxCommandEvent &ev);
+    void OnChangeMember(wxCommandEvent &ev);
 
     void showDefinition(bool isComposite);
 
     wxArrayString memberTypes;
-    wxWindowList compositeWindows, externalWindows;
 
     DECLARE_EVENT_TABLE();
 };
