@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: dlgUser.h,v 1.13 2004/07/20 12:38:05 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // dlgUser.h - User property 
@@ -13,6 +14,8 @@
 #define __DLG_USERPROP
 
 #include "dlgProperty.h"
+#include "calbox.h"
+#include "timespin.h"
 
 class pgUser;
 
@@ -20,9 +23,12 @@ class dlgUser : public dlgProperty
 {
 public:
     dlgUser(frmMain *frame, pgUser *node=0);
+
+    void CheckChange();
     wxString GetSql();
     pgObject *CreateObject(pgCollection *collection);
     pgObject *GetObject();
+    wxString GetHelpPage() const;
 
     int Go(bool modal);
 
@@ -30,15 +36,17 @@ private:
     pgUser *user;
     wxArrayString varInfo;
 
-    void OnChange(wxNotifyEvent &ev);
-    void OnGroupAdd(wxNotifyEvent &ev);
-    void OnGroupRemove(wxNotifyEvent &ev);
+    void OnChangeSuperuser(wxCommandEvent &ev);
+    void OnChangeSpin(wxSpinEvent &ev);
+    void OnChangeCal(wxCalendarEvent &ev);
+    void OnGroupAdd(wxCommandEvent &ev);
+    void OnGroupRemove(wxCommandEvent &ev);
 
-    void OnVarAdd(wxNotifyEvent &ev);
-    void OnVarRemove(wxNotifyEvent &ev);
+    void OnVarAdd(wxCommandEvent &ev);
+    void OnVarRemove(wxCommandEvent &ev);
     void OnVarSelChange(wxListEvent &ev);
 
-    void OnVarnameSelChange(wxNotifyEvent &ev);
+    void OnVarnameSelChange(wxCommandEvent &ev);
 
     wxArrayString groupsIn;
 
