@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: dlgView.h,v 1.7 2004/07/20 12:38:05 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // dlgView.h - View property 
@@ -24,6 +25,7 @@ public:
     dlgView(frmMain *frame, pgView *v, pgSchema *sch);
     int Go(bool modal);
 
+    void CheckChange();
     wxString GetSql();
     pgObject *CreateObject(pgCollection *collection);
     pgObject *GetObject();
@@ -33,10 +35,6 @@ private:
     pgView *view;
     wxString oldDefinition;
 
-    void OnChange(wxNotifyEvent &ev);
-
-    ctlSQLBox *sqlBox;
-
     DECLARE_EVENT_TABLE();
 };
 
