@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: frmExport.h,v 1.6 2004/06/11 18:17:52 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // frmExport.h - The export file dialogue
@@ -18,12 +19,17 @@
 #include "pgAdmin3.h"
 
 class ctlSQLResult;
+class pgSet;
+
 // Class declarations
-class frmExport : public wxDialog
+class frmExport : public pgDialog
 {
 public:
-    frmExport(ctlSQLResult *parent);
+    frmExport(wxWindow *parent);
     ~frmExport();
+
+    bool Export(ctlSQLResult *data) { return Export(data, 0); }
+    bool Export(pgSet *set) { return Export(0, set); }
     
 private:
     void OnChange(wxCommandEvent &ev);
@@ -31,8 +37,9 @@ private:
     void OnOK(wxCommandEvent &ev);
     void OnCancel(wxCommandEvent &ev);
     void OnBrowseFile(wxCommandEvent &ev);
+    bool Export(ctlSQLResult *data, pgSet *set);
 
-    ctlSQLResult *data;
+    wxWindow *parent;
 
     DECLARE_EVENT_TABLE()
 };
