@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: frmHelp.h,v 1.7 2004/06/11 18:17:52 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // frmHelp.h - Help Screen
@@ -17,7 +18,7 @@
 
 class ctlHelpWindow;
 class frmMain;
-class frmHelp : public wxFrame
+class frmHelp : public pgFrame
 {
 public:
     frmHelp(wxWindow *fr);
