@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: frmMaintenance.h,v 1.5 2004/06/11 18:17:52 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // frmMaintenance.h - Maintenance options selection dialogue
@@ -18,25 +19,20 @@
 #include "pgAdmin3.h"
 #include "frmMain.h"
 
-
 // Class declarations
-class frmMaintenance : public DialogWithHelp
+class frmMaintenance : public ExecutionDialog
 {
 public:
     frmMaintenance(frmMain *form, pgObject *_object);
     ~frmMaintenance();
+    wxString GetSql();
 
     void Go();
     
 private:
     wxString GetHelpPage() const;
     void OnAction(wxCommandEvent& ev);
-    void OnOK(wxCommandEvent& ev);
-    void OnCancel(wxCommandEvent& ev);
-    void OnClose(wxCloseEvent& event);
-    void Abort();
-    pgObject *object;
-    pgQueryThread *thread;
+
     DECLARE_EVENT_TABLE()
 };
 
