@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: frmOptions.h,v 1.9 2004/06/11 18:17:52 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // frmOptions.h - The main options dialogue
@@ -20,7 +21,7 @@
 class frmMain;
 
 // Class declarations
-class frmOptions : public wxDialog
+class frmOptions : public pgDialog
 {
 public:
     frmOptions(frmMain *parent);
@@ -28,9 +29,10 @@ public:
     
 private:
     frmMain *mainForm;
-    wxFont currentFont;
+    wxFont currentFont, currentSqlFont;
 
     void OnBrowseLogFile(wxCommandEvent &ev);
+    void OnSqlFontSelect(wxCommandEvent &ev);
     void OnFontSelect(wxCommandEvent &ev);
     void OnOK(wxCommandEvent &ev);
     void OnCancel(wxCommandEvent &ev);
