@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: frmPassword.h,v 1.9 2004/06/11 18:17:52 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // frmPassword.h - Change password
@@ -19,7 +20,7 @@
 #include "pgServer.h"
 
 // Class declarations
-class frmPassword : public wxDialog
+class frmPassword : public pgDialog
 {
 public:
     frmPassword(wxFrame *parent);
