@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: frmQuery.h,v 1.27 2004/10/10 17:26:41 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // frmQuery.h - The SQL Query form
@@ -19,10 +20,10 @@
 #include "ctlSQLResult.h"
 
 
-class frmQuery : public wxFrame
+class frmQuery : public pgFrame
 {
 public:
-    frmQuery(frmMain *form, const wxString& _title, pgConn *conn, const wxPoint& pos, const wxSize& size, const wxString& qry);
+    frmQuery(frmMain *form, const wxString& _title, pgConn *conn, const wxString& qry);
     ~frmQuery();
     void Go();
 
@@ -33,24 +34,22 @@ private:
     wxNotebook *output;
     ctlSQLResult *sqlResult;
     wxTextCtrl *msgResult, *msgHistory;
-    wxStatusBar *statusBar;
-    wxToolBar *toolBar;
     pgConn *conn;
     wxLongLong elapsedQuery, elapsedRetrieve;
 
+    void OnChangeStc(wxStyledTextEvent& event);
     void OnClose(wxCloseEvent& event);
+    void OnSetFocus(wxFocusEvent& event);
     void OnContents(wxCommandEvent& event);
     void OnHelp(wxCommandEvent& event);
     void OnCancel(wxCommandEvent& event);
     void OnExecute(wxCommandEvent& event);
+    void OnExecFile(wxCommandEvent& event);
     void OnExplain(wxCommandEvent& event);
     void OnOpen(wxCommandEvent& event);
     void OnSave(wxCommandEvent& event);
     void OnSaveAs(wxCommandEvent& event);
     void OnExport(wxCommandEvent& event);
-    void OnChange(wxNotifyEvent& event);
-    void OnExit(wxCommandEvent& event);
-    void OnRecent(wxCommandEvent& event);
     void OnCut(wxCommandEvent& event);
     void OnCopy(wxCommandEvent& event);
     void OnPaste(wxCommandEvent& event);
@@ -60,22 +59,22 @@ private:
     void OnRedo(wxCommandEvent& event);
     void OnSaveHistory(wxCommandEvent& event);
     void OnClearHistory(wxCommandEvent& event);
-    void OnKeyDown(wxKeyEvent& event);
     void OnActivate(wxActivateEvent& event);
+    void OnFocus(wxFocusEvent& event);
 
-    void updateRecentFiles();
-    void openLastFile();
-    void updateMenu();
-    void execQuery(const wxString &query, int resultToRetrieve=0, bool singleResult=false, const int queryOffset=0);
+    bool CheckChanged(bool canVeto);
+    void OpenLastFile();
+    void updateMenu(wxObject *obj=0);
+    void execQuery(const wxString &query, int resultToRetrieve=0, bool singleResult=false, const int queryOffset=0, bool toFile=false);
     void setTools(const bool running);
     void showMessage(const wxString& msg, const wxString &msgShort=wxT(""));
     void setExtendedTitle();
-    wxMenuBar *menuBar;
-    wxMenu *fileMenu, *recentFileMenu, *editMenu, *queryMenu;
+    wxWindow *currentControl();
+    wxMenu *queryMenu;
     wxString title;
-    wxString lastFilename, lastDir, lastPath;
+    wxString lastFilename, lastDir;
 
-    bool changed, aborted;
+    bool aborted;
     bool lastFileFormat;
 
     DECLARE_EVENT_TABLE()
