@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: frmSplash.h,v 1.7 2004/06/11 18:17:52 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // frmSplash.h - Splash Screen
@@ -19,7 +20,7 @@
 #include "pgAdmin3.h"
 
 // Class declarations
-class frmSplash : public wxFrame
+class frmSplash : public pgFrame
 {
 public:
     frmSplash(wxFrame *parent);
