@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: menu.h,v 1.20 2004/10/10 17:26:41 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // misc.h - Miscellaneous Utilties
@@ -38,14 +39,23 @@ enum
     MNU_NEWOBJECT,
     MNU_DROP,
     MNU_PROPERTIES,
-    MNU_QUERYBUILDER,
+    MNU_ADDTABLEVIEW,
+	MNU_ADDCOLUMN,
+	MNU_CLOSE,
+	MNU_MIN,
     MNU_SQL,
     MNU_VIEWDATA,
+    MNU_VIEWFILTEREDDATA,
     MNU_MAINTENANCE,
+    MNU_INDEXCHECK,
+    MNU_GRANTWIZARD,
+    MNU_BACKUP,
+    MNU_RESTORE,
     MNU_CREATE,
     MNU_RECORD,
     MNU_STOP,
     MNU_STATUS,
+    MNU_COUNT,
     MNU_APPEND,
     MNU_DELETE,
     MNU_OPEN,
@@ -61,13 +71,19 @@ enum
     MNU_REDO,
     MNU_CANCEL,
     MNU_EXECUTE,
+    MNU_EXECFILE,
     MNU_EXPLAIN,
     MNU_EXPLAINOPTIONS,
     MNU_VERBOSE,
     MNU_ANALYZE,
-    MNU_RELOAD,
+    MNU_STARTSERVICE,
+    MNU_STOPSERVICE,
     MNU_CLEARHISTORY,
     MNU_SAVEHISTORY,
+    MNU_CHECKALIVE,
+    MNU_CONFIG,
+    MNU_HBACONFIG,
+    MNU_HINT,
     MNU_RECENT, // leave space for recent file entries
     MNU_NEW=MNU_RECENT+15
 };
