@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: pgAdmin3.h,v 1.32 2004/09/27 10:40:52 dpage Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // pgAdmin3.h - The main application header
@@ -11,10 +12,15 @@
 #ifndef PGADMIN3_H
 #define PGADMIN3_H
 
-
 // wxWindows headers
 #include <wx/wx.h>
 #include <wx/listctrl.h>
+#include <wx/xrc/xmlres.h>
+
+#include "misc.h"
+#include "ctlListView.h"
+#include "ctlComboBox.h"
+#include "dlgClasses.h"
 
 // App headers
 #include "sysSettings.h"
