@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: pgCollection.h,v 1.15 2004/06/19 19:18:01 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // pgCollection.h - Simple object for use with 'collection' nodes
@@ -38,8 +39,9 @@ public:
     pgSchema *GetSchema() const { return schema; }
 
     int GetIcon();
-    void ShowTreeDetail(wxTreeCtrl *browser, frmMain *form=0, wxListCtrl *properties=0, wxListCtrl *statistics=0, ctlSQLBox *sqlPane=0);
-    void ShowList(const wxString& name, wxTreeCtrl *browser, wxListCtrl *properties);
+    void ShowTreeDetail(wxTreeCtrl *browser, frmMain *form=0, ctlListView *properties=0, ctlSQLBox *sqlPane=0);
+    void ShowStatistics(frmMain *form, ctlListView *statistics);
+    void ShowList(const wxString& name, wxTreeCtrl *browser, ctlListView *properties);
     void UpdateChildCount(wxTreeCtrl *browser, int substract=0);
     pgObject *FindChild(wxTreeCtrl *browser, const int index);
     bool CanCreate();
@@ -55,7 +57,7 @@ class pgServers: public pgCollection
 {
 public:
     pgServers() : pgCollection(PG_SERVERS, (pgServer*)0) {}
-    void ShowTreeDetail(wxTreeCtrl *browser, frmMain *form=0, wxListCtrl *properties=0, wxListCtrl *statistics=0, ctlSQLBox *sqlPane=0) {};
+    void ShowTreeDetail(wxTreeCtrl *browser, frmMain *form=0, ctlListView *properties=0, ctlSQLBox *sqlPane=0) {};
 };
 
 
