@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: pgColumn.h,v 1.27 2004/06/19 19:18:01 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // pgColumn.h PostgreSQL Column
@@ -29,11 +30,16 @@ public:
     ~pgColumn();
 
     int GetIcon() { return PGICON_COLUMN; }
-    void ShowTreeDetail(wxTreeCtrl *browser, frmMain *form=0, wxListCtrl *properties=0, wxListCtrl *statistics=0, ctlSQLBox *sqlPane=0);
+    void ShowTreeDetail(wxTreeCtrl *browser, frmMain *form=0, ctlListView *properties=0, ctlSQLBox *sqlPane=0);
+    void ShowStatistics(frmMain *form, ctlListView *statistics);
+    void ShowDependsOn(frmMain *form, ctlListView *dependsOn, const wxString &where=wxEmptyString);
+    void ShowReferencedBy(frmMain *form, ctlListView *referencedBy, const wxString &where=wxEmptyString);
     static pgObject *ReadObjects(pgCollection *collection, wxTreeCtrl *browser, const wxString &restriction=wxT(""));
 
     wxString GetDefinition();
 
+    bool IsReferenced();
+
     wxString GetRawTypename() const { return rawTypename; }
     void iSetRawTypename(const wxString& s) { rawTypename=s; }
     wxString GetVarTypename() const { return varTypename; }
@@ -70,10 +76,17 @@ public:
     void iSetInheritedCount(const long l) { inheritedCount=l; }
     OID  GetAttTypId() const { return attTypId; }
     void iSetAttTypId(const OID o) { attTypId =o; }
+    long GetAttstattarget() const { return attstattarget; }
+    void iSetAttstattarget(const long l) { attstattarget=l; }
+    wxString GetSerialSequence() const { return serialSequence; }
+    void iSetSerialSequence(const wxString &s) { serialSequence=s; }
+    wxString GetSerialSchema() const { return serialSchema; }
+    void iSetSerialSchema(const wxString &s) { serialSchema=s; }
 
     bool DropObject(wxFrame *frame, wxTreeCtrl *browser);
     bool GetSystemObject() const { return colNumber < 0; }
     wxString GetSql(wxTreeCtrl *browser);
+	wxString GetCommentSql();
     wxString GetHelpPage(bool forCreate) const { return wxT("sql-createtable"); }
 
     virtual bool CanDrop() { return inheritedCount == 0 && pgSchemaObject::CanDrop(); }
@@ -81,10 +94,12 @@ public:
 
 private:
     wxString varTypename, quotedTypename, defaultVal, tableName, quotedFullTable, storage, rawTypename;
-    long colNumber, length, precision, statistics;
+    wxString serialSequence, serialSchema;
+    long colNumber, length, precision, statistics, attstattarget;
     long typlen, typmod, inheritedCount;
     bool isPK, isFK, notNull, isArray;
     OID attTypId;
+    int isReferenced;
 };
 
 #endif
