@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: pgConn.h,v 1.27 2004/10/13 19:09:13 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // pgConn.h - PostgreSQL Connection class
@@ -29,7 +30,8 @@ enum
     PGCONN_BAD = CONNECTION_BAD,
     PGCONN_REFUSED,
     PGCONN_DNSERR,
-    PGCONN_ABORTED
+    PGCONN_ABORTED,     // connect user aborted
+    PGCONN_BROKEN       // tcp/pipe broken
 };
 
 enum 
@@ -48,9 +50,12 @@ enum
 class pgConn
 {
 public:
-    pgConn(const wxString& server = wxT(""), const wxString& database = wxT(""), const wxString& username = wxT(""), const wxString& password = wxT(""), int port = 5432, int sslmode=0);
+    pgConn(const wxString& server = wxT(""), const wxString& database = wxT(""), const wxString& username = wxT(""), const wxString& password = wxT(""), int port = 5432, int sslmode=0, OID oid=0);
     ~pgConn();
+
+    void Close();
     bool HasPrivilege(const wxString &objTyp, const wxString &objName, const wxString &priv);
+    bool HasFeature(int feature=0);
     bool ExecuteVoid(const wxString& sql);
     wxString ExecuteScalar(const wxString& sql);
     pgSet *ExecuteSet(const wxString& sql);
@@ -62,11 +67,18 @@ public:
     wxString GetOptions() const { return wxString(PQoptions(conn), *conv); }
     int GetBackendPID() const { return PQbackendPID(conn); }
     int GetStatus() const;
-    wxString GetLastError() const { return wxString(PQerrorMessage(conn), *conv); }
+    int GetLastResultStatus() const { return lastResultStatus; }
+    bool IsAlive();
+    wxString GetLastError() const;
     wxString GetVersionString();
     OID GetLastSystemOID() const { return lastSystemOID; }
+    OID GetDbOid() const { return dbOid; }
     bool BackendMinimumVersion(int major, int minor);
     void RegisterNoticeProcessor(PQnoticeProcessor proc, void *arg);
+    wxString SystemNamespaceRestriction(const wxString &nsp);
+
+    void LogError();
+
 #ifdef SSL
     bool IsSSLconnected();
 #endif
@@ -75,11 +87,16 @@ public:
 
 private:
     PGconn *conn;
+    int lastResultStatus;
+    bool features[32];
+
+    int connStatus;
     int minorVersion, majorVersion;
     wxMBConv *conv;
-    bool resolvedIP, needColQuoting;
+    bool needColQuoting;
     wxString dbHost;
     OID lastSystemOID;
+    OID dbOid;
 
     void *noticeArg;
     PQnoticeProcessor noticeProc;
