@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: pgFunction.h,v 1.27 2004/09/15 20:31:47 dpage Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // pgFunction.h PostgreSQL Function
@@ -30,21 +31,27 @@ public:
 
 
     int GetIcon() { return PGICON_FUNCTION; }
-    void ShowTreeDetail(wxTreeCtrl *browser, frmMain *form=0, wxListCtrl *properties=0, wxListCtrl *statistics=0, ctlSQLBox *sqlPane=0);
+    void ShowTreeDetail(wxTreeCtrl *browser, frmMain *form=0, ctlListView *properties=0, ctlSQLBox *sqlPane=0);
     static pgObject *ReadObjects(pgCollection *collection, wxTreeCtrl *browser);
     static pgFunction *pgFunction::AppendFunctions(pgObject *obj, pgSchema *schema, wxTreeCtrl *browser, const wxString &restriction);
 
-    bool ReloadLibrary();
-
     wxString GetFullName() const {return GetName()+wxT("(")+GetArgTypes()+wxT(")"); }
+    wxString GetArgTypeNames() const { return argTypeNames; }
+    void iSetArgTypeNames(const wxString& s) { argTypeNames=s; }
+    wxString GetQuotedArgTypeNames() const { return quotedArgTypeNames; }
+    void iSetQuotedArgTypeNames(const wxString& s) { quotedArgTypeNames=s; }
     wxString GetArgTypes() const { return argTypes; }
     void iSetArgTypes(const wxString& s) { argTypes=s; }
     wxString GetQuotedArgTypes() const { return quotedArgTypes; }
     void iSetQuotedArgTypes(const wxString& s) { quotedArgTypes=s; }
     wxString GetArgTypeOids() const { return argTypeOids; }
+    wxArrayString &GetArgNames() { return argNames; }
+    void iAddArgName(const wxString &s) { argNames.Add(s); }
     void iSetArgTypeOids(const wxString& s) { argTypeOids = s; }
     wxString GetReturnType() const { return returnType; }
+	wxString GetQuotedReturnType() const { return quotedReturnType; }
     void iSetReturnType(const wxString& s) { returnType = s; }
+	void iSetQuotedReturnType(const wxString& s) { quotedReturnType = s; }
     wxString GetLanguage() const { return language; }
     void iSetLanguage(const wxString& s) { language = s; }
     wxString GetVolatility() const { return volatility; }
@@ -62,7 +69,7 @@ public:
     bool GetIsStrict() const { return isStrict; }
     void iSetIsStrict(bool b) { isStrict = b; }
 
-    bool CanReload();
+    bool CanRestore() { return true; }
     bool DropObject(wxFrame *frame, wxTreeCtrl *browser);
     wxString GetSql(wxTreeCtrl *browser);
     wxString GetHelpPage(bool forCreate) const { return wxT("sql-createfunction"); }
@@ -72,7 +79,10 @@ protected:
     pgFunction(pgSchema *newSchema, int newType, const wxString& newName = wxT(""));
 
 private:
-    wxString argTypeOids, argTypes, returnType, language, volatility, source, bin, quotedArgTypes;
+    wxString argTypeOids, argTypes, quotedArgTypes,
+        argTypeNames, quotedArgTypeNames,
+        returnType, quotedReturnType, language, volatility, source, bin;
+    wxArrayString argNames;
     bool returnAsSet, secureDefiner, isStrict;
     long argCount;
 };
