@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: pgServer.h,v 1.41 2004/10/13 19:09:13 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // pgServer.h - PostgreSQL Server
@@ -21,31 +22,47 @@
 // Class declarations
 
 
+class frmMain;
+
 class pgServer : public pgObject
 {
 public:
-    pgServer(const wxString& newServer = wxT(""), const wxString& newDatabase = wxT(""), const wxString& newUsername = wxT(""), int newPort = 5432, bool trusted=false, int sslMode=0);
+    pgServer(const wxString& newServer = wxT(""), const wxString& newDescription = wxT(""), const wxString& newDatabase = wxT(""), const wxString& newUsername = wxT(""), int newPort = 5432, bool trusted=false, int sslMode=0);
     ~pgServer();
     int GetType() const { return PG_SERVER; }
     wxString GetTypeName() const { return wxT("Server"); }
-    int Connect(wxFrame *form, bool lockFields = FALSE);
+    int Connect(frmMain *form, bool askPassword=true, const wxString &pwd=wxEmptyString);
     bool Disconnect();
 
+    bool StartService();
+    bool StopService();
+    bool GetServerRunning();
+    bool GetServerControllable();
+
+    pgServer *GetServer() const;
+
     wxString GetIdentifier() const;
     wxString GetVersionString();
     wxString GetVersionNumber();
     OID GetLastSystemOID();
-    wxString GetDatabase() const { return database; }
+    wxString GetDatabaseName() const { return database; }
     wxString GetUsername() const { return username; }
     wxString GetPassword() const { return password; }
     bool GetTrusted() const { return trusted; }
+    bool GetNeedPwd() const { return !trusted; }
     wxString GetLastError() const;
 
+    bool GetDiscovered() const { return discovered; }
+    void iSetDiscovered(const bool b) { discovered=b; }
+    wxString GetServiceID() const { return serviceId; }
+    void iSetServiceID(const wxString& s);
+
     bool GetCreatePrivilege() const { return createPrivilege; }
     void iSetCreatePrivilege(const bool b) { createPrivilege=b; }
     bool GetSuperUser() const { return superUser; }
     void iSetSuperUser(const bool b) { superUser=b; }
 
+    pgConn *CreateConn(wxString dbName=wxEmptyString, OID oid=0);
 
     wxString GetLastDatabase() const { return lastDatabase; }
     void iSetLastDatabase(const wxString& s) { lastDatabase=s; }
@@ -59,21 +76,28 @@ public:
     int GetSSL() const { return ssl; }
     bool GetConnected() const { return connected; }
     void iSetDatabase(const wxString& newVal) { database = newVal; }
+    void iSetPort(int newval) { port=newval; }
+    void iSetSSL(int newval) { ssl=newval; }
     void iSetUsername(const wxString& newVal) { username = newVal; }
     void iSetPassword(const wxString& newVal) { password = newVal; }
     void iSetTrusted(const bool b) { trusted=b; }
+    void iSetNeedPwd(const bool b) { trusted=!b; }
     bool SetPassword(const wxString& newVal);
+    wxDateTime GetUpSince() { return upSince; }
+    void iSetUpSince(const wxDateTime &d) { upSince = d; }
 
-    void iSetPort(int newVal) { port = newVal; }
     bool HasPrivilege(const wxString &objTyp, const wxString &objName, const wxString &priv) { return conn->HasPrivilege(objTyp, objName, priv); }
     bool ExecuteVoid(const wxString& sql) { return conn->ExecuteVoid(sql); }
     wxString ExecuteScalar(const wxString& sql) { return conn->ExecuteScalar(sql); }
     pgSet *ExecuteSet(const wxString& sql) { return conn->ExecuteSet(sql); }
-    void ShowTreeDetail(wxTreeCtrl *browser, frmMain *form=0, wxListCtrl *properties=0, wxListCtrl *statistics=0, ctlSQLBox *sqlPane=0);
+    void ShowTreeDetail(wxTreeCtrl *browser, frmMain *form=0, ctlListView *properties=0, ctlSQLBox *sqlPane=0);
+    void ShowStatistics(frmMain *form, ctlListView *statistics);
     wxString GetHelpPage(bool forCreate) const { return wxT("managing-databases"); }
     int GetIcon() { return PGICON_SERVER; }
+    wxMenu *GetNewMenu();
 
     bool DropObject(wxFrame *frame, wxTreeCtrl *browser) { return true; }
+    bool CanEdit() { return true; }
     bool CanDrop() { return true; }
 
     pgConn *connection() { return conn; }
@@ -82,12 +106,18 @@ private:
     pgConn *conn;
     bool connected;
     wxString database, username, password, ver, error;
-    wxString lastDatabase, lastSchema, description;
+    wxString lastDatabase, lastSchema, description, serviceId;
+    wxDateTime upSince;
     int port, ssl;
-    bool trusted, createPrivilege, superUser;
+    bool trusted, discovered, createPrivilege, superUser;
     OID lastSystemOID;
+    OID dbOid;
     wxString versionNum;
-    wxFrame *parentWin;
+
+#ifdef WIN32
+    SC_HANDLE scmHandle;
+    SC_HANDLE serviceHandle;
+#endif
 };
 
 #endif
