@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: pgSet.h,v 1.29 2004/06/11 18:17:52 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // pgSet.h - PostgreSQL ResultSet class
@@ -13,7 +14,6 @@
 
 // wxWindows headers
 #include <wx/wx.h>
-#include <wx/datetime.h>
 
 // PostgreSQL headers
 #include <libpq-fe.h>
@@ -39,7 +39,7 @@ class pgConn;
 class pgSet
 {
 public:
-    pgSet(PGresult *newRes, PGconn *newConn, wxMBConv &cnv, bool needColQt);
+    pgSet(PGresult *newRes, pgConn *newConn, wxMBConv &cnv, bool needColQt);
     ~pgSet();
     long NumRows() const { return nRows; }
     long NumCols() const { return PQnfields(res); }
@@ -58,6 +58,7 @@ public:
     pgTypClass ColTypClass(int col) const;
 
     OID GetInsertedOid() const { return PQoidValue(res); }
+    long GetInsertedCount() const;
     int ColSize(int col) const { return PQfsize(res, col); }
     bool IsNull(int col) const { return (PQgetisnull(res, pos-1, col) != 0); }
     int ColScale(int col) const;
@@ -79,9 +80,14 @@ public:
     OID GetOid(const int col) const;
     OID GetOid(const wxString &col) const;
 
+    char *GetCharPtr(const int col) const;
+    char *GetCharPtr(const wxString &col) const;
+
+    wxMBConv &GetConversion() const { return conv; }
+
 
 private:
-    PGconn *conn;
+    pgConn *conn;
     PGresult *res;
     long pos, nRows;
     wxString ExecuteScalar(const wxString& sql) const;
@@ -101,6 +107,8 @@ public:
     bool DataValid() const { return dataSet != NULL; }
     pgSet *DataSet() { return dataSet; }
     int ReturnCode() const { return rc; }
+    long RowsInserted() const { return rowsInserted; }
+    OID InsertedOid() const { return insertedOid; }
     wxString GetMessagesAndClear();
     bool IsRunning() const;
     void appendMessage(const wxString &str);
@@ -108,6 +116,8 @@ public:
 private:
     int rc;
     int resultToRetrieve;
+    long rowsInserted;
+    OID insertedOid;
 
     wxString query;
     pgConn *conn;
