@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: pgTable.h,v 1.27 2004/06/20 21:43:31 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // pgTable.h PostgreSQL Table
@@ -30,9 +31,11 @@ public:
 
     int GetIcon() { return PGICON_TABLE; }
     wxString GetAllConstraints(wxTreeCtrl *browser, wxTreeItemId collectionId, int type);
-    void ShowTreeDetail(wxTreeCtrl *browser, frmMain *form=0, wxListCtrl *properties=0, wxListCtrl *statistics=0, ctlSQLBox *sqlPane=0);
+    void ShowTreeDetail(wxTreeCtrl *browser, frmMain *form=0, ctlListView *properties=0, ctlSQLBox *sqlPane=0);
+    void ShowStatistics(frmMain *form, ctlListView *statistics);
+
     static pgObject *ReadObjects(pgCollection *collection, wxTreeCtrl *browser, const wxString &restriction=wxT(""));
-    static void ShowStatistics(pgCollection *collection, wxListCtrl *statistics);
+    static void ShowStatistics(pgCollection *collection, ctlListView *statistics);
 
     bool GetHasOids() const { return hasOids; }
     void iSetHasOids(bool b) { hasOids=b; }
@@ -44,19 +47,24 @@ public:
     void iSetPrimaryKeyColNumbers(const wxString& s) {primaryKeyColNumbers = s; }
     wxString GetPrimaryKeyName() const { return primaryKeyName; }
     void iSetPrimaryKeyName(const wxString& s) {primaryKeyName = s; }
-    long GetEstimatedRows() const { return estimatedRows; }
-    void iSetEstimatedRows(const long l) { estimatedRows=l; }
-    long GetRows() const { return rows; }
+    double GetEstimatedRows() const { return estimatedRows; }
+    void iSetEstimatedRows(const double d) { estimatedRows=d; }
+    wxString GetTablespace() const { return tablespace; };
+    void iSetTablespace(const wxString& newVal) { tablespace = newVal; }
+    wxULongLong GetRows() const { return rows; }
     long GetInheritedTableCount() { if (inheritedTableCount < 0) UpdateInheritance(); return inheritedTableCount; }
     wxString GetInheritedTables() { GetInheritedTableCount(); return inheritedTables; }
     wxString GetQuotedInheritedTables() { GetInheritedTableCount(); return quotedInheritedTables; }
     wxArrayString GetQuotedInheritedTablesList() { GetInheritedTableCount(); return quotedInheritedTablesList; }
+    wxString GetCoveringIndex(wxTreeCtrl *browser, const wxString &collist);
     bool GetHasSubclass() const { return hasSubclass; }
     void iSetHasSubclass(bool b) { hasSubclass = b; }
     void UpdateRows();
     bool DropObject(wxFrame *frame, wxTreeCtrl *browser);
     bool CanView() { return true; }
     bool CanMaintenance() { return true; }
+    bool CanBackup() { return true; }
+    bool CanRestore() { return true; }
     bool WantDummyChild() { return true; }
 
     wxMenu *GetNewMenu();
@@ -67,11 +75,12 @@ public:
 private:
     void UpdateInheritance();
 
-    long rows, estimatedRows;
-    bool hasOids, hasSubclass;
+    wxULongLong rows;
+    double estimatedRows;
+    bool hasOids, hasSubclass, rowsCounted;
     long inheritedTableCount;
     wxString quotedInheritedTables, inheritedTables, primaryKey, quotedPrimaryKey,
-        primaryKeyName, primaryKeyColNumbers;
+        primaryKeyName, primaryKeyColNumbers, tablespace;
     wxArrayString quotedInheritedTablesList;
 };
 
