@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: pgUser.h,v 1.21 2004/06/22 16:14:17 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // pgUser.h - PostgreSQL User
@@ -32,10 +33,8 @@ public:
     bool GetSystemObject() const { return userId < 100; }
     long GetUserId() const { return userId; }
     void iSetUserId(const long l) { userId=l; }
-    wxString GetConfigList() const { return configList; }
-    void iSetConfigList(const wxString& s) { configList=s; }
-    wxString GetAccountExpires() const { return accountExpires; }
-    void iSetAccountExpires(const wxString& s) { accountExpires=s; }
+    wxDateTime GetAccountExpires() const { return accountExpires; }
+    void iSetAccountExpires(const wxDateTime & dt) { accountExpires=dt; }
     wxString GetPassword() const { return password; }
     void iSetPassword(const wxString& s) { password=s; }
     bool GetCreateDatabase() const { return createDatabase; }
@@ -45,12 +44,14 @@ public:
     bool GetUpdateCatalog() const { return updateCatalog; }
     void iSetUpdateCatalog(const bool b) { updateCatalog=b; }
     wxArrayString& GetGroupsIn() { return groupsIn; }
+    wxArrayString& GetConfigList() { return configList; }
 
 
     // Tree object creation
     int GetIcon() { return PGICON_USER; }
-    void ShowTreeDetail(wxTreeCtrl *browser, frmMain *form=0, wxListCtrl *properties=0, wxListCtrl *statistics=0, ctlSQLBox *sqlPane=0);
+    void ShowTreeDetail(wxTreeCtrl *browser, frmMain *form=0, ctlListView *properties=0, ctlSQLBox *sqlPane=0);
     static pgObject *ReadObjects(pgCollection *collection, wxTreeCtrl *browser, const wxString &restriction=wxT(""));
+    void ShowReferencedBy(frmMain *form, ctlListView *referencedBy, const wxString &where);
     
     // virtual methods
     wxString GetSql(wxTreeCtrl *browser);
@@ -58,9 +59,11 @@ public:
     bool DropObject(wxFrame *frame, wxTreeCtrl *browser);
 
 private:
-    wxString accountExpires, password, configList;
+    wxString password;
+    wxDateTime accountExpires;
     bool createDatabase, superuser, updateCatalog;
     wxArrayString groupsIn;
+    wxArrayString configList;
     long userId;
 };
 
