@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: wxgridsel.h,v 1.6 2004/01/02 23:15:00 dpage Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // wxgridsel.h - replacement for wx/generic/gridsel.h
@@ -10,6 +11,10 @@
 //
 //////////////////////////////////////////////////////////////////////////
 
+#ifndef __WXGRIDSEL_H
+#define __WXGRIDSEL_H
+
+
 class wxGridSelection
 {
     wxGridCellCoordsArray dummy1[3];
@@ -18,3 +23,4 @@ class wxGridSelection
     wxGrid::wxGridSelectionModes dummy4;
 };
 
+#endif
