@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: pgCollection.cpp,v 1.35 2004/07/25 20:33:51 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // pgCollection.cpp - Simple object for use with 'collection' nodes
@@ -19,6 +20,7 @@
 #include "pgGroup.h"
 #include "pgUser.h"
 #include "pgLanguage.h"
+#include "pgTablespace.h"
 #include "pgAggregate.h"
 #include "pgCast.h"
 #include "pgConversion.h"
@@ -35,6 +37,8 @@
 #include "pgRule.h"
 #include "pgTrigger.h"
 
+
+
 pgCollection::pgCollection(int newType, pgServer *sv)
 : pgObject(newType, typesList[newType].typName)
 { 
@@ -71,7 +75,7 @@ pgCollection::~pgCollection()
 }
 
 
-void pgCollection::ShowList(const wxString& name, wxTreeCtrl *browser, wxListCtrl *properties)
+void pgCollection::ShowList(const wxString& name, wxTreeCtrl *browser, ctlListView *properties)
 {
     if (properties)
     {
@@ -115,6 +119,7 @@ bool pgCollection::CanCreate()
     {
         case PG_USERS:
         case PG_GROUPS:
+        case PG_TABLESPACES:
             return GetServer()->GetSuperUser();
         case PG_DATABASES:
             return GetServer()->GetCreatePrivilege();
@@ -155,6 +160,7 @@ int pgCollection::GetIcon()
         case PG_CASTS:              return PGICON_CAST;
         case PG_LANGUAGES:          return PGICON_LANGUAGE;
         case PG_SCHEMAS:            return PGICON_SCHEMA;
+        case PG_TABLESPACES:        return PGICON_TABLESPACE;
         case PG_AGGREGATES:         return PGICON_AGGREGATE;
         case PG_CONVERSIONS:        return PGICON_CONVERSION;
         case PG_DOMAINS:            return PGICON_DOMAIN;
@@ -199,7 +205,7 @@ pgObject *pgCollection::FindChild(wxTreeCtrl *browser, int index)
 
 
 
-void pgCollection::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, wxListCtrl *properties, wxListCtrl *statistics, ctlSQLBox *sqlPane)
+void pgCollection::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, ctlListView *properties, ctlSQLBox *sqlPane)
 {
     if (browser->GetChildrenCount(GetId(), FALSE) == 0)
     {
@@ -220,6 +226,9 @@ void pgCollection::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, wxListCtrl
             case PG_SCHEMAS:
                 pgSchema::ReadObjects(this, browser);
                 break;
+            case PG_TABLESPACES:
+                pgTablespace::ReadObjects(this, browser);
+                break;
             case PG_AGGREGATES:
                 pgAggregate::ReadObjects(this, browser);
                 break;
@@ -273,23 +282,26 @@ void pgCollection::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, wxListCtrl
         }
     }
 
-
-    if (statistics)
-    {
-        switch (GetType())
-        {
-            case PG_DATABASES:
-                pgDatabase::ShowStatistics(this, statistics);
-                break;
-            case PG_TABLES:
-                pgTable::ShowStatistics(this, statistics);
-                break;
-            default:
-                break;
-        }
-    }
-
     UpdateChildCount(browser);
     if (properties)
         ShowList(typesList[GetType()+1].typName, browser, properties);
 }
+
+
+void pgCollection::ShowStatistics(frmMain *form, ctlListView *statistics)
+{
+    switch (GetType())
+    {
+        case PG_DATABASES:
+            pgDatabase::ShowStatistics(this, statistics);
+            break;
+        case PG_TABLES:
+            pgTable::ShowStatistics(this, statistics);
+            break;
+        case PG_TABLESPACES:
+            pgTablespace::ShowStatistics(this, statistics);
+            break;
+        default:
+            break;
+    }
+}
\ No newline at end of file
