@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: pgColumn.cpp,v 1.38 2004/07/27 10:36:01 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // pgColumn.cpp - Column class
@@ -27,6 +28,7 @@ pgColumn::pgColumn(pgSchema *newSchema, const wxString& newName)
 {
     isFK=false;
     isPK=false;
+    isReferenced = -1;
 }
 
 pgColumn::~pgColumn()
@@ -34,6 +36,21 @@ pgColumn::~pgColumn()
 }
 
 
+bool pgColumn::IsReferenced()
+{
+    if (isReferenced < 0)
+    {
+        isReferenced = (int)StrToLong(GetConnection()->ExecuteScalar(
+            wxT("SELECT COUNT(1) FROM pg_depend dep\n")
+            wxT("  JOIN pg_class cl ON dep.classid=cl.oid AND relname='pg_rewrite'\n")
+            wxT(" WHERE refobjid=") + GetTableOidStr()
+            + wxT(" AND refobjsubid=") + NumToStr(GetColNumber())));
+    }
+
+    return (isReferenced != 0);
+}
+
+
 bool pgColumn::DropObject(wxFrame *frame, wxTreeCtrl *browser)
 {
     wxString sql = wxT("ALTER TABLE ") + GetQuotedFullTable();
@@ -43,6 +60,21 @@ bool pgColumn::DropObject(wxFrame *frame, wxTreeCtrl *browser)
 }
 
 
+void pgColumn::ShowDependsOn(frmMain *form, ctlListView *dependsOn, const wxString &where)
+{
+    pgObject::ShowDependsOn(form, dependsOn, 
+        wxT("\n WHERE dep.objid=") + NumToStr(tableOid) +
+        wxT(" AND dep.objsubid=") + NumToStr(colNumber));
+}
+
+
+void pgColumn::ShowReferencedBy(frmMain *form, ctlListView *referencedBy, const wxString &where)
+{
+    pgObject::ShowReferencedBy(form, referencedBy, 
+        wxT("\n WHERE dep.refobjid=") + NumToStr(tableOid) +
+        wxT(" AND dep.refobjsubid=") + NumToStr(colNumber));
+}
+
 wxString pgColumn::GetSql(wxTreeCtrl *browser)
 {
     if (sql.IsNull() && !GetSystemObject())
@@ -69,22 +101,36 @@ wxString pgColumn::GetSql(wxTreeCtrl *browser)
             if (!GetDefault().IsEmpty())
                 sql += wxT("ALTER TABLE ") + GetQuotedFullTable()
                     + wxT(" ALTER COLUMN ") + GetQuotedIdentifier()
-                    + wxT(" DEFAULT ") + GetDefault() + wxT(";\n");
-            if (!GetComment().IsEmpty())
-                sql += wxT("COMMENT ON COLUMN ") + GetQuotedFullTable() + wxT(".") + GetQuotedIdentifier()
-                    +  wxT(" IS ") + qtString(GetComment()) + wxT(";\n");
+                    + wxT(" SET DEFAULT ") + GetDefault() + wxT(";\n");
+            if (GetAttstattarget() >= 0)
+                sql += wxT("ALTER TABLE ") + GetQuotedFullTable()
+                    + wxT(" ALTER COLUMN ") + GetQuotedIdentifier()
+                    + wxT(" SET STATISTICS ") + NumToStr(GetAttstattarget()) + wxT(";\n");
+
+			sql += GetCommentSql();
         }
     }
 
     return sql;
 }
 
+wxString pgColumn::GetCommentSql()
+{
+	wxString commentSql;
+
+	if (!GetComment().IsEmpty())
+		commentSql = wxT("COMMENT ON COLUMN ") + GetQuotedFullTable() + wxT(".") + GetQuotedIdentifier()
+        +  wxT(" IS ") + qtString(GetComment()) + wxT(";\n");
+	
+	return commentSql;
+}
 
 wxString pgColumn::GetDefinition()
 {
     wxString sql = GetQuotedTypename();
 
-    if ((sql == wxT("int4") || sql == wxT("int8"))
+    if ((sql == wxT("int4") || sql == wxT("int8") || 
+         sql == wxT("pg_catalog.int4") || sql == wxT("pg_catalog.int8"))
         && GetDefault() == wxT("nextval('") 
                         + schema->GetName() + wxT(".") + GetTableName() 
                         + wxT("_") + GetName() + wxT("_seq'::text)"))
@@ -107,7 +153,7 @@ wxString pgColumn::GetDefinition()
 }
 
 
-void pgColumn::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, wxListCtrl *properties, wxListCtrl *statistics, ctlSQLBox *sqlPane)
+void pgColumn::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, ctlListView *properties, ctlSQLBox *sqlPane)
 {
     if (!expandedKids)
     {
@@ -170,23 +216,28 @@ void pgColumn::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, wxListCtrl *pr
     if (properties)
     {
         CreateListColumns(properties);
-        int pos=0;
 
-        InsertListItem(properties, pos++, _("Name"), GetName());
-        InsertListItem(properties, pos++, _("Position"), GetColNumber());
-        InsertListItem(properties, pos++, _("Data type"), GetVarTypename());
-        InsertListItem(properties, pos++, _("Default"), GetDefault());
-        InsertListItem(properties, pos++, _("Not NULL?"), GetNotNull());
-        InsertListItem(properties, pos++, _("Primary key?"), GetIsPK());
-        InsertListItem(properties, pos++, _("Foreign key?"), GetIsFK());
-        InsertListItem(properties, pos++, _("Storage"), GetStorage());
-        InsertListItem(properties, pos++, _("Inherited"), GetInheritedCount() != 0);
+        properties->AppendItem(_("Name"), GetName());
+        properties->AppendItem(_("Position"), GetColNumber());
+        properties->AppendItem(_("Data type"), GetVarTypename());
+        properties->AppendItem(_("Default"), GetDefault());
+        properties->AppendItem(_("Sequence"), database->GetSchemaPrefix(GetSerialSchema()) + GetSerialSequence());
+        properties->AppendItem(_("Not NULL?"), GetNotNull());
+        properties->AppendItem(_("Primary key?"), GetIsPK());
+        properties->AppendItem(_("Foreign key?"), GetIsFK());
+        properties->AppendItem(_("Storage"), GetStorage());
+        properties->AppendItem(_("Inherited"), GetInheritedCount() != 0);
+        properties->AppendItem(_("Statistics"), GetAttstattarget());
 
-        InsertListItem(properties, pos++, _("System column?"), GetSystemObject());
-        InsertListItem(properties, pos++, _("Comment"), GetComment());
+
+        properties->AppendItem(_("System column?"), GetSystemObject());
+        properties->AppendItem(_("Comment"), GetComment());
     }
+}
 
-    // statistic 
+
+void pgColumn::ShowStatistics(frmMain *form, ctlListView *statistics)
+{
     DisplayStatistics(statistics,
         wxT("SELECT null_frac AS ") + qtIdent(_("Null Fraction")) +
              wxT(", avg_width AS ") + qtIdent(_("Average Width")) +
@@ -202,7 +253,6 @@ void pgColumn::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, wxListCtrl *pr
 }
 
 
-
 pgObject *pgColumn::Refresh(wxTreeCtrl *browser, const wxTreeItemId item)
 {
     pgObject *column=0;
@@ -221,13 +271,16 @@ pgObject *pgColumn::Refresh(wxTreeCtrl *browser, const wxTreeItemId item)
 pgObject *pgColumn::ReadObjects(pgCollection *collection, wxTreeCtrl *browser, const wxString &restriction)
 {
     pgColumn *column=0;
+    pgDatabase *database=collection->GetDatabase();
 
     wxString systemRestriction;
     if (!settings->GetShowSystemObjects())
         systemRestriction = wxT("\n   AND attnum > 0");
         
-    pgSet *columns= collection->GetDatabase()->ExecuteSet(
-        wxT("SELECT att.*, def.*, CASE when attndims > 0 THEN 1 ELSE 0 END AS isarray, CASE WHEN ty.typname = 'bpchar' THEN 'char' WHEN ty.typname = '_bpchar' THEN '_char' ELSE ty.typname END AS typname, tn.nspname as typnspname, et.typname as elemtypname, relname, na.nspname, description\n")
+    pgSet *columns= database->ExecuteSet(
+        wxT("SELECT att.*, def.*, CASE WHEN attndims > 0 THEN 1 ELSE 0 END AS isarray, CASE WHEN ty.typname = 'bpchar' THEN 'char' WHEN ty.typname = '_bpchar' THEN '_char' ELSE ty.typname END AS typname, tn.nspname as typnspname, et.typname as elemtypname,\n")
+        wxT("  cl.relname, na.nspname, att.attstattarget, description, cs.relname AS sername, ns.nspname AS serschema,\n")
+        wxT("  (SELECT count(1) FROM pg_type t2 WHERE t2.typname=ty.typname) > 1 AS isdup\n")
         wxT("  FROM pg_attribute att\n")
         wxT("  JOIN pg_type ty ON ty.oid=atttypid\n")
         wxT("  JOIN pg_namespace tn ON tn.oid=ty.typnamespace\n")
@@ -236,6 +289,8 @@ pgObject *pgColumn::ReadObjects(pgCollection *collection, wxTreeCtrl *browser, c
         wxT("  LEFT OUTER JOIN pg_type et ON et.oid=ty.typelem\n")
         wxT("  LEFT OUTER JOIN pg_attrdef def ON adrelid=attrelid AND adnum=attnum\n")
         wxT("  LEFT OUTER JOIN pg_description des ON des.objoid=attrelid AND des.objsubid=attnum\n")
+        wxT("  LEFT OUTER JOIN (pg_depend JOIN pg_class cs ON objid=cs.oid AND cs.relkind='S') ON refobjid=attrelid AND refobjsubid=attnum\n")
+        wxT("  LEFT OUTER JOIN pg_namespace ns ON ns.oid=cs.relnamespace\n")
         wxT(" WHERE attrelid = ") + collection->GetOidStr()
         + restriction + systemRestriction + wxT("\n")
         wxT("   AND attisdropped IS FALSE\n")
@@ -252,6 +307,9 @@ pgObject *pgColumn::ReadObjects(pgCollection *collection, wxTreeCtrl *browser, c
             column->iSetColNumber(columns->GetLong(wxT("attnum")));
             column->iSetIsArray(columns->GetBool(wxT("isarray")));
             column->iSetComment(columns->GetVal(wxT("description")));
+            column->iSetSerialSequence(columns->GetVal(wxT("sername")));
+            column->iSetSerialSchema(columns->GetVal(wxT("serschema")));
+
             if (columns->GetBool(wxT("atthasdef")))
                 column->iSetDefault(columns->GetVal(wxT("adsrc")));
             column->iSetStatistics(columns->GetLong(wxT("attstattarget")));
@@ -266,7 +324,9 @@ pgObject *pgColumn::ReadObjects(pgCollection *collection, wxTreeCtrl *browser, c
             column->iSetTyplen(columns->GetLong(wxT("attlen")));
 
             long typmod=columns->GetLong(wxT("atttypmod"));
-            pgDatatype dt(columns->GetVal(wxT("typname")), columns->GetBool(wxT("isarray"))? 1 : 0, typmod);
+            pgDatatype dt(columns->GetVal(wxT("typnspname")), columns->GetVal(wxT("typname")), 
+                columns->GetBool(wxT("isdup")),
+                columns->GetBool(wxT("isarray"))? 1 : 0, typmod);
 
 
             column->iSetTypmod(typmod);
@@ -274,23 +334,15 @@ pgObject *pgColumn::ReadObjects(pgCollection *collection, wxTreeCtrl *browser, c
             column->iSetPrecision(dt.Precision());
             column->iSetRawTypename(dt.Name());
 
-            wxString nsp=columns->GetVal(wxT("typnspname"));
-            if (nsp == wxT("pg_catalog"))
-            {
-                column->iSetVarTypename(dt.FullName());
-                column->iSetQuotedTypename(dt.QuotedFullName());
-            }
-            else
-            {
-                column->iSetVarTypename(nsp + wxT(".") + dt.FullName());
-                column->iSetQuotedTypename(qtIdent(nsp) + wxT(".") + dt.QuotedFullName());
-            }
+            column->iSetVarTypename(dt.GetSchemaPrefix(database) + dt.FullName());
+            column->iSetQuotedTypename(dt.GetQuotedSchemaPrefix(database) + dt.QuotedFullName());
 
             column->iSetNotNull(columns->GetBool(wxT("attnotnull")));
-            column->iSetQuotedFullTable(qtIdent(columns->GetVal(wxT("nspname"))) + wxT(".")
+            column->iSetQuotedFullTable(database->GetQuotedSchemaPrefix(columns->GetVal(wxT("nspname")))
                 + qtIdent(columns->GetVal(wxT("relname"))));
             column->iSetTableName(columns->GetVal(wxT("relname")));
             column->iSetInheritedCount(columns->GetLong(wxT("attinhcount")));
+            column->iSetAttstattarget(columns->GetLong(wxT("attstattarget")));
 
             if (browser)
             {
