@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: pgForeignKey.cpp,v 1.25 2004/10/21 14:34:38 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // pgForeignKey.cpp - ForeignKey class
@@ -15,6 +16,7 @@
 #include "pgAdmin3.h"
 #include "misc.h"
 #include "pgObject.h"
+#include "pgTable.h"
 #include "pgForeignKey.h"
 #include "pgCollection.h"
 
@@ -32,7 +34,7 @@ pgForeignKey::~pgForeignKey()
 bool pgForeignKey::DropObject(wxFrame *frame, wxTreeCtrl *browser)
 {
     return GetDatabase()->ExecuteVoid(wxT(
-        "ALTER TABLE ") + qtIdent(fkSchema) + wxT(".") + qtIdent(fkTable)
+        "ALTER TABLE ") + GetSchemaPrefix(fkSchema) + qtIdent(fkTable)
         + wxT(" DROP CONSTRAINT ") + GetQuotedIdentifier());
 }
 
@@ -43,7 +45,7 @@ wxString pgForeignKey::GetDefinition()
     // MATCH FULL/PARTIAL missing; where is this stored?!?
 
     sql = wxT("(") + GetQuotedFkColumns()
-        +  wxT(") REFERENCES ") + qtIdent(GetRefSchema()) + wxT(".") + qtIdent(GetReferences()) 
+        +  wxT(") REFERENCES ") + GetQuotedSchemaPrefix(GetRefSchema()) + qtIdent(GetReferences()) 
         +  wxT(" (") + GetQuotedRefColumns()
         +  wxT(") ON UPDATE ") + GetOnUpdate()
         +  wxT(" ON DELETE ") + GetOnDelete();
@@ -74,13 +76,13 @@ wxString pgForeignKey::GetSql(wxTreeCtrl *browser)
     if (sql.IsNull())
     {
         sql = wxT("-- Foreign Key: ") + GetQuotedFullIdentifier() + wxT("\n\n")
-            + wxT("-- ALTER TABLE ") + qtIdent(fkSchema) + wxT(".") + qtIdent(fkTable)
+            + wxT("-- ALTER TABLE ") + GetQuotedSchemaPrefix(fkSchema) + qtIdent(fkTable)
             + wxT(" DROP CONSTRAINT ") + GetQuotedIdentifier() + wxT(";")
-            + wxT("\n\nALTER TABLE ") + qtIdent(fkSchema) + wxT(".") + qtIdent(fkTable)
+            + wxT("\n\nALTER TABLE ") + GetQuotedSchemaPrefix(fkSchema) + qtIdent(fkTable)
             + wxT("\n  ADD CONSTRAINT ") + GetConstraint() 
             + wxT(";\n");
         if (!GetComment().IsEmpty())
-            sql += wxT("COMMENT ON CONSTRAINT ") + GetQuotedIdentifier() + wxT(" ON ") + qtIdent(fkSchema) + wxT(".") + qtIdent(fkTable)
+            sql += wxT("COMMENT ON CONSTRAINT ") + GetQuotedIdentifier() + wxT(" ON ") + GetQuotedSchemaPrefix(fkSchema) + qtIdent(fkTable)
                 +  wxT(" IS ") + qtString(GetComment()) + wxT(";\n");
     }
 
@@ -93,7 +95,7 @@ wxString pgForeignKey::GetFullName() const
     return GetName() + wxT(" -> ") + GetReferences();
 }
 
-void pgForeignKey::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, wxListCtrl *properties, wxListCtrl *statistics, ctlSQLBox *sqlPane)
+void pgForeignKey::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, ctlListView *properties, ctlSQLBox *sqlPane)
 {
     if (!expandedKids)
     {
@@ -103,6 +105,7 @@ void pgForeignKey::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, wxListCtrl
         wxStringTokenizer c2l(GetConfkey(), wxT(","));
         wxString c1, c2;
 
+        // resolve column names
         while (c1l.HasMoreTokens())
         {
             c1=c1l.GetNextToken();
@@ -128,26 +131,37 @@ void pgForeignKey::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, wxListCtrl
                 delete set;
             }
         }
+        wxTreeItemId item=browser->GetItemParent(GetId());
+        while (item)
+        {
+            pgTable *table=(pgTable*)browser->GetItemData(item);
+            if (table->GetType() == PG_TABLE)
+            {
+                coveringIndex = table->GetCoveringIndex(browser, fkColumns);
+                break;
+            }
+            item = browser->GetItemParent(item);
+        }
     }
 
     if (properties)
     {
         CreateListColumns(properties);
-        int pos=0;
 
-        InsertListItem(properties, pos++, _("Name"), GetName());
-        InsertListItem(properties, pos++, _("OID"), NumToStr(GetOid()));
-        InsertListItem(properties, pos++, _("Child columns"), GetFkColumns());
-        InsertListItem(properties, pos++, _("References"), GetReferences() 
+        properties->AppendItem(_("Name"), GetName());
+        properties->AppendItem(_("OID"), NumToStr(GetOid()));
+        properties->AppendItem(_("Child columns"), GetFkColumns());
+        properties->AppendItem(_("References"), GetReferences() 
             + wxT("(") +GetRefColumns() + wxT(")"));
 
-        InsertListItem(properties, pos++, _("On update"), GetOnUpdate());
-        InsertListItem(properties, pos++, _("On delete"), GetOnDelete());
-        InsertListItem(properties, pos++, _("Deferrable?"), BoolToYesNo(GetDeferrable()));
-        InsertListItem(properties, pos++, _("Initially?"), 
+        properties->AppendItem(_("Covering index"), GetCoveringIndex());
+        properties->AppendItem(_("On update"), GetOnUpdate());
+        properties->AppendItem(_("On delete"), GetOnDelete());
+        properties->AppendItem(_("Deferrable?"), BoolToYesNo(GetDeferrable()));
+        properties->AppendItem(_("Initially?"), 
             GetDeferred() ? wxT("DEFERRED") : wxT("IMMEDIATE"));
-        InsertListItem(properties, pos++, _("System foreign key?"), BoolToYesNo(GetSystemObject()));
-        InsertListItem(properties, pos++, _("Comment"), GetComment());
+        properties->AppendItem(_("System foreign key?"), BoolToYesNo(GetSystemObject()));
+        properties->AppendItem(_("Comment"), GetComment());
     }
 }
 
@@ -207,13 +221,13 @@ pgObject *pgForeignKey::ReadObjects(pgCollection *collection, wxTreeCtrl *browse
                 onUpd.IsSameAs('a') ? wxT("NO ACTION") :
                 onUpd.IsSameAs('r') ? wxT("RESTRICT") :
                 onUpd.IsSameAs('c') ? wxT("CASCADE") :
-                onUpd.IsSameAs('d') ? wxT("DEFAULT") :
+                onUpd.IsSameAs('d') ? wxT("SET DEFAULT") :
                 onUpd.IsSameAs('n') ? wxT("SET NULL") : wxT("Unknown"));
             foreignKey->iSetOnDelete(
                 onDel.IsSameAs('a') ? wxT("NO ACTION") :
                 onDel.IsSameAs('r') ? wxT("RESTRICT") :
                 onDel.IsSameAs('c') ? wxT("CASCADE") :
-                onDel.IsSameAs('d') ? wxT("DEFAULT") :
+                onDel.IsSameAs('d') ? wxT("SET DEFAULT") :
                 onDel.IsSameAs('n') ? wxT("SET NULL") : wxT("Unknown"));
             wxString cn=foreignKeys->GetVal(wxT("conkey"));
             cn = cn.Mid(1, cn.Length()-2);
