@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: pgFunction.cpp,v 1.40 2004/10/08 13:52:19 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // pgFunction.cpp - function class
@@ -45,29 +46,19 @@ pgTriggerFunction::pgTriggerFunction(pgSchema *newSchema, const wxString& newNam
 }
 
 
-bool pgFunction::CanReload()
-{
-    return GetLanguage().IsSameAs(wxT("C"), false);
-}
-
-
-bool pgFunction::ReloadLibrary()
-{
-    return ExecuteVoid(wxT("LOAD ") + qtString(GetBin()));
-}
-
-
 wxString pgFunction::GetSql(wxTreeCtrl *browser)
 {
     if (sql.IsNull())
     {
-        sql = wxT("-- Function: ") + GetQuotedFullIdentifier() + wxT("(") + GetQuotedArgTypes() + wxT(")\n\n")
-            + wxT("-- DROP FUNCTION ") + GetQuotedFullIdentifier()  + wxT("(") + GetQuotedArgTypes() + wxT(");")
-            + wxT("\n\nCREATE OR REPLACE FUNCTION ") + GetQuotedFullIdentifier() + wxT("(") + GetQuotedArgTypes()
-            + wxT(")\n  RETURNS ");
+        wxString qtName = wxT("FUNCTION ") + GetQuotedFullIdentifier()  + wxT("(") + GetQuotedArgTypes() + wxT(")");
+
+        sql = wxT("-- Function: ") + GetFullIdentifier() + wxT("(") + GetArgTypeNames() + wxT(")\n\n")
+            + wxT("-- DROP ") + qtName + wxT(";")
+            + wxT("\n\nCREATE OR REPLACE ") + qtName
+            + wxT("\n  RETURNS ");
         if (GetReturnAsSet())
             sql += wxT("SETOF ");
-        sql +=GetReturnType() 
+        sql +=GetQuotedReturnType() 
             + wxT(" AS\n");
         
         if (GetLanguage().IsSameAs(wxT("C"), false))
@@ -77,7 +68,7 @@ wxString pgFunction::GetSql(wxTreeCtrl *browser)
         else
         {
             if (GetConnection()->BackendMinimumVersion(7, 5))
-                sql += qtDocumentHere(GetSource());
+                sql += qtStringDollar(GetSource());
             else
                 sql += qtString(GetSource());
         }
@@ -87,48 +78,49 @@ wxString pgFunction::GetSql(wxTreeCtrl *browser)
             sql += wxT(" STRICT");
         if (GetSecureDefiner())
             sql += wxT(" SECURITY DEFINER");
-        sql += wxT(";\n");
+        sql += wxT(";\n")
+            +  GetOwnerSql(8, 0, qtName)
+            +  GetGrant(wxT("X"), qtName);
 
-    if (!GetComment().IsNull())
-    {
-        sql += wxT("COMMENT ON FUNCTION ") + GetQuotedFullIdentifier() + wxT("(") + GetQuotedArgTypes() + wxT(")")
-            + wxT(" IS ") + qtString(GetComment()) + wxT(";\n");
+        if (!GetComment().IsNull())
+        {
+            sql += wxT("COMMENT ON ") + qtName
+                + wxT(" IS ") + qtString(GetComment()) + wxT(";\n");
+        }
     }
-}
 
     return sql;
 }
 
 
-void pgFunction::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, wxListCtrl *properties, wxListCtrl *statistics, ctlSQLBox *sqlPane)
+void pgFunction::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, ctlListView *properties, ctlSQLBox *sqlPane)
 {
     if (properties)
     {
         CreateListColumns(properties);
-        int pos=0;
 
-        InsertListItem(properties, pos++, _("Name"), GetName());
-        InsertListItem(properties, pos++, _("OID"), GetOid());
-        InsertListItem(properties, pos++, _("Owner"), GetOwner());
-        InsertListItem(properties, pos++, _("Argument count"), GetArgCount());
-        InsertListItem(properties, pos++, _("Arguments"), GetArgTypes());
-        InsertListItem(properties, pos++, _("Return type"), GetReturnType());
-        InsertListItem(properties, pos++, _("Language"), GetLanguage());
-        InsertListItem(properties, pos++, _("Returns a set?"), GetReturnAsSet());
+        properties->AppendItem(_("Name"), GetName());
+        properties->AppendItem(_("OID"), GetOid());
+        properties->AppendItem(_("Owner"), GetOwner());
+        properties->AppendItem(_("Argument count"), GetArgCount());
+        properties->AppendItem(_("Arguments"), GetArgTypeNames());
+        properties->AppendItem(_("Return type"), GetReturnType());
+        properties->AppendItem(_("Language"), GetLanguage());
+        properties->AppendItem(_("Returns a set?"), GetReturnAsSet());
         if (GetLanguage().IsSameAs(wxT("C"), false))
         {
-            InsertListItem(properties, pos++, _("Object file"), GetBin());
-            InsertListItem(properties, pos++, _("Link symbol"), GetSource());
+            properties->AppendItem(_("Object file"), GetBin());
+            properties->AppendItem(_("Link symbol"), GetSource());
         }
         else
-            InsertListItem(properties, pos++, _("Source"), GetSource());
+            properties->AppendItem(_("Source"), GetSource());
 
-        InsertListItem(properties, pos++, _("Volatility"), GetVolatility());
-        InsertListItem(properties, pos++, _("Security of definer?"), GetSecureDefiner());
-        InsertListItem(properties, pos++, _("Strict?"), GetIsStrict());
-        InsertListItem(properties, pos++, _("ACL"), GetAcl());
-        InsertListItem(properties, pos++, _("System function?"), GetSystemObject());
-        InsertListItem(properties, pos++, _("Comment"), GetComment());
+        properties->AppendItem(_("Volatility"), GetVolatility());
+        properties->AppendItem(_("Security of definer?"), GetSecureDefiner());
+        properties->AppendItem(_("Strict?"), GetIsStrict());
+        properties->AppendItem(_("ACL"), GetAcl());
+        properties->AppendItem(_("System function?"), GetSystemObject());
+        properties->AppendItem(_("Comment"), GetComment());
     }
 }
 
@@ -138,18 +130,23 @@ void pgFunction::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, wxListCtrl *
 pgFunction *pgFunction::AppendFunctions(pgObject *obj, pgSchema *schema, wxTreeCtrl *browser, const wxString &restriction)
 {
     pgFunction *function=0;
+    wxString argNamesCol;
+    if (obj->GetConnection()->BackendMinimumVersion(7, 5))
+        argNamesCol = wxT("proargnames, ");
 
     pgSet *functions = obj->GetDatabase()->ExecuteSet(
-            wxT("SELECT pr.oid, pr.*, TYP.typname, lanname, pg_get_userbyid(proowner) as funcowner, description\n")
+            wxT("SELECT pr.oid, pr.*, TYP.typname, TYPNS.nspname AS typnsp, lanname, ") + argNamesCol + 
+                        wxT("pg_get_userbyid(proowner) as funcowner, description\n")
             wxT("  FROM pg_proc pr\n")
             wxT("  JOIN pg_type TYP ON TYP.oid=prorettype\n")
+            wxT("  JOIN pg_namespace TYPNS ON TYPNS.oid=TYP.typnamespace\n")
             wxT("  JOIN pg_language LNG ON LNG.oid=prolang\n")
             wxT("  LEFT OUTER JOIN pg_description des ON des.objoid=pr.oid\n")
             + restriction +
             wxT(" ORDER BY proname"));
 
     pgSet *types = obj->GetDatabase()->ExecuteSet(wxT(
-                    "SELECT oid, typname FROM pg_type"));
+                    "SELECT t.oid, t.typname, n.nspname FROM pg_type t, pg_namespace n WHERE t.typnamespace = n.oid"));
 
     if (functions)
     {
@@ -164,13 +161,18 @@ pgFunction *pgFunction::AppendFunctions(pgObject *obj, pgSchema *schema, wxTreeC
             function->iSetOwner(functions->GetVal(wxT("funcowner")));
             function->iSetAcl(functions->GetVal(wxT("proacl")));
             function->iSetArgCount(functions->GetLong(wxT("pronargs")));
-            function->iSetReturnType(functions->GetVal(wxT("typname")));
+            function->iSetReturnType(obj->GetDatabase()->GetSchemaPrefix(functions->GetVal(wxT("typnsp"))) + functions->GetVal(wxT("typname")));
+			function->iSetQuotedReturnType(obj->GetDatabase()->GetQuotedSchemaPrefix(functions->GetVal(wxT("typnsp"))) + qtIdent(functions->GetVal(wxT("typname"))));
             function->iSetComment(functions->GetVal(wxT("description")));
             wxString oids=functions->GetVal(wxT("proargtypes"));
+            wxString argNames;
+            if (obj->GetConnection()->BackendMinimumVersion(7, 5))
+                argNames = functions->GetVal(wxT("proargnames"));
             function->iSetArgTypeOids(oids);
 
-            wxString str, argTypes, quotedArgTypes;
+            wxString str, argTypes, quotedArgTypes, argTypeNames, quotedArgTypeNames;
             wxStringTokenizer args(oids);
+            wxStringTokenizer names(argNames.Mid(1, argNames.Length()-2), wxT(","));
             while (args.HasMoreTokens())
             {
                 str = args.GetNextToken();
@@ -178,22 +180,35 @@ pgFunction *pgFunction::AppendFunctions(pgObject *obj, pgSchema *schema, wxTreeC
                 if (types)
                 {
                     types->MoveFirst();
-                    while (types->GetVal(0) != str)
+                    while (types->GetVal(wxT("oid")) != str)
                         types->MoveNext();
 
                     if (!argTypes.IsNull())
-		    {
+                    {
                         argTypes += wxT(", ");
                         quotedArgTypes += wxT(", ");
-		    }
-                    argTypes += types->GetVal(1);
-                    quotedArgTypes += qtIdent(types->GetVal(1));
+                        argTypeNames += wxT(", ");
+                        quotedArgTypeNames += wxT(", ");
+                    }
+                    str = names.GetNextToken();
+                    if (!str.IsNull())
+                    {
+                        function->iAddArgName(str);
+                        argTypeNames += str + wxT(" ");
+                        quotedArgTypeNames += qtIdent(str) + wxT(" ");
+                    }
 
+    				argTypeNames += obj->GetDatabase()->GetSchemaPrefix(types->GetVal(wxT("nspname"))) + types->GetVal(wxT("typname"));
+					quotedArgTypeNames += obj->GetDatabase()->GetQuotedSchemaPrefix(types->GetVal(wxT("nspname"))) + qtIdent(types->GetVal(wxT("typname")));
+    				argTypes += obj->GetDatabase()->GetSchemaPrefix(types->GetVal(wxT("nspname"))) + types->GetVal(wxT("typname"));
+					quotedArgTypes += obj->GetDatabase()->GetQuotedSchemaPrefix(types->GetVal(wxT("nspname"))) + qtIdent(types->GetVal(wxT("typname")));
                 }
             }
 
             function->iSetArgTypes(argTypes);
             function->iSetQuotedArgTypes(quotedArgTypes);
+            function->iSetArgTypeNames(argTypeNames);
+            function->iSetQuotedArgTypeNames(quotedArgTypeNames);
 
             function->iSetLanguage(functions->GetVal(wxT("lanname")));
             function->iSetSecureDefiner(functions->GetBool(wxT("prosecdef")));
