@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: pgRule.cpp,v 1.26 2004/01/02 23:15:00 dpage Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // pgRule.cpp - Rule class
@@ -50,29 +51,28 @@ wxString pgRule::GetSql(wxTreeCtrl *browser)
 }
 
 
-void pgRule::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, wxListCtrl *properties, wxListCtrl *statistics, ctlSQLBox *sqlPane)
+void pgRule::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, ctlListView *properties, ctlSQLBox *sqlPane)
 {
     if (properties)
     {
         CreateListColumns(properties);
-        int pos=0;
         wxString def=GetDefinition().Left(250);
 
-        InsertListItem(properties, pos++, _("Name"), GetName());
-        InsertListItem(properties, pos++, _("OID"), GetOid());
-        InsertListItem(properties, pos++, _("Event"), GetEvent());
-        InsertListItem(properties, pos++, _("Condition"), GetCondition());
-        InsertListItem(properties, pos++, _("Do instead?"), GetDoInstead());
-        InsertListItem(properties, pos++, _("Action"), GetAction().Left(250));
+        properties->AppendItem(_("Name"), GetName());
+        properties->AppendItem(_("OID"), GetOid());
+        properties->AppendItem(_("Event"), GetEvent());
+        properties->AppendItem(_("Condition"), GetCondition());
+        properties->AppendItem(_("Do instead?"), GetDoInstead());
+        properties->AppendItem(_("Action"), GetAction().Left(250));
         if (def.IsEmpty())
-            InsertListItem(properties, pos++, _("Definition"), wxT("NOTHING"));
+            properties->AppendItem(_("Definition"), wxT("NOTHING"));
         else
         {
             def.Replace(wxT("\n"), wxT(" "));
-            InsertListItem(properties, pos++, _("Definition"), def);
+            properties->AppendItem(_("Definition"), def);
         }
-        InsertListItem(properties, pos++, _("System rule?"), GetSystemObject());
-        InsertListItem(properties, pos++, _("Comment"), GetComment());
+        properties->AppendItem(_("System rule?"), GetSystemObject());
+        properties->AppendItem(_("Comment"), GetComment());
     }
 }
 
@@ -125,7 +125,7 @@ pgObject *pgRule::ReadObjects(pgCollection *collection, wxTreeCtrl *browser, con
                 rule->iSetCondition(definition.Mid(wherePos+7, doPos-wherePos-7));
 
             rule->iSetDefinition(definition);
-            rule->iSetQuotedFullTable(qtIdent(rules->GetVal(wxT("nspname"))) + wxT(".")
+            rule->iSetQuotedFullTable(collection->GetDatabase()->GetSchemaPrefix(rules->GetVal(wxT("nspname"))) 
                 + qtIdent(rules->GetVal(wxT("relname"))));
             wxChar *evts[] = {0, wxT("SELECT"), wxT("UPDATE"), wxT("INSERT"), wxT("DELETE")};
             int evno=StrToLong(rules->GetVal(wxT("ev_type")));
