@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: pgSequence.cpp,v 1.28 2004/10/08 13:52:19 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // pgSequence.cpp - Sequence class
@@ -68,60 +69,64 @@ wxString pgSequence::GetSql(wxTreeCtrl *browser)
             + wxT("\n  CACHE ") + GetCacheValue().ToString();
         if (GetCycled())
             sql += wxT("\n  CYCLE");
+        AppendIfFilled(sql, wxT("\n  TABLESPACE "), qtIdent(tablespace));
         sql += wxT(";\n")
-            + GetGrant(wxT("arwdRxt"), wxT("TABLE"))
+            + GetOwnerSql(7, 3, wxT("TABLE ") + GetQuotedFullIdentifier())
+            + GetGrant(wxT("arwdRxt"), wxT("TABLE ") + GetQuotedFullIdentifier())
             + GetCommentSql();
     }
 
     return sql;
 }
 
-void pgSequence::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, wxListCtrl *properties, wxListCtrl *statistics, ctlSQLBox *sqlPane)
+void pgSequence::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, ctlListView *properties, ctlSQLBox *sqlPane)
 {
     UpdateValues();
     if (properties)
     {
         CreateListColumns(properties);
-        int pos=0;
 
-        InsertListItem(properties, pos++, _("Name"), GetName());
-        InsertListItem(properties, pos++, _("OID"), GetOid());
-        InsertListItem(properties, pos++, _("Owner"), GetOwner());
-        InsertListItem(properties, pos++, _("ACL"), GetAcl());
-        InsertListItem(properties, pos++, _("Current value"), GetLastValue().ToString());
-        InsertListItem(properties, pos++, _("Minimum"), GetMinValue().ToString());
-        InsertListItem(properties, pos++, _("Maximum"), GetMaxValue().ToString());
-        InsertListItem(properties, pos++, _("Increment"), GetIncrement().ToString());
-        InsertListItem(properties, pos++, _("Cache"), GetCacheValue().ToString());
-        InsertListItem(properties, pos++, _("Cycled"), GetCycled());
-        InsertListItem(properties, pos++, _("System sequence?"), GetSystemObject());
-        InsertListItem(properties, pos++, _("Comment"), GetComment());
-    }
-
-    wxLogInfo(wxT("Displaying statistics for sequence on ") +GetSchema()->GetIdentifier());
-
-    if (statistics)
-    {
-        // Add the statistics view columns
-        CreateListColumns(statistics, _("Statistic"), _("Value"));
-
-        pgSet *stats = GetSchema()->GetDatabase()->ExecuteSet(wxT(
-            "SELECT blks_read, blks_hit FROM pg_statio_all_sequences WHERE relid = ") + GetOidStr());
-
-        if (stats)
-        {
-            statistics->InsertItem(0, wxT("Blocks Read"), PGICON_STATISTICS);
-            statistics->SetItem(0l, 1, stats->GetVal(wxT("blks_read")));
-            statistics->InsertItem(1, wxT("Blocks Hit"), PGICON_STATISTICS);
-            statistics->SetItem(1, 1, stats->GetVal(wxT("blks_hit")));
-
-            delete stats;
-        }
+        properties->AppendItem(_("Name"), GetName());
+        properties->AppendItem(_("OID"), GetOid());
+        properties->AppendItem(_("Owner"), GetOwner());
+        if (!tablespace.IsEmpty())
+            properties->AppendItem(_("Tablespace"), tablespace);
+        properties->AppendItem(_("ACL"), GetAcl());
+        properties->AppendItem(_("Current value"), GetLastValue());
+        properties->AppendItem(_("Minimum"), GetMinValue());
+        properties->AppendItem(_("Maximum"), GetMaxValue());
+        properties->AppendItem(_("Increment"), GetIncrement());
+        properties->AppendItem(_("Cache"), GetCacheValue());
+        properties->AppendItem(_("Cycled"), GetCycled());
+        properties->AppendItem(_("System sequence?"), GetSystemObject());
+        properties->AppendItem(_("Comment"), GetComment());
     }
 }
 
 
 
+void pgSequence::ShowStatistics(frmMain *form, ctlListView *statistics)
+{
+    wxLogInfo(wxT("Displaying statistics for sequence on ") +GetSchema()->GetIdentifier());
+
+    // Add the statistics view columns
+    CreateListColumns(statistics, _("Statistic"), _("Value"));
+
+    pgSet *stats = GetSchema()->GetDatabase()->ExecuteSet(wxT(
+        "SELECT blks_read, blks_hit FROM pg_statio_all_sequences WHERE relid = ") + GetOidStr());
+
+    if (stats)
+    {
+        statistics->InsertItem(0, wxT("Blocks Read"), PGICON_STATISTICS);
+        statistics->SetItem(0l, 1, stats->GetVal(wxT("blks_read")));
+        statistics->InsertItem(1, wxT("Blocks Hit"), PGICON_STATISTICS);
+        statistics->SetItem(1, 1, stats->GetVal(wxT("blks_hit")));
+
+        delete stats;
+    }
+}
+
+
 pgObject *pgSequence::Refresh(wxTreeCtrl *browser, const wxTreeItemId item)
 {
     pgObject *sequence=0;
@@ -141,14 +146,28 @@ pgObject *pgSequence::ReadObjects(pgCollection *collection, wxTreeCtrl *browser,
 {
     pgSequence *sequence=0;
 
-    pgSet *sequences= collection->GetDatabase()->ExecuteSet(
-        wxT("SELECT cl.oid, relname, pg_get_userbyid(relowner) AS seqowner, relacl, description\n")
-        wxT("  FROM pg_class cl\n")
-        wxT("  LEFT OUTER JOIN pg_description des ON des.objoid=cl.oid\n")
-        wxT(" WHERE relkind = 'S' AND relnamespace  = ") + collection->GetSchema()->GetOidStr()
-        + restriction + wxT("\n")
-        wxT(" ORDER BY relname"));
-
+    pgSet *sequences;
+    if (collection->GetConnection()->BackendMinimumVersion(7, 5))
+    {
+        sequences = collection->GetDatabase()->ExecuteSet(
+            wxT("SELECT cl.oid, relname, spcname, pg_get_userbyid(relowner) AS seqowner, relacl, description\n")
+            wxT("  FROM pg_class cl\n")
+            wxT("  LEFT OUTER JOIN pg_tablespace ta on ta.oid=cl.reltablespace\n")
+            wxT("  LEFT OUTER JOIN pg_description des ON des.objoid=cl.oid\n")
+            wxT(" WHERE relkind = 'S' AND relnamespace  = ") + collection->GetSchema()->GetOidStr()
+            + restriction + wxT("\n")
+            wxT(" ORDER BY relname"));
+    }
+    else
+    {
+        sequences = collection->GetDatabase()->ExecuteSet(
+            wxT("SELECT cl.oid, relname, pg_get_userbyid(relowner) AS seqowner, relacl, description\n")
+            wxT("  FROM pg_class cl\n")
+            wxT("  LEFT OUTER JOIN pg_description des ON des.objoid=cl.oid\n")
+            wxT(" WHERE relkind = 'S' AND relnamespace  = ") + collection->GetSchema()->GetOidStr()
+            + restriction + wxT("\n")
+            wxT(" ORDER BY relname"));
+    }
     if (sequences)
     {
         while (!sequences->Eof())
@@ -160,6 +179,8 @@ pgObject *pgSequence::ReadObjects(pgCollection *collection, wxTreeCtrl *browser,
             sequence->iSetComment(sequences->GetVal(wxT("description")));
             sequence->iSetOwner(sequences->GetVal(wxT("seqowner")));
             sequence->iSetAcl(sequences->GetVal(wxT("relacl")));
+            if (collection->GetConnection()->BackendMinimumVersion(7, 5))
+                sequence->iSetTablespace(sequences->GetVal(wxT("spcname")));
 
             if (browser)
             {
