@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: pgTable.cpp,v 1.59 2004/11/19 15:42:10 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // pgTable.cpp - Table class
@@ -23,12 +24,13 @@
 #include "pgForeignKey.h"
 #include "pgCheck.h"
 #include "sysSettings.h"
+#include "pgfeatures.h"
 
 pgTable::pgTable(pgSchema *newSchema, const wxString& newName)
 : pgSchemaObject(newSchema, PG_TABLE, newName)
 {
     inheritedTableCount=0;
-    rows=-1;
+    rowsCounted = false;
 }
 
 pgTable::~pgTable()
@@ -97,7 +99,7 @@ wxString pgTable::GetAllConstraints(wxTreeCtrl *browser, wxTreeItemId collection
                     sql += ((pgForeignKey*)data)->GetDefinition();
                     break;
                 case PG_CHECK:
-                    sql += ((pgCheck*)data)->GetDefinition();
+				  sql += wxT("(") + ((pgCheck*)data)->GetDefinition() + wxT(")");
                     break;
             }
         }
@@ -111,6 +113,7 @@ wxString pgTable::GetAllConstraints(wxTreeCtrl *browser, wxTreeItemId collection
 
 wxString pgTable::GetSql(wxTreeCtrl *browser)
 {
+	wxString colDetails;
     if (sql.IsNull())
     {
         // make sure all kids are appended
@@ -163,6 +166,12 @@ wxString pgTable::GetSql(wxTreeCtrl *browser)
                         sql += wxT("  ") + column->GetQuotedIdentifier() + wxT(" ")
                             + column->GetDefinition();
 
+						// Whilst we are looping round the columns, grab their comments as well.
+						// Perhaps we should also get storage types here?
+						colDetails += column->GetCommentSql();
+						if (colDetails.Length() > 0)
+							if (colDetails.Last() != '\n') colDetails += wxT("\n");
+
                         colCount++;
                     }
                 }
@@ -189,31 +198,73 @@ wxString pgTable::GetSql(wxTreeCtrl *browser)
         }
 
         if (GetHasOids())
-            sql += wxT("WITH OIDS;\n");
+            sql += wxT("\nWITH OIDS");
         else
-            sql += wxT("WITHOUT OIDS;\n");
+            sql += wxT("\nWITHOUT OIDS");
 
-        sql += GetGrant(wxT("arwdRxt")) 
-            + GetCommentSql()
-            + wxT("\n\n");
+        AppendIfFilled(sql, wxT(" TABLESPACE "), qtIdent(tablespace));
+
+        sql += wxT(";\n")
+            + GetOwnerSql(7, 3)
+            + GetGrant(wxT("arwdRxt")) 
+            + GetCommentSql();
+
+		// Column comments
+		sql += colDetails;
 
         // add indexes here
 
         // add triggers here 
 
+		if (sql.Length() > 0)
+			if (sql.Last() != '\n') sql += wxT("\n");
 
     }
     return sql;
 }
 
 
+wxString pgTable::GetCoveringIndex(wxTreeCtrl *browser, const wxString &collist)
+{
+    // delivers the name of the index which covers the named columns
+    wxCookieType cookie;
+
+    wxTreeItemId collItem=browser->GetFirstChild(GetId(), cookie);
+    while (collItem)
+    {
+        pgObject *data=(pgObject*)browser->GetItemData(collItem);
+        if (data && (data->GetType() == PG_CONSTRAINTS || data->GetType() == PG_INDEXES))
+        {
+            wxCookieType cookie2;
+            wxTreeItemId item=browser->GetFirstChild(collItem, cookie2);
+            while (item)
+            {
+                pgIndex *index=(pgIndex*)browser->GetItemData(item);
+                if (index && (index->GetType() == PG_INDEX || index->GetType() == PG_PRIMARYKEY || index->GetType() == PG_UNIQUE))
+                {
+                    index->ShowTreeDetail(browser);
+                    if (collist == index->GetColumns() || 
+                        collist + wxT(",") == index->GetColumns().Left(collist.Length()+1))
+                        return index->GetName();
+                }
+                item=browser->GetNextChild(collItem, cookie2);
+            }
+        }
+        collItem=browser->GetNextChild(GetId(), cookie);
+    }
+
+    return wxEmptyString;
+}
+
+
 void pgTable::UpdateRows()
 {
     pgSet *props = ExecuteSet(wxT("SELECT count(*) AS rows FROM ") + GetQuotedFullIdentifier());
     if (props)
     {
-        rows = StrToLong(props->GetVal(0));
+        rows = props->GetLongLong(0);
         delete props;
+        rowsCounted = true;
     }
 }
 
@@ -234,12 +285,15 @@ void pgTable::UpdateInheritance()
         while (!props->Eof())
         {
             if (inheritedTableCount)
+            {
                 inheritedTables += wxT(", ");
+                quotedInheritedTables += wxT(", ");
+            }
             inheritedTables += props->GetVal(wxT("relname"));
-            quotedInheritedTables += qtIdent(props->GetVal(wxT("nspname")))
-                    +wxT(".")+qtIdent(props->GetVal(wxT("relname")));
-            quotedInheritedTablesList.Add(qtIdent(props->GetVal(wxT("nspname")))
-                    +wxT(".")+qtIdent(props->GetVal(wxT("relname"))));
+            quotedInheritedTables += GetQuotedSchemaPrefix(props->GetVal(wxT("nspname")))
+                    + qtIdent(props->GetVal(wxT("relname")));
+            quotedInheritedTablesList.Add(GetQuotedSchemaPrefix(props->GetVal(wxT("nspname")))
+                    + qtIdent(props->GetVal(wxT("relname"))));
             props->MoveNext();
             inheritedTableCount++;
         }
@@ -251,7 +305,7 @@ void pgTable::UpdateInheritance()
 
 
 
-void pgTable::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, wxListCtrl *properties, wxListCtrl *statistics, ctlSQLBox *sqlPane)
+void pgTable::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, ctlListView *properties, ctlSQLBox *sqlPane)
 {
     if (!expandedKids)
     {
@@ -322,34 +376,39 @@ void pgTable::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, wxListCtrl *pro
     if (properties)
     {
         CreateListColumns(properties);
-        int pos=0;
 
-        InsertListItem(properties, pos++, _("Name"), GetName());
-        InsertListItem(properties, pos++, _("OID"), GetOid());
-        InsertListItem(properties, pos++, _("Owner"), GetOwner());
-        InsertListItem(properties, pos++, _("ACL"), GetAcl());
+        properties->AppendItem(_("Name"), GetName());
+        properties->AppendItem(_("OID"), GetOid());
+        properties->AppendItem(_("Owner"), GetOwner());
+        if (!tablespace.IsEmpty())
+            properties->AppendItem(_("Tablespace"), tablespace);
+        properties->AppendItem(_("ACL"), GetAcl());
         if (GetPrimaryKey().IsNull())
-            InsertListItem(properties, pos++, _("Primary key"), _("<none>"));
+            properties->AppendItem(_("Primary key"), _("<no primary key>"));
         else
-            InsertListItem(properties, pos++, _("Primary key"), GetPrimaryKey());
+            properties->AppendItem(_("Primary key"), GetPrimaryKey());
 
-        InsertListItem(properties, pos++, _("Rows (estimated)"), GetEstimatedRows());
+        properties->AppendItem(_("Rows (estimated)"), GetEstimatedRows());
 
-        if (rows < 0)
-            InsertListItem(properties, pos++, _("Rows (counted)"), _("Refresh to count rows"));
+        if (rowsCounted)
+            properties->AppendItem(_("Rows (counted)"), rows);
         else
-            InsertListItem(properties, pos++, _("Rows (counted)"), rows);
+            properties->AppendItem(_("Rows (counted)"), _("not counted"));
 
-        InsertListItem(properties, pos++, _("Inherits tables"), GetHasSubclass());
-        InsertListItem(properties, pos++, _("Inherited tables count"), GetInheritedTableCount());
+        properties->AppendItem(_("Inherits tables"), GetHasSubclass());
+        properties->AppendItem(_("Inherited tables count"), GetInheritedTableCount());
         if (GetInheritedTableCount())
-            InsertListItem(properties, pos++, _("Inherited tables"), GetInheritedTables());
-        InsertListItem(properties, pos++, _("Has OIDs?"), GetHasOids());
-        InsertListItem(properties, pos++, _("System table?"), GetSystemObject());
-        InsertListItem(properties, pos++, _("Comment"), GetComment());
+            properties->AppendItem(_("Inherited tables"), GetInheritedTables());
+        properties->AppendItem(_("Has OIDs?"), GetHasOids());
+        properties->AppendItem(_("System table?"), GetSystemObject());
+        properties->AppendItem(_("Comment"), GetComment());
     }
+}
 
-    DisplayStatistics(statistics, 
+
+void pgTable::ShowStatistics(frmMain *form, ctlListView *statistics)
+{
+    wxString sql =
         wxT("SELECT seq_scan AS ") + qtIdent(_("Sequential Scans")) +
              wxT(", seq_tup_read AS ") + qtIdent(_("Sequential Tuples Read")) +
              wxT(", idx_scan AS ") + qtIdent(_("Index Scans")) +
@@ -364,12 +423,18 @@ void pgTable::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, wxListCtrl *pro
              wxT(", toast_blks_read AS ") + qtIdent(_("Toast Blocks Read")) +
              wxT(", toast_blks_hit AS ") + qtIdent(_("Toast Blocks Hit")) +
              wxT(", tidx_blks_read AS ") + qtIdent(_("Toast Index Blocks Read")) +
-             wxT(", tidx_blks_hit AS ") + qtIdent(_("Toast Index Blocks Hit")) + wxT("\n")
+             wxT(", tidx_blks_hit AS ") + qtIdent(_("Toast Index Blocks Hit"));
+    
+    if (GetConnection()->HasFeature(FEATURE_SIZE))
+        sql += wxT(", pg_size_pretty(pg_relation_size(stat.relid)) AS ") + qtIdent(_("Table Size"));
+
+    sql +=  wxT("\n")
         wxT("  FROM pg_stat_all_tables stat, pg_statio_all_tables statio\n")
         wxT(" WHERE stat.relid = statio.relid\n")
-        wxT("   AND stat.relid = ") + GetOidStr());
-}
+        wxT("   AND stat.relid = ") + GetOidStr();
 
+    DisplayStatistics(statistics, sql);
+}
 
 
 
@@ -381,11 +446,7 @@ pgObject *pgTable::Refresh(wxTreeCtrl *browser, const wxTreeItemId item)
     {
         pgObject *obj=(pgObject*)browser->GetItemData(parentItem);
         if (obj->GetType() == PG_TABLES)
-        {
             table = (pgTable*)ReadObjects((pgCollection*)obj, 0, wxT("\n   AND rel.oid=") + GetOidStr());
-            if (table && table->GetRows() < 0)
-                table->UpdateRows();
-        }
     }
     return table;
 }
@@ -396,16 +457,32 @@ pgObject *pgTable::ReadObjects(pgCollection *collection, wxTreeCtrl *browser, co
 {
     pgTable *table=0;
 
-    pgSet *tables= collection->GetDatabase()->ExecuteSet(
-        wxT("SELECT rel.oid, relname, pg_get_userbyid(relowner) AS relowner, relacl, relhasoids, ")
-                wxT("relhassubclass, reltuples, description, conname, conkey\n")
-        wxT("  FROM pg_class rel\n")
-        wxT("  LEFT OUTER JOIN pg_description des ON des.objoid=rel.oid AND des.objsubid=0\n")
-        wxT("  LEFT OUTER JOIN pg_constraint c ON c.conrelid=rel.oid AND c.contype='p'\n")
-        wxT(" WHERE ((relkind = 'r') OR (relkind = 's')) AND relnamespace = ") + collection->GetSchema()->GetOidStr() + wxT("\n")
-        + restriction + 
-        wxT(" ORDER BY relname"));
-
+    pgSet *tables;
+    if (collection->GetConnection()->BackendMinimumVersion(7, 5))
+    {
+        tables= collection->GetDatabase()->ExecuteSet(
+            wxT("SELECT rel.oid, relname, spcname, pg_get_userbyid(relowner) AS relowner, relacl, relhasoids, ")
+                    wxT("relhassubclass, reltuples, description, conname, conkey\n")
+            wxT("  FROM pg_class rel\n")
+            wxT("  LEFT OUTER JOIN pg_tablespace ta on ta.oid=rel.reltablespace\n")
+            wxT("  LEFT OUTER JOIN pg_description des ON des.objoid=rel.oid AND des.objsubid=0\n")
+            wxT("  LEFT OUTER JOIN pg_constraint c ON c.conrelid=rel.oid AND c.contype='p'\n")
+            wxT(" WHERE ((relkind = 'r') OR (relkind = 's')) AND relnamespace = ") + collection->GetSchema()->GetOidStr() + wxT("\n")
+            + restriction + 
+            wxT(" ORDER BY relname"));
+    }
+    else
+    {
+        tables= collection->GetDatabase()->ExecuteSet(
+            wxT("SELECT rel.oid, relname, pg_get_userbyid(relowner) AS relowner, relacl, relhasoids, ")
+                    wxT("relhassubclass, reltuples, description, conname, conkey\n")
+            wxT("  FROM pg_class rel\n")
+            wxT("  LEFT OUTER JOIN pg_description des ON des.objoid=rel.oid AND des.objsubid=0\n")
+            wxT("  LEFT OUTER JOIN pg_constraint c ON c.conrelid=rel.oid AND c.contype='p'\n")
+            wxT(" WHERE ((relkind = 'r') OR (relkind = 's')) AND relnamespace = ") + collection->GetSchema()->GetOidStr() + wxT("\n")
+            + restriction + 
+            wxT(" ORDER BY relname"));
+    }
     if (tables)
     {
         while (!tables->Eof())
@@ -415,9 +492,11 @@ pgObject *pgTable::ReadObjects(pgCollection *collection, wxTreeCtrl *browser, co
             table->iSetOid(tables->GetOid(wxT("oid")));
             table->iSetOwner(tables->GetVal(wxT("relowner")));
             table->iSetAcl(tables->GetVal(wxT("relacl")));
+            if (collection->GetConnection()->BackendMinimumVersion(7, 5))
+                table->iSetTablespace(tables->GetVal(wxT("spcname")));
             table->iSetComment(tables->GetVal(wxT("description")));
             table->iSetHasOids(tables->GetBool(wxT("relhasoids")));
-            table->iSetEstimatedRows(tables->GetLong(wxT("reltuples")));
+            table->iSetEstimatedRows(tables->GetDouble(wxT("reltuples")));
             table->iSetHasSubclass(tables->GetBool(wxT("relhassubclass")));
             table->iSetPrimaryKeyName(tables->GetVal(wxT("conname")));
             wxString cn=tables->GetVal(wxT("conkey"));
@@ -439,19 +518,30 @@ pgObject *pgTable::ReadObjects(pgCollection *collection, wxTreeCtrl *browser, co
 }
 
 
-void pgTable::ShowStatistics(pgCollection *collection, wxListCtrl *statistics)
+void pgTable::ShowStatistics(pgCollection *collection, ctlListView *statistics)
 {
     wxLogInfo(wxT("Displaying statistics for tables on ")+ collection->GetSchema()->GetIdentifier());
 
+    bool hasSize=collection->GetConnection()->HasFeature(FEATURE_SIZE);
+
     // Add the statistics view columns
     statistics->ClearAll();
-    statistics->InsertColumn(0, _("Table"), wxLIST_FORMAT_LEFT, 150);
-    statistics->InsertColumn(1, _("Tuples inserted"), wxLIST_FORMAT_LEFT, 80);
-    statistics->InsertColumn(2, _("Tuples updated"), wxLIST_FORMAT_LEFT, 80);
-    statistics->InsertColumn(3, _("Tuples deleted"), wxLIST_FORMAT_LEFT, 80);
+    statistics->AddColumn(_("Table"), 100);
+    statistics->AddColumn(_("Tuples inserted"), 50);
+    statistics->AddColumn(_("Tuples updated"), 50);
+    statistics->AddColumn(_("Tuples deleted"), 50);
+    if (hasSize)
+        statistics->AddColumn(_("Size"), 60);
 
-    pgSet *stats = collection->GetDatabase()->ExecuteSet(wxT(
-        "SELECT relname, n_tup_ins, n_tup_upd, n_tup_del FROM pg_stat_all_tables ORDER BY relname"));
+    wxString sql=wxT("SELECT relname, n_tup_ins, n_tup_upd, n_tup_del");
+    if (hasSize)
+        sql += wxT(", pg_size_pretty(pg_relation_size(relid)) AS size");
+    
+    sql += wxT("\n  FROM pg_stat_all_tables")
+	  wxT("\n WHERE schemaname = ") + qtString(collection->GetSchema()->GetName())
+	    +  wxT("\n ORDER BY relname");
+
+    pgSet *stats = collection->GetDatabase()->ExecuteSet(sql);
 
     if (stats)
     {
@@ -462,6 +552,8 @@ void pgTable::ShowStatistics(pgCollection *collection, wxListCtrl *statistics)
             statistics->SetItem(pos, 1, stats->GetVal(wxT("n_tup_ins")));
             statistics->SetItem(pos, 2, stats->GetVal(wxT("n_tup_upd")));
             statistics->SetItem(pos, 3, stats->GetVal(wxT("n_tup_del")));
+            if (hasSize)
+                statistics->SetItem(pos, 4, stats->GetVal(wxT("size")));
             stats->MoveNext();
             pos++;
         }
