@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: pgTrigger.cpp,v 1.25 2004/01/02 23:15:00 dpage Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // pgTrigger.cpp - Trigger class
@@ -110,7 +111,7 @@ wxString pgTrigger::GetForEach() const
 
 
 
-void pgTrigger::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, wxListCtrl *properties, wxListCtrl *statistics, ctlSQLBox *sqlPane)
+void pgTrigger::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, ctlListView *properties, ctlSQLBox *sqlPane)
 {
     if (!expandedKids)
     {
@@ -134,17 +135,16 @@ void pgTrigger::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, wxListCtrl *p
     if (properties)
     {
         CreateListColumns(properties);
-        int pos=0;
 
-        InsertListItem(properties, pos++, _("Name"), GetName());
-        InsertListItem(properties, pos++, _("OID"), GetOid());
-        InsertListItem(properties, pos++, _("Fires"), GetFireWhen());
-        InsertListItem(properties, pos++, _("Event"), GetEvent());
-        InsertListItem(properties, pos++, _("For each"), GetForEach());
-        InsertListItem(properties, pos++, _("Function"), GetFunction() + wxT("(") + GetArguments() + wxT(")"));
-        InsertListItem(properties, pos++, _("Enabled?"), GetEnabled());
-        InsertListItem(properties, pos++, _("System trigger?"), GetSystemObject());
-        InsertListItem(properties, pos++, _("Comment"), GetComment());
+        properties->AppendItem(_("Name"), GetName());
+        properties->AppendItem(_("OID"), GetOid());
+        properties->AppendItem(_("Fires"), GetFireWhen());
+        properties->AppendItem(_("Event"), GetEvent());
+        properties->AppendItem(_("For each"), GetForEach());
+        properties->AppendItem(_("Function"), GetFunction() + wxT("(") + GetArguments() + wxT(")"));
+        properties->AppendItem(_("Enabled?"), GetEnabled());
+        properties->AppendItem(_("System trigger?"), GetSystemObject());
+        properties->AppendItem(_("Comment"), GetComment());
     }
 }
 
@@ -195,7 +195,7 @@ pgObject *pgTrigger::ReadObjects(pgCollection *collection, wxTreeCtrl *browser,
             trigger->iSetFunctionOid(triggers->GetOid(wxT("tgfoid")));
             trigger->iSetEnabled(triggers->GetBool(wxT("tgenabled")));
             trigger->iSetTriggerType(triggers->GetLong(wxT("tgtype")));
-            trigger->iSetQuotedFullTable(qtIdent(triggers->GetVal(wxT("nspname")))+wxT(".")+qtIdent(triggers->GetVal(wxT("relname"))));
+            trigger->iSetQuotedFullTable(collection->GetDatabase()->GetQuotedSchemaPrefix(triggers->GetVal(wxT("nspname"))) + qtIdent(triggers->GetVal(wxT("relname"))));
             wxString arglist=triggers->GetVal(wxT("tgargs"));
             wxString args;
 
