@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: pgType.cpp,v 1.24 2004/10/08 13:52:19 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // pgType.cpp - Type class
@@ -61,6 +62,7 @@ wxString pgType::GetSql(wxTreeCtrl *browser)
                 + wxT(", STORAGE=") + GetStorage());
         }
         sql += wxT(");\n")
+            + GetOwnerSql(8, 0)
             + GetCommentSql();
     }
 
@@ -69,7 +71,7 @@ wxString pgType::GetSql(wxTreeCtrl *browser)
 
 
 
-void pgType::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, wxListCtrl *properties, wxListCtrl *statistics, ctlSQLBox *sqlPane)
+void pgType::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, ctlListView *properties, ctlSQLBox *sqlPane)
 {
     if (!expandedKids)
     {
@@ -77,7 +79,8 @@ void pgType::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, wxListCtrl *prop
         if (isComposite)
         {
             pgSet *set=ExecuteSet(
-                wxT("SELECT attname, t.typname, attndims, atttypmod, nspname\n")
+                wxT("SELECT attname, t.typname, attndims, atttypmod, nspname,\n")
+                wxT("       (SELECT COUNT(1) from pg_type t2 WHERE t2.typname=t.typname) > 1 AS isdup\n")
                 wxT("  FROM pg_attribute att\n")
                 wxT("  JOIN pg_type t ON t.oid=atttypid\n")
                 wxT("  JOIN pg_namespace nsp ON t.typnamespace=nsp.oid\n")
@@ -89,24 +92,24 @@ void pgType::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, wxListCtrl *prop
                 int anzvar=0;
                 while (!set->Eof())
                 {
+                    wxString element;
                     if (anzvar++)
                     {
                         typesList += wxT(", ");
                         quotedTypesList += wxT(",\n    ");
                     }
                     typesList += set->GetVal(wxT("attname")) + wxT(" ");
+                    typesArray.Add(set->GetVal(wxT("attname")));
                     quotedTypesList += qtIdent(set->GetVal(wxT("attname"))) + wxT(" ");
 
-                    pgDatatype dt(set->GetVal(wxT("typname")), set->GetLong(wxT("attndims")) > 0, set->GetLong(wxT("atttypmod")));
-                    wxString nspname=set->GetVal(wxT("nspname"));
-                    if (!nspname.IsSameAs(wxT("pg_catalog")))
-                    {
-                        typesList += nspname + wxT(".");
-                        quotedTypesList += qtIdent(nspname) + wxT(".");
-                    }
+                    pgDatatype dt(set->GetVal(wxT("nspname")), set->GetVal(wxT("typname")),
+                        set->GetBool(wxT("isdup")), set->GetLong(wxT("attndims")) > 0, set->GetLong(wxT("atttypmod")));
 
-                    typesList +=  dt.FullName();
-                    quotedTypesList += dt.QuotedFullName();
+                    wxString nspname=set->GetVal(wxT("nspname"));
+
+                    typesList += dt.GetSchemaPrefix(GetDatabase()) + dt.FullName();
+                    typesArray.Add(dt.GetSchemaPrefix(GetDatabase()) + dt.FullName());
+                    quotedTypesList += dt.GetQuotedSchemaPrefix(GetDatabase()) + dt.QuotedFullName();
 
                     set->MoveNext();
                 }
@@ -119,37 +122,36 @@ void pgType::ShowTreeDetail(wxTreeCtrl *browser, frmMain *form, wxListCtrl *prop
         wxLogInfo(wxT("Displaying properties for type %s"), GetIdentifier().c_str());
 
         CreateListColumns(properties);
-        int pos=0;
 
-        InsertListItem(properties, pos++, _("Name"), GetName());
-        InsertListItem(properties, pos++, _("OID"), GetOid());
-        InsertListItem(properties, pos++, _("Owner"), GetOwner());
+        properties->AppendItem(_("Name"), GetName());
+        properties->AppendItem(_("OID"), GetOid());
+        properties->AppendItem(_("Owner"), GetOwner());
         if (isComposite)
         {
-            InsertListItem(properties, pos++, _("Members"), GetTypesList());
+            properties->AppendItem(_("Members"), GetTypesList());
         }
         else
         {
-            InsertListItem(properties, pos++, _("Alignment"), GetAlignment());
-            InsertListItem(properties, pos++, _("Internal length"), GetInternalLength());
-            InsertListItem(properties, pos++, _("Default"), GetDefault());
-            InsertListItem(properties, pos++, _("Passed by Value?"), BoolToYesNo(GetPassedByValue()));
+            properties->AppendItem(_("Alignment"), GetAlignment());
+            properties->AppendItem(_("Internal length"), GetInternalLength());
+            properties->AppendItem(_("Default"), GetDefault());
+            properties->AppendItem(_("Passed by Value?"), BoolToYesNo(GetPassedByValue()));
             if (!GetElement().IsEmpty())
             {
-                InsertListItem(properties, pos++, _("Element"), GetElement());
-                InsertListItem(properties, pos++, _("Delimiter"), GetDelimiter());
+                properties->AppendItem(_("Element"), GetElement());
+                properties->AppendItem(_("Delimiter"), GetDelimiter());
             }
-            InsertListItem(properties, pos++, _("Input function"), GetInputFunction());
-            InsertListItem(properties, pos++, _("Output function"), GetOutputFunction());
+            properties->AppendItem(_("Input function"), GetInputFunction());
+            properties->AppendItem(_("Output function"), GetOutputFunction());
             if (GetConnection()->BackendMinimumVersion(7, 4))
             {
-            InsertListItem(properties, pos++, _("Receive function"), GetReceiveFunction());
-            InsertListItem(properties, pos++, _("Send function"), GetSendFunction());
+            properties->AppendItem(_("Receive function"), GetReceiveFunction());
+            properties->AppendItem(_("Send function"), GetSendFunction());
             }
-            InsertListItem(properties, pos++, _("Storage"), GetStorage());
+            properties->AppendItem(_("Storage"), GetStorage());
         }
-        InsertListItem(properties, pos++, _("System type?"), GetSystemObject());
-        InsertListItem(properties, pos++, _("Comment"), GetComment());
+        properties->AppendItem(_("System type?"), GetSystemObject());
+        properties->AppendItem(_("Comment"), GetComment());
     }
 }
 
