@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: ctlSQLResult.cpp,v 1.23 2004/06/17 17:47:02 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // ctlSQLResult.cpp - SQL Query result window
@@ -33,18 +34,77 @@ ctlSQLResult::~ctlSQLResult()
 
 bool ctlSQLResult::Export()
 {
-    if (!CanExport())
-        return false;
+    if (rowsRetrieved>0 || (thread && thread->DataSet()->NumRows() > 0))
+    {
+        frmExport dlg(this);
+        if (dlg.ShowModal() > 0)
+        {
+            if (rowsRetrieved> 0)
+                return dlg.Export(this);
+            else
+                return dlg.Export(thread->DataSet());
+        }
+    }
+    return false;
+}
+
+
+
+wxString ctlSQLResult::GetExportLine(int row)
+{
+    if (GetColumnCount() <= 1)
+        return GetItemText(row);
+
+    wxString str;
+    int col;
+    for (col=1 ; col < GetColumnCount() ; col++)
+    {
+        if (col > 1)
+            str.Append(settings->GetExportColSeparator());
+
+        wxString text=GetItemText(row, col);
+
+        bool needQuote=(settings->GetExportQuoting() > 1);
+
     
-    frmExport dlg(this);
-    return dlg.ShowModal() > 0;
+        switch (colTypClasses.Item(col))
+        {
+            case PGTYPCLASS_NUMERIC:
+            case PGTYPCLASS_BOOL:
+                break;
+            default:
+                needQuote=true;
+                break;
+        }
+        if (needQuote)
+            str.Append(settings->GetExportQuoteChar());
+        str.Append(text);
+        if (needQuote)
+            str.Append(settings->GetExportQuoteChar());
+    }    
+    return str;
 }
 
 
 int ctlSQLResult::Execute(const wxString &query, int resultToRetrieve)
 {
     Abort();
+
+    colSizes.Empty();
+    colHeaders.Empty();
+    int i;
+    wxListItem item;
+    item.SetMask(wxLIST_MASK_TEXT|wxLIST_MASK_WIDTH);
+
+    for (i=0 ; i < GetColumnCount() ; i++)
+    {
+        GetColumn(i, item);
+        colHeaders.Add(item.GetText());
+        colSizes.Add(item.GetWidth());
+    }
+
     ClearAll();
+
     rowsRetrieved=0;
     colNames.Empty();
     colTypes.Empty();
@@ -86,11 +146,16 @@ int ctlSQLResult::RetrieveOne()
     if (!rowsRetrieved)
     {
         int w, h;
-        GetSize(&w, &h);
+        if (colSizes.GetCount() == 1)
+            w = colSizes.Item(0);
+        else
+            GetSize(&w, &h);
+
         colNames.Add(thread->DataSet()->ColName(0));
         colTypes.Add(wxT(""));
         colTypClasses.Add(0L);
 
+
         InsertColumn(0, thread->DataSet()->ColName(0), wxLIST_FORMAT_LEFT, w);
 
         while (!thread->DataSet()->Eof())
@@ -124,6 +189,8 @@ int ctlSQLResult::Retrieve(long chunk)
         InsertColumn(0, _("Row"), wxLIST_FORMAT_RIGHT, 30);
         colNames.Add(wxT("Row"));
 
+        size_t hdrIndex=0;
+
         for (col=0 ; col < nCols ; col++)
         {
             colName = thread->DataSet()->ColName(col);
@@ -132,7 +199,23 @@ int ctlSQLResult::Retrieve(long chunk)
             colTypes.Add(colType);
             colTypClasses.Add(thread->DataSet()->ColTypClass(col));
 
-            InsertColumn(col+1, colName +wxT(" (")+ colType +wxT(")"), wxLIST_FORMAT_LEFT, -1);
+            wxString colHeader=colName +wxT(" (")+ colType +wxT(")");
+
+            int w;
+            if (hdrIndex < colHeaders.GetCount() && colHeaders.Item(hdrIndex) == colHeader)
+                w = colSizes.Item(hdrIndex++);
+            else
+            {
+                if (hdrIndex+1 < colHeaders.GetCount() && colHeaders.Item(hdrIndex+1) == colHeader)
+                {
+                    hdrIndex++;
+                    w = colSizes.Item(hdrIndex++);
+                }
+                else
+                    w=-1;
+            }
+
+            InsertColumn(col+1, colHeader, wxLIST_FORMAT_LEFT, w);
         }
     }
 
@@ -164,6 +247,20 @@ int ctlSQLResult::Retrieve(long chunk)
 }
 
 
+wxString ctlSQLResult::GetItemText(int row, int col)
+{
+    if (col < 0)
+        return wxListCtrl::GetItemText(row);
+
+    wxListItem item;
+    item.m_mask=wxLIST_MASK_TEXT;
+    item.m_itemId=row;
+    item.m_col=col;
+    GetItem(item);
+    return item.GetText();
+}
+
+
 wxString ctlSQLResult::GetMessagesAndClear()
 {
     if (thread)
@@ -174,13 +271,12 @@ wxString ctlSQLResult::GetMessagesAndClear()
 
 wxString ctlSQLResult::GetErrorMessage()
 {
-    wxString errmsg=wxString(PQerrorMessage(conn->connection()), wxConvUTF8);
-    return errmsg;
+    return conn->GetLastError();
 }
 
 
 
-long ctlSQLResult::NumRows()
+long ctlSQLResult::NumRows() const
 {
     if (thread && thread->DataValid())
         return thread->DataSet()->NumRows();
@@ -188,6 +284,22 @@ long ctlSQLResult::NumRows()
 }
 
 
+long ctlSQLResult::InsertedCount() const
+{
+    if (thread)
+        return thread->RowsInserted();
+    return -1;
+}
+
+
+OID ctlSQLResult::InsertedOid() const
+{
+    if (thread)
+        return thread->InsertedOid();
+    return (OID)-1;
+}
+
+
 int ctlSQLResult::RunStatus()
 {
     if (!thread)
