@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: dlgIndex.cpp,v 1.25 2004/11/03 19:39:14 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // dlgIndex.cpp - PostgreSQL Index Property
@@ -24,18 +25,20 @@
 #include "images/index.xpm"
 
 
-#define cbType          CTRL("cbType", wxComboBox)
-#define chkUnique       CTRL("chkUnique", wxCheckBox)
-#define txtWhere        CTRL("txtWhere", wxTextCtrl)
+#define cbTablespace    CTRL_COMBOBOX("cbTablespace")
+#define cbType          CTRL_COMBOBOX("cbType")
+#define chkUnique       CTRL_CHECKBOX("chkUnique")
+#define chkClustered    CTRL_CHECKBOX("chkClustered")
+#define txtWhere        CTRL_TEXT("txtWhere")
 
-#define btnAddCol       CTRL("btnAddCol", wxButton)
-#define btnRemoveCol    CTRL("btnRemoveCol", wxButton)
+#define btnAddCol       CTRL_BUTTON("btnAddCol")
+#define btnRemoveCol    CTRL_BUTTON("btnRemoveCol")
 
 
 
 BEGIN_EVENT_TABLE(dlgIndexBase, dlgCollistProperty)
-    EVT_TEXT(XRCID("txtName"),                      dlgIndexBase::OnChange)
-    EVT_TEXT(XRCID("txtComment"),                   dlgIndexBase::OnChange)
+    EVT_TEXT(XRCID("cbTablespace"),                 dlgProperty::OnChange)
+    EVT_COMBOBOX(XRCID("cbTablespace"),             dlgProperty::OnChange)
     EVT_BUTTON(XRCID("btnAddCol"),                  dlgIndexBase::OnAddCol)
     EVT_BUTTON(XRCID("btnRemoveCol"),               dlgIndexBase::OnRemoveCol)
 END_EVENT_TABLE();
@@ -48,17 +51,17 @@ dlgIndexBase::dlgIndexBase(frmMain *frame, const wxString &resName, pgIndex *nod
     index=node;
     wxASSERT(!table || table->GetType() == PG_TABLE);
 
-    CreateListColumns(lstColumns, _("Columns"), wxT(""), 0);
+    lstColumns->CreateColumns(frame, _("Columns"), wxT(""), 0);
 }
 
 
-dlgIndexBase::dlgIndexBase(frmMain *frame, const wxString &resName, wxListCtrl *colList)
+dlgIndexBase::dlgIndexBase(frmMain *frame, const wxString &resName, ctlListView *colList)
 : dlgCollistProperty(frame, resName, colList)
 {
     SetIcon(wxIcon(index_xpm));
     index=0;
     
-    CreateListColumns(lstColumns, _("Columns"), wxT(""), 0);
+    lstColumns->CreateColumns(frame, _("Columns"), wxT(""), 0);
 }
 
 
@@ -73,20 +76,17 @@ int dlgIndexBase::Go(bool modal)
     if (index)
     {
         // edit mode: view only
-        txtName->SetValue(index->GetName());
         txtName->Disable();
-
         btnAddCol->Disable();
         btnRemoveCol->Disable();
         cbColumns->Disable();
-	txtComment->SetValue(index->GetComment());
 
-        int pos=0;
+	    int pos = 0;
         wxStringTokenizer cols(index->GetColumns(), wxT(","));
         while (cols.HasMoreTokens())
         {
             wxString str=cols.GetNextToken();
-            if (pos)
+            if (str.Strip() == wxT(""))
                 str.RemoveLast();       // there's a space
             lstColumns->InsertItem(pos++, str, PGICON_COLUMN);
         }
@@ -100,7 +100,7 @@ int dlgIndexBase::Go(bool modal)
 }
 
 
-void dlgIndexBase::OnAddCol(wxNotifyEvent &ev)
+void dlgIndexBase::OnAddCol(wxCommandEvent &ev)
 {
     wxString col=cbColumns->GetValue();
     if (!col.IsEmpty())
@@ -110,32 +110,30 @@ void dlgIndexBase::OnAddCol(wxNotifyEvent &ev)
         if (cbColumns->GetCount())
             cbColumns->SetSelection(0);
 
-        wxNotifyEvent event;
-        OnChange(event);
+        CheckChange();
     }
 }
 
 
-void dlgIndexBase::OnRemoveCol(wxNotifyEvent &ev)
+void dlgIndexBase::OnRemoveCol(wxCommandEvent &ev)
 {
-    long pos=lstColumns->GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
+    long pos=lstColumns->GetSelection();
     if (pos >= 0)
     {
         wxString col=lstColumns->GetItemText(pos);
         lstColumns->DeleteItem(pos);
         cbColumns->Append(col);
 
-        wxNotifyEvent event;
-        OnChange(event);
+        CheckChange();
     }
 }
 
 
-void dlgIndexBase::OnChange(wxNotifyEvent &ev)
+void dlgIndexBase::CheckChange()
 {
     if (index)
     {
-        btnOK->Enable(txtComment->GetValue() != index->GetComment());
+        EnableOK(txtComment->GetValue() != index->GetComment());
     }
     else
     {
@@ -165,7 +163,8 @@ wxString dlgIndexBase::GetColumns()
 
 
 BEGIN_EVENT_TABLE(dlgIndex, dlgIndexBase)
-    EVT_TEXT(XRCID("txtName"),                      dlgIndex::OnChange)
+    EVT_TEXT(XRCID("cbTablespace"),                 dlgProperty::OnChange)
+    EVT_CHECKBOX(XRCID("chkClustered"),             dlgProperty::OnChange)
 END_EVENT_TABLE();
 
         
@@ -175,11 +174,11 @@ dlgIndex::dlgIndex(frmMain *frame, pgIndex *index, pgTable *parentNode)
 }
 
 
-void dlgIndex::OnChange(wxNotifyEvent &ev)
+void dlgIndex::CheckChange()
 {
     if (index)
     {
-        btnOK->Enable(txtComment->GetValue() != index->GetComment());
+        EnableOK(txtComment->GetValue() != index->GetComment() || chkClustered->GetValue() != index->GetIsClustered());
     }
     else
     {
@@ -195,21 +194,27 @@ void dlgIndex::OnChange(wxNotifyEvent &ev)
 
 int dlgIndex::Go(bool modal)
 {
+    if (!connection->BackendMinimumVersion(7, 4))
+        chkClustered->Disable();
+
     if (index)
     {
         // edit mode: view only
         
         cbType->Append(index->GetIndexType());
         chkUnique->SetValue(index->GetIsUnique());
+        chkClustered->SetValue(index->GetIsClustered());
         txtWhere->SetValue(index->GetConstraint());
         cbType->SetSelection(0);
         cbType->Disable();
         txtWhere->Disable();
         chkUnique->Disable();
+        PrepareTablespace(cbTablespace, index->GetTablespace());
     }
     else
     {
         // create mode
+        PrepareTablespace(cbTablespace);
         cbType->Append(wxT(""));
         pgSet *set=connection->ExecuteSet(wxT(
             "SELECT amname FROM pg_am"));
@@ -234,21 +239,31 @@ wxString dlgIndex::GetSql()
 
     if (table)
     {
+        wxString name=GetName();
         if (!index)
         {
             sql = wxT("CREATE ");
             if (chkUnique->GetValue())
                 sql += wxT("UNIQUE ");
-            sql += wxT("INDEX ") + qtIdent(GetName());
+            sql += wxT("INDEX ") + qtIdent(name);
             sql += wxT("\n   ON ") + table->GetQuotedFullIdentifier();
             AppendIfFilled(sql, wxT(" USING "), cbType->GetValue());
             sql += wxT(" (") + GetColumns()
                 + wxT(")");
 
-
+            AppendIfFilled(sql, wxT("\n       TABLESPACE "), qtIdent(cbTablespace->GetValue()));
             AppendIfFilled(sql, wxT(" WHERE "), txtWhere->GetValue());
             sql +=  wxT(";\n");
         }
+        if (connection->BackendMinimumVersion(7, 4))
+        {
+            if (index && index->GetIsClustered() && !chkClustered->GetValue())
+                sql += wxT("ALTER TABLE ") + table->GetQuotedFullIdentifier()
+                    +  wxT(" SET WITHOUT CLUSTER;\n");
+            else if (chkClustered->GetValue() && (!index || !index->GetIsClustered()))
+                sql += wxT("ALTER TABLE ") + table->GetQuotedFullIdentifier()
+                    +  wxT(" CLUSTER ON ") + qtIdent(name) + wxT(";\n");
+        }
         AppendComment(sql, wxT("INDEX"), table->GetSchema(), index);
     }
     return sql;
