@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: dlgIndexConstraint.cpp,v 1.12 2004/08/09 16:57:03 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // dlgIndexConstraint.cpp - PostgreSQL IndexConstraint Property
@@ -23,9 +24,10 @@
 
 
 
-#define chkDeferrable   CTRL("chkDeferrable",   wxCheckBox)
-#define chkDeferred     CTRL("chkDeferred",     wxCheckBox)
-#define stDeferred      CTRL("stDeferred",      wxStaticText)
+#define cbTablespace    CTRL_COMBOBOX("cbTablespace")
+#define chkDeferrable   CTRL_CHECKBOX("chkDeferrable")
+#define chkDeferred     CTRL_CHECKBOX("chkDeferred")
+#define stDeferred      CTRL_STATIC("stDeferred")
 
 
 
@@ -41,7 +43,7 @@ dlgIndexConstraint::dlgIndexConstraint(frmMain *frame, const wxString &resName,
 }
 
 
-dlgIndexConstraint::dlgIndexConstraint(frmMain *frame, const wxString &resName, wxListCtrl *colList)
+dlgIndexConstraint::dlgIndexConstraint(frmMain *frame, const wxString &resName, ctlListView *colList)
 : dlgIndexBase(frame, resName, colList)
 {
 }
@@ -50,7 +52,7 @@ dlgIndexConstraint::dlgIndexConstraint(frmMain *frame, const wxString &resName,
 int dlgIndexConstraint::Go(bool modal)
 {
     wxNotifyEvent event;
-    OnCheckDeferrable(event);
+    PrepareTablespace(cbTablespace);
 
     if (index)
     {
@@ -60,12 +62,17 @@ int dlgIndexConstraint::Go(bool modal)
         chkDeferred->SetValue(idc->GetDeferred());
         chkDeferrable->Disable();
         chkDeferred->Disable();
+        if (!idc->GetTablespace().IsEmpty())
+            cbTablespace->SetValue(idc->GetTablespace());
+        cbTablespace->Enable(connection->BackendMinimumVersion(7, 5));
     }
     else
     {
         txtComment->Disable();
     }
 
+    OnCheckDeferrable(event);
+
     return dlgIndexBase::Go(modal);
 }
 
@@ -75,12 +82,20 @@ wxString dlgIndexConstraint::GetDefinition()
     wxString sql;
 
     sql = wxT("(") + GetColumns() + wxT(")");
-
+    AppendIfFilled(sql, wxT(" USING INDEX TABLESPACE "),qtIdent(cbTablespace->GetValue()));
+    if (chkDeferrable->GetValue())
+    {
+        sql += wxT(" DEFERRABLE INITIALLY ");
+        if (chkDeferred->Enable())
+            sql += wxT("DEFERRED");
+        else
+            sql += wxT("IMMEDIATE");
+    }
     return sql;
 }
 
 
-void dlgIndexConstraint::OnCheckDeferrable(wxNotifyEvent &ev)
+void dlgIndexConstraint::OnCheckDeferrable(wxCommandEvent &ev)
 {
     bool canDef=chkDeferrable->GetValue();
     stDeferred->Enable(canDef);
@@ -120,7 +135,7 @@ dlgPrimaryKey::dlgPrimaryKey(frmMain *frame, pgIndex *index, pgTable *parentNode
 }
 
 
-dlgPrimaryKey::dlgPrimaryKey(frmMain *frame, wxListCtrl *colList)
+dlgPrimaryKey::dlgPrimaryKey(frmMain *frame, ctlListView *colList)
 : dlgIndexConstraint(frame, wxT("dlgIndexConstraint"), colList)
 {
     objectType=PG_PRIMARYKEY;
@@ -147,7 +162,7 @@ dlgUnique::dlgUnique(frmMain *frame, pgIndex *index, pgTable *parentNode)
 }
 
 
-dlgUnique::dlgUnique(frmMain *frame, wxListCtrl *colList)
+dlgUnique::dlgUnique(frmMain *frame, ctlListView *colList)
 : dlgIndexConstraint(frame, wxT("dlgIndexConstraint"), colList)
 {
     objectType = PG_UNIQUE;
