@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: dlgSequence.cpp,v 1.29 2004/11/10 18:36:53 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // dlgSequence.cpp - PostgreSQL Sequence Property
@@ -23,28 +24,27 @@
 #include "images/sequence.xpm"
 
 
-#define txtIncrement        CTRL("txtIncrement",    wxTextCtrl)
-#define cbOwner             CTRL("cbOwner",         wxComboBox)
-#define txtMin              CTRL("txtMin",          wxTextCtrl)
-#define txtMax              CTRL("txtMax",          wxTextCtrl)
-#define txtStart            CTRL("txtStart",        wxTextCtrl)
-#define txtCache            CTRL("txtCache",        wxTextCtrl)
-#define chkCycled           CTRL("chkCycled",       wxCheckBox)
-#define stStart             CTRL("stStart",         wxStaticText)
+#define txtIncrement        CTRL_TEXT("txtIncrement")
+#define cbOwner             CTRL_COMBOBOX2("cbOwner")
+#define cbTablespace        CTRL_COMBOBOX("cbTablespace")
+#define txtMin              CTRL_TEXT("txtMin")
+#define txtMax              CTRL_TEXT("txtMax")
+#define txtStart            CTRL_TEXT("txtStart")
+#define txtCache            CTRL_TEXT("txtCache")
+#define chkCycled           CTRL_CHECKBOX("chkCycled")
+#define stStart             CTRL_STATIC("stStart")
 
 // pointer to controls
 
 BEGIN_EVENT_TABLE(dlgSequence, dlgSecurityProperty)
-    EVT_TEXT(XRCID("txtName"),                      dlgSequence::OnChange)
-    EVT_TEXT(XRCID("cbOwner"),                      dlgSequence::OnChange)
-    EVT_TEXT(XRCID("txtStart"),                     dlgSequence::OnChange)
-    EVT_TEXT(XRCID("txtMin"),                       dlgSequence::OnChange)
-    EVT_TEXT(XRCID("txtMax"),                       dlgSequence::OnChange)
-    EVT_TEXT(XRCID("txtCache"),                     dlgSequence::OnChange)
-    EVT_TEXT(XRCID("txtIncrement"),                 dlgSequence::OnChange)
-    EVT_CHECKBOX(XRCID("chkCycled"),                dlgSequence::OnChange)
-
-    EVT_TEXT(XRCID("txtComment"),                   dlgSequence::OnChange)
+    EVT_TEXT(XRCID("cbTablespace"),                 dlgProperty::OnChange)
+    EVT_COMBOBOX(XRCID("cbTablespace"),             dlgProperty::OnChange)
+    EVT_TEXT(XRCID("txtStart"),                     dlgProperty::OnChange)
+    EVT_TEXT(XRCID("txtMin"),                       dlgProperty::OnChange)
+    EVT_TEXT(XRCID("txtMax"),                       dlgProperty::OnChange)
+    EVT_TEXT(XRCID("txtCache"),                     dlgProperty::OnChange)
+    EVT_TEXT(XRCID("txtIncrement"),                 dlgProperty::OnChange)
+    EVT_CHECKBOX(XRCID("chkCycled"),                dlgProperty::OnChange)
 END_EVENT_TABLE();
 
 
@@ -54,8 +54,6 @@ dlgSequence::dlgSequence(frmMain *frame, pgSequence *node, pgSchema *sch)
     SetIcon(wxIcon(sequence_xpm));
     schema=sch;
     sequence=node;
-
-    txtOID->Disable();
 }
 
 
@@ -75,10 +73,6 @@ int dlgSequence::Go(bool modal)
     if (sequence)
     {
         // edit mode
-        txtName->SetValue(sequence->GetName());
-        txtOID->SetValue(NumToStr((long)sequence->GetOid()));
-        cbOwner->SetValue(sequence->GetOwner());
-        txtComment->SetValue(sequence->GetComment());
         txtIncrement->SetValue(sequence->GetIncrement().ToString());
         txtStart->SetValue(sequence->GetLastValue().ToString());
         txtMin->SetValue(sequence->GetMinValue().ToString());
@@ -96,10 +90,12 @@ int dlgSequence::Go(bool modal)
             txtCache->Disable();
             chkCycled->Disable();
         }
+        PrepareTablespace(cbTablespace, sequence->GetTablespace());
     }
     else
     {
         // create mode
+        PrepareTablespace(cbTablespace);
         txtIncrement->SetValidator(numericValidator);
         txtMin->SetValidator(numericValidator);
         txtMax->SetValidator(numericValidator);
@@ -122,7 +118,7 @@ pgObject *dlgSequence::CreateObject(pgCollection *collection)
 }
 
 
-void dlgSequence::OnChange(wxNotifyEvent &ev)
+void dlgSequence::CheckChange()
 {
     wxString name=GetName();
     if (sequence)
@@ -135,7 +131,8 @@ void dlgSequence::OnChange(wxNotifyEvent &ev)
                || txtMax->GetValue() != sequence->GetMaxValue().ToString()
                || txtCache->GetValue() != sequence->GetCacheValue().ToString()
                || txtIncrement->GetValue() != sequence->GetIncrement().ToString()
-               || chkCycled->GetValue() != sequence->GetCycled());
+               || chkCycled->GetValue() != sequence->GetCycled()
+               || cbTablespace->GetValue() != sequence->GetTablespace());
     }
     else
     {
@@ -150,10 +147,23 @@ wxString dlgSequence::GetSql()
 {
     wxString sql;
 
+    wxString name=GetName();
+
     if (sequence)
     {
         // edit mode
 
+        if (GetName() != sequence->GetName())
+        {
+            sql += wxT("ALTER TABLE ") + sequence->GetQuotedFullIdentifier()
+                +  wxT(" RENAME TO ") + qtIdent(name) + wxT(";\n");
+        }
+        if (sequence->GetOwner() != cbOwner->GetValue())
+        {
+            sql += wxT("ALTER TABLE ") + schema->GetQuotedPrefix() + qtIdent(name)
+                +  wxT(" OWNER TO ") + qtIdent(cbOwner->GetValue()) + wxT(";\n");
+        }
+
         if (connection->BackendMinimumVersion(7, 4))
         {
             wxString tmp;
@@ -190,35 +200,26 @@ wxString dlgSequence::GetSql()
 
             if (!tmp.IsEmpty())
             {
-                sql += wxT("ALTER SEQUENCE ") + sequence->GetQuotedFullIdentifier()
+                sql += wxT("ALTER SEQUENCE ") + schema->GetQuotedPrefix() + qtIdent(name)
                     +  tmp + wxT(";\n");
             }
- 
         }
         else
         {
             if (txtStart->GetValue() != sequence->GetLastValue().ToString())
-                sql += wxT("SELECT setval('") + sequence->GetQuotedFullIdentifier()
+                sql += wxT("SELECT setval('") + qtIdent(schema->GetName()) + wxT(".") + qtIdent(name)
                     +  wxT("', ") + txtStart->GetValue()
-                    +  wxT(");\n");
-        }
-
-        if (cbOwner->GetValue() != sequence->GetOwner())
-        {
-            sql += wxT("ALTER TABLE ") + sequence->GetQuotedFullIdentifier()
-                +  wxT(" OWNER TO ") + qtIdent(cbOwner->GetValue()) + wxT(";\n");
-        }
-        if (GetName() != sequence->GetName())
-        {
-            sql += wxT("ALTER TABLE ") + sequence->GetQuotedFullIdentifier()
-                +  wxT(" RENAME TO ") + GetName() + wxT(";\n");
+                    +  wxT(", false);\n");
         }
+        if (cbTablespace->GetValue() != sequence->GetTablespace())
+            sql += wxT("ALTER TABLE ") + schema->GetQuotedPrefix()+ qtIdent(name)
+                +  wxT(" SET TABLESPACE ") + qtIdent(cbTablespace->GetValue())
+                +  wxT(";\n");
     }
     else
     {
         // create mode
-        sql = wxT("CREATE SEQUENCE ") + schema->GetQuotedFullIdentifier()
-            + wxT(".") + qtIdent(GetName());
+        sql = wxT("CREATE SEQUENCE ") + schema->GetQuotedPrefix() + qtIdent(name);
         if (chkCycled->GetValue())
             sql += wxT(" CYCLE");
         AppendIfFilled(sql, wxT("\n   INCREMENT "), txtIncrement->GetValue());
@@ -226,17 +227,17 @@ wxString dlgSequence::GetSql()
         AppendIfFilled(sql, wxT("\n   MINVALUE "), txtMin->GetValue());
         AppendIfFilled(sql, wxT("\n   MAXVALUE "), txtMax->GetValue());
         AppendIfFilled(sql, wxT("\n   CACHE "), txtCache->GetValue());
+        AppendIfFilled(sql, wxT("\n   TABLESPACE "), qtIdent(cbTablespace->GetValue()));
 
         sql += wxT(";\n");
-        if (cbOwner->GetSelection() > 0)
+        if (cbOwner->GetGuessedSelection() > 0)
         {
-            sql += wxT("ALTER TABLE ")  + schema->GetQuotedFullIdentifier()
-                + wxT(".") + qtIdent(GetName())
+            sql += wxT("ALTER TABLE ")  + schema->GetQuotedPrefix() + qtIdent(name)
                 +  wxT(" OWNER TO ") + qtIdent(cbOwner->GetValue()) + wxT(";\n");
         }
     }
 
-    sql +=  GetGrant(wxT("arwdRxt"), wxT("TABLE ") + schema->GetQuotedFullIdentifier() + wxT(".") + qtIdent(GetName()));
+    sql +=  GetGrant(wxT("arwdRxt"), wxT("TABLE ") + schema->GetQuotedPrefix() + qtIdent(name));
     AppendComment(sql, wxT("SEQUENCE"), schema, sequence);
 
     return sql;
