@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: dlgTable.cpp,v 1.48 2004/11/09 13:41:41 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // dlgTable.cpp - PostgreSQL Table Property
@@ -34,29 +35,29 @@
 // Images
 #include "images/table.xpm"
 
-#define cbOwner         CTRL("cbOwner",         wxComboBox)
-#define stHasOids       CTRL("stHasOids",       wxStaticText)
-#define chkHasOids      CTRL("chkHasOids",      wxCheckBox)
-#define lbTables        CTRL("lbTables",        wxListBox)
-#define btnAddTable     CTRL("btnAddTable",     wxButton)
-#define btnRemoveTable  CTRL("btnRemoveTable",  wxButton)
-#define cbTables        CTRL("cbTables",        wxComboBox)
+#define stHasOids       CTRL_STATIC("stHasOids")
+#define chkHasOids      CTRL_CHECKBOX("chkHasOids")
+#define lbTables        CTRL_LISTBOX("lbTables")
+#define btnAddTable     CTRL_BUTTON("btnAddTable")
+#define btnRemoveTable  CTRL_BUTTON("btnRemoveTable")
+#define cbTables        CTRL_COMBOBOX2("cbTables")
+#define cbTablespace    CTRL_COMBOBOX("cbTablespace")
 
-#define btnAddCol       CTRL("btnAddCol"        wxButton)
-#define btnChangeCol    CTRL("btnChangeCol",     wxButton)
-#define btnRemoveCol    CTRL("btnRemoveCol",    wxButton)
+#define btnAddCol       CTRL_BUTTON("btnAddCol")
+#define btnChangeCol    CTRL_BUTTON("btnChangeCol")
+#define btnRemoveCol    CTRL_BUTTON("btnRemoveCol")
 
-#define lstConstraints  CTRL("lstConstraints",  wxListCtrl)
-#define btnAddConstr    CTRL("btnAddConstr"     wxButton)
-#define cbConstrType    CTRL("cbConstrType",    wxComboBox)
-#define btnRemoveConstr CTRL("btnRemoveConstr", wxButton)
+#define lstConstraints  CTRL_LISTVIEW("lstConstraints")
+#define btnAddConstr    CTRL_BUTTON("btnAddConstr")
+#define cbConstrType    CTRL_COMBOBOX("cbConstrType")
+#define btnRemoveConstr CTRL_BUTTON("btnRemoveConstr")
 
 
 BEGIN_EVENT_TABLE(dlgTable, dlgSecurityProperty)
-    EVT_TEXT(XRCID("txtName"),                      dlgTable::OnChange)
-    EVT_TEXT(XRCID("txtComment"),                   dlgTable::OnChange)
-    EVT_CHECKBOX(XRCID("chkHasOids"),               dlgTable::OnChange)
-    EVT_TEXT(XRCID("cbOwner"),                      dlgTable::OnChange)
+    EVT_CHECKBOX(XRCID("chkHasOids"),               dlgProperty::OnChange)
+    EVT_TEXT(XRCID("cbTablespace"),                 dlgProperty::OnChange)
+    EVT_COMBOBOX(XRCID("cbTablespace"),             dlgProperty::OnChange)
+    EVT_TEXT(XRCID("cbTables"),                     dlgTable::OnChangeTable)
     EVT_BUTTON(XRCID("btnAddTable"),                dlgTable::OnAddTable)
     EVT_BUTTON(XRCID("btnRemoveTable"),             dlgTable::OnRemoveTable)
     EVT_LISTBOX(XRCID("lbTables"),                  dlgTable::OnSelChangeTable)
@@ -78,15 +79,14 @@ dlgTable::dlgTable(frmMain *frame, pgTable *node, pgSchema *sch)
     schema=sch;
     table=node;
 
-    txtOID->Disable();
     btnRemoveTable->Disable();
 
-    CreateListColumns(lstColumns, _("Column name"), _("Definition"), 150);
-    lstColumns->InsertColumn(2, wxT("Inherited from table"), wxLIST_FORMAT_LEFT, 0);
-    lstColumns->InsertColumn(3, wxT("Column definition"), wxLIST_FORMAT_LEFT, 0);
-    lstColumns->InsertColumn(4, wxT("Column"), wxLIST_FORMAT_LEFT, 0);
+    lstColumns->CreateColumns(frame, _("Column name"), _("Definition"), 90);
+    lstColumns->AddColumn(wxT("Inherited from table"), 0);
+    lstColumns->AddColumn(wxT("Column definition"), 0);
+    lstColumns->AddColumn(wxT("Column"), 0);
 
-    CreateListColumns(lstConstraints, _("Constraint name"), _("Definition"), 150);
+    lstConstraints->CreateColumns(frame, _("Constraint name"), _("Definition"), 90);
 }
 
 
@@ -102,18 +102,17 @@ int dlgTable::Go(bool modal)
         cbOwner->Append(wxT(""));
     AddGroups();
     AddUsers(cbOwner);
+    PrepareTablespace(cbTablespace);
 
     hasPK=false;
 
     if (table)
     {
         // edit mode
-        txtName->SetValue(table->GetName());
-        txtOID->SetValue(NumToStr(table->GetOid()));
         chkHasOids->SetValue(table->GetHasOids());
-        txtComment->SetValue(table->GetComment());
 
-        cbOwner->SetValue(table->GetOwner());
+        if (!table->GetTablespace().IsEmpty())
+            cbTablespace->SetValue(table->GetTablespace());
 
         wxArrayString qitl=table->GetQuotedInheritedTablesList();
         size_t i;
@@ -124,8 +123,7 @@ int dlgTable::Go(bool modal)
         lbTables->Disable();
         cbTables->Disable();
         chkHasOids->Disable();
-
-        txtOID->Disable();
+        cbTablespace->Enable(connection->BackendMinimumVersion(7, 5));
 
         wxCookieType cookie;
         pgObject *data=0;
@@ -165,8 +163,8 @@ int dlgTable::Go(bool modal)
                     if (column->GetColNumber() > 0)
                     {
                         bool inherited = (column->GetInheritedCount() != 0);
-                        int pos=AppendListItem(lstColumns, column->GetName(), column->GetDefinition(), 
-                            (inherited ? PGICON_TABLE : column->GetIcon()));
+                        int pos=lstColumns->AppendItem((inherited ? PGICON_TABLE : column->GetIcon()), 
+                            column->GetName(), column->GetDefinition());
                         previousColumns.Add(column->GetQuotedIdentifier() 
                             + wxT(" ") + column->GetDefinition());
                         lstColumns->SetItem(pos, 4, NumToStr((long)column));
@@ -199,7 +197,7 @@ int dlgTable::Go(bool modal)
                     {
                         pgIndexConstraint *obj=(pgIndexConstraint*)data;
 
-                        AppendListItem(lstConstraints, obj->GetName(), obj->GetDefinition(), data->GetIcon());
+                        lstConstraints->AppendItem(data->GetIcon(), obj->GetName(), obj->GetDefinition());
                         previousConstraints.Add(obj->GetQuotedIdentifier() 
                             + wxT(" ") + obj->GetTypeName().Upper() + wxT(" ") + obj->GetDefinition());
                         break;
@@ -208,7 +206,7 @@ int dlgTable::Go(bool modal)
                     {
                         pgForeignKey *obj=(pgForeignKey*)data;
 
-                        AppendListItem(lstConstraints, obj->GetName(), obj->GetDefinition(), data->GetIcon());
+                        lstConstraints->AppendItem(data->GetIcon(), obj->GetName(), obj->GetDefinition());
                         previousConstraints.Add(obj->GetQuotedIdentifier() 
                             + wxT(" ") + obj->GetTypeName().Upper() + wxT(" ") + obj->GetDefinition());
                         break;
@@ -217,7 +215,7 @@ int dlgTable::Go(bool modal)
                     {
                         pgCheck *obj=(pgCheck*)data;
 
-                        AppendListItem(lstConstraints, obj->GetName(), obj->GetDefinition(), data->GetIcon());
+                       lstConstraints->AppendItem(data->GetIcon(), obj->GetName(), obj->GetDefinition());
                         previousConstraints.Add(obj->GetQuotedIdentifier() 
                             + wxT(" ") + obj->GetTypeName().Upper() + wxT(" ") + obj->GetDefinition());
                         break;
@@ -236,9 +234,8 @@ int dlgTable::Go(bool modal)
         wxString systemRestriction;
         if (!settings->GetShowSystemObjects())
         systemRestriction = 
-            wxT("   AND (n.oid = 2200 OR n.oid >= ") + NumToStr(connection->GetLastSystemOID()) + wxT(")\n")
-            wxT("   AND n.nspname NOT LIKE 'pg\\_temp\\_%'\n");
-
+            wxT("   AND ") + connection->SystemNamespaceRestriction(wxT("n.nspname"));
+            
         pgSet *set=connection->ExecuteSet(
             wxT("SELECT c.oid, c.relname , nspname\n")
             wxT("  FROM pg_class c\n")
@@ -250,8 +247,8 @@ int dlgTable::Go(bool modal)
         {
             while (!set->Eof())
             {
-                cbTables->Append(qtIdent(set->GetVal(wxT("nspname")))
-                        +wxT(".")+qtIdent(set->GetVal(wxT("relname"))));
+                cbTables->Append(database->GetQuotedSchemaPrefix(set->GetVal(wxT("nspname")))
+                        + qtIdent(set->GetVal(wxT("relname"))));
 
                 tableOids.Add(set->GetVal(wxT("oid")));
                 set->MoveNext();
@@ -265,13 +262,14 @@ int dlgTable::Go(bool modal)
     btnChangeCol->Disable();
     btnRemoveCol->Disable();
     btnRemoveConstr->Disable();
+    btnOK->Disable();
 
     return dlgSecurityProperty::Go();
 }
 
 
 
-wxString dlgTable::GetItemConstraintType(wxListCtrl *list, long pos)
+wxString dlgTable::GetItemConstraintType(ctlListView *list, long pos)
 {
     wxString con;
     wxListItem item;
@@ -301,50 +299,45 @@ wxString dlgTable::GetItemConstraintType(wxListCtrl *list, long pos)
 wxString dlgTable::GetSql()
 {
     wxString sql;
-    wxString tabname=schema->GetQuotedFullIdentifier() + wxT(".") + qtIdent(GetName());
+    wxString tabname=schema->GetQuotedPrefix() + qtIdent(GetName());
 
     if (table)
     {
         int pos;
-        int index;
+        int index=-1;
 
         wxString definition;
         wxArrayString tmpDef=previousColumns;
 
-        if (GetName() != table->GetName())
-            sql += wxT("ALTER TABLE ") + table->GetQuotedFullIdentifier()
-                +  wxT(" RENAME TO ") + qtIdent(GetName())
-                +  wxT(";\n");
-
-        if (cbOwner->GetValue() != table->GetOwner())
-            sql += wxT("ALTER TABLE ") + tabname 
-                +  wxT(" OWNER TO ") + qtIdent(cbOwner->GetValue())
-                + wxT(";\n");
-
-
         for (pos=0; pos < lstColumns->GetItemCount() ; pos++)
         {
-            definition = GetListText(lstColumns, pos, 3);
+            definition = lstColumns->GetText(pos, 3);
             if (definition.IsEmpty())
             {
-                definition=qtIdent(lstColumns->GetItemText(pos)) + wxT(" ") + GetListText(lstColumns, pos, 1);
+                definition=qtIdent(lstColumns->GetText(pos)) + wxT(" ") + lstColumns->GetText(pos, 1);
                 index=tmpDef.Index(definition);
                 if (index < 0)
-                    sql += wxT("ALTER TABLE ") + tabname
+                    sql += wxT("ALTER TABLE ") + table->GetQuotedFullIdentifier()
                         +  wxT(" ADD COLUMN ") + definition + wxT(";\n");
             }
             else
             {
                 sql += definition;
 
-                pgColumn *column=(pgColumn*) StrToLong(GetListText(lstColumns, pos, 4));
-                index=tmpDef.Index(column->GetQuotedIdentifier() 
-                            + wxT(" ") + column->GetDefinition());
+                pgColumn *column=(pgColumn*) StrToLong(lstColumns->GetText(pos, 4));
+                if (column)
+                {
+                    index=tmpDef.Index(column->GetQuotedIdentifier() 
+                                + wxT(" ") + column->GetDefinition());
+                }
             }
             if (index >= 0)
                 tmpDef.RemoveAt(index);
         }
 
+        AppendNameChange(sql);
+        AppendOwnerChange(sql, wxT("TABLE ") + tabname);
+
         for (index=0 ; index < (int)tmpDef.GetCount() ; index++)
         {
             definition = tmpDef.Item(index);
@@ -365,7 +358,7 @@ wxString dlgTable::GetSql()
             wxString conname= qtIdent(lstConstraints->GetItemText(pos));
             definition = conname;
             definition += wxT(" ") + GetItemConstraintType(lstConstraints, pos) 
-                        + wxT(" ") + GetListText(lstConstraints, pos, 1);
+                        + wxT(" ") + lstConstraints->GetText(pos, 1);
             index=tmpDef.Index(definition);
             if (index >= 0)
                 tmpDef.RemoveAt(index);
@@ -395,6 +388,13 @@ wxString dlgTable::GetSql()
             sql += wxT("ALTER TABLE ") + tabname 
                 +  wxT(" WITHOUT OIDS;\n");
         }
+        if (cbTablespace->GetValue() != table->GetTablespace())
+        {
+            if (cbTablespace->GetSelection() > 0)
+                sql += wxT("ALTER TABLE ") + tabname 
+                    +  wxT(" SET TABLESPACE ") + qtIdent(cbTablespace->GetValue())
+                    + wxT(";\n");
+        }
     }
     else
     {
@@ -405,7 +405,7 @@ wxString dlgTable::GetSql()
         bool needComma=false;
         for (pos=0 ; pos < lstColumns->GetItemCount() ; pos++)
         {
-            if (GetListText(lstColumns, pos, 2).IsEmpty())
+            if (lstColumns->GetText(pos, 2).IsEmpty())
             {
                 // standard definition, not inherited
                 if (needComma)
@@ -413,8 +413,8 @@ wxString dlgTable::GetSql()
                 else
                     needComma=true;
 
-                wxString name=lstColumns->GetItemText(pos);
-                wxString definition = GetListText(lstColumns, pos, 1);
+                wxString name=lstColumns->GetText(pos);
+                wxString definition = lstColumns->GetText(pos, 1);
 
                 sql += wxT("\n   ") + qtIdent(name)
                     + wxT(" ") + definition;
@@ -424,7 +424,7 @@ wxString dlgTable::GetSql()
         for (pos=0 ; pos < lstConstraints->GetItemCount() ; pos++)
         {
             wxString name=lstConstraints->GetItemText(pos);
-            wxString definition = GetListText(lstConstraints, pos, 1);
+            wxString definition = lstConstraints->GetText(pos, 1);
 
             if (needComma)
                 sql += wxT(", ");
@@ -438,6 +438,7 @@ wxString dlgTable::GetSql()
         }
         sql += wxT("\n) ");
 
+
         if (lbTables->GetCount() > 0)
         {
             sql += wxT("\nINHERITS (");
@@ -451,12 +452,13 @@ wxString dlgTable::GetSql()
             }
             sql += wxT(")\n");
         }
-        sql += (chkHasOids->GetValue() ? wxT("WITH OIDS;\n") : wxT("WITHOUT OIDS;\n"));
+        sql += (chkHasOids->GetValue() ? wxT("WITH OIDS") : wxT("WITHOUT OIDS"));
+        if (cbTablespace->GetSelection() > 0)
+            sql += wxT("\nTABLESPACE ") + qtIdent(cbTablespace->GetValue());
 
-        if (cbOwner->GetSelection() > 0)
-            sql += wxT("ALTER TABLE ") + tabname 
-                +  wxT(" OWNER TO ") + qtIdent(cbOwner->GetValue())
-                + wxT(";\n");
+        sql += wxT(";\n");
+
+        AppendOwnerNew(sql, wxT("TABLE ") + tabname);
     }
     AppendComment(sql, wxT("TABLE"), schema, table);
     sql +=  GetGrant(wxT("arwdRxt"), wxT("TABLE ") + tabname);
@@ -491,20 +493,20 @@ pgObject *dlgTable::CreateObject(pgCollection *collection)
 }
 
 
-void dlgTable::OnChange(wxNotifyEvent &ev)
+void dlgTable::OnChangeTable(wxCommandEvent &ev)
+{
+    cbTables->GuessSelection(ev);
+}
+
+
+void dlgTable::CheckChange()
 {
     if (table)
     {
         bool changed=false;
         if (connection->BackendMinimumVersion(7, 4) || lstColumns->GetItemCount() > 0)
         {
-            if (GetName() != table->GetName() ||
-                txtComment->GetValue() != table->GetComment() ||
-                cbOwner->GetValue() != table->GetOwner())
-//                 (hasPK && chkHasOids->GetValue() != table->GetHasOids())
-                changed=true;
-            else
-                changed = !GetSql().IsEmpty();
+            changed = !GetSql().IsEmpty();
         }
         EnableOK(changed);
     }
@@ -520,9 +522,9 @@ void dlgTable::OnChange(wxNotifyEvent &ev)
 }
 
 
-void dlgTable::OnAddTable(wxNotifyEvent &ev)
+void dlgTable::OnAddTable(wxCommandEvent &ev)
 {
-    int sel=cbTables->GetSelection();
+    int sel=cbTables->GetGuessedSelection();
     if (sel >= 0)
     {
         wxString tabname=cbTables->GetValue();
@@ -540,19 +542,19 @@ void dlgTable::OnAddTable(wxNotifyEvent &ev)
             int row;
             while (!set->Eof())
             {
-                row=AppendListItem(lstColumns, set->GetVal(wxT("attname")), 
-                    wxString::Format(_("Inherited from table %s"), tabname.c_str()), PGICON_TABLE);
+                row=lstColumns->AppendItem(PGICON_TABLE, set->GetVal(wxT("attname")), 
+                    wxString::Format(_("Inherited from table %s"), tabname.c_str()));
                 lstColumns->SetItem(row, 2, tabname);
                 set->MoveNext();
             }
             delete set;
         }        
-        OnChange(ev);
+        CheckChange();
     }
 }
 
 
-void dlgTable::OnRemoveTable(wxNotifyEvent &ev)
+void dlgTable::OnRemoveTable(wxCommandEvent &ev)
 {
     int sel=lbTables->GetSelection();
     if (sel >= 0)
@@ -567,74 +569,76 @@ void dlgTable::OnRemoveTable(wxNotifyEvent &ev)
         size_t row=lstColumns->GetItemCount();
         while (row--)
         {
-            if (tabname == GetListText(lstColumns, row, 2))
+            if (tabname == lstColumns->GetText(row, 2))
                 lstColumns->DeleteItem(row);
         }
-        OnChange(ev);
+        CheckChange();
     }
     btnRemoveTable->Disable();
 }
 
 
-void dlgTable::OnSelChangeTable(wxListEvent &ev)
+void dlgTable::OnSelChangeTable(wxCommandEvent &ev)
 {
     btnRemoveTable->Enable();
 }
 
 
-void dlgTable::OnChangeCol(wxNotifyEvent &ev)
+void dlgTable::OnChangeCol(wxCommandEvent &ev)
 {
-    long pos=GetListSelected(lstColumns);
-    pgColumn *column=(pgColumn*) StrToLong(GetListText(lstColumns, pos, 4));
+    long pos=lstColumns->GetSelection();
+    pgColumn *column=(pgColumn*) StrToLong(lstColumns->GetText(pos, 4));
 
     dlgColumn col(mainForm, column, table);
     col.CenterOnParent();
-    col.SetConnection(connection);
+    col.SetDatabase(database);
     if (col.Go(true) >= 0)
     {
         lstColumns->SetItem(pos, 0, col.GetName());
         lstColumns->SetItem(pos, 1, col.GetDefinition());
         lstColumns->SetItem(pos, 3, col.GetSql());
     }
-    wxNotifyEvent event;
-    OnChange(event);
+    CheckChange();
 }
 
 
-void dlgTable::OnAddCol(wxNotifyEvent &ev)
+void dlgTable::OnAddCol(wxCommandEvent &ev)
 {
     dlgColumn col(mainForm, NULL, table);
     col.CenterOnParent();
-    col.SetConnection(connection);
+    col.SetDatabase(database);
     if (col.Go(true) >= 0)
-        AppendListItem(lstColumns, col.GetName(), col.GetDefinition(), PGICON_COLUMN);
-    wxNotifyEvent event;
-    OnChange(event);
+    {
+        long pos = lstColumns->AppendItem(PGICON_COLUMN, col.GetName(), col.GetDefinition());
+        if (table && !connection->BackendMinimumVersion(8, 0))
+            lstColumns->SetItem(pos, 3, col.GetSql());
+    }
+
+    CheckChange();
 }
 
 
-void dlgTable::OnRemoveCol(wxNotifyEvent &ev)
+void dlgTable::OnRemoveCol(wxCommandEvent &ev)
 {
-    lstColumns->DeleteItem(GetListSelected(lstColumns));
+    lstColumns->DeleteCurrentItem();
 
     btnRemoveCol->Disable();
 
-    wxNotifyEvent event;
-    OnChange(event);
+    CheckChange();
 }
 
 
 void dlgTable::OnSelChangeCol(wxListEvent &ev)
 {
-    long pos=GetListSelected(lstColumns);
-    wxString inheritedFromTable=GetListText(lstColumns, pos, 2);
+    long pos=lstColumns->GetSelection();
+    wxString inheritedFromTable=lstColumns->GetText(pos, 2);
     
     btnRemoveCol->Enable(inheritedFromTable.IsEmpty());
-    btnChangeCol->Enable(table != 0 && !GetListText(lstColumns, pos, 4).IsEmpty());
+    btnChangeCol->Enable(table != 0 && !lstColumns->GetText(pos, 4).IsEmpty());
 }
 
 
-void dlgTable::OnAddConstr(wxNotifyEvent &ev)
+void dlgTable::OnAddConstr(wxCommandEvent &ev)
 {
     int sel=cbConstrType->GetSelection();
     if (hasPK)
@@ -646,10 +650,10 @@ void dlgTable::OnAddConstr(wxNotifyEvent &ev)
         {
             dlgPrimaryKey pk(mainForm, lstColumns);
             pk.CenterOnParent();
-            pk.SetConnection(connection);
+            pk.SetDatabase(database);
             if (pk.Go(true) >= 0)
             {
-                AppendListItem(lstConstraints, pk.GetName(), pk.GetDefinition(), PGICON_PRIMARYKEY);
+                lstConstraints->AppendItem(PGICON_PRIMARYKEY, pk.GetName(), pk.GetDefinition());
                 hasPK=true;
                 FillConstraint();
             }
@@ -659,12 +663,12 @@ void dlgTable::OnAddConstr(wxNotifyEvent &ev)
         {
             dlgForeignKey fk(mainForm, lstColumns);
             fk.CenterOnParent();
-            fk.SetConnection(connection);
+            fk.SetDatabase(database);
             if (fk.Go(true) >= 0)
             {
                 wxString str=fk.GetDefinition();
                 str.Replace(wxT("\n"), wxT(" "));
-                AppendListItem(lstConstraints, fk.GetName(), str, PGICON_FOREIGNKEY);
+                lstConstraints->AppendItem(PGICON_FOREIGNKEY, fk.GetName(), str);
             }
             break;
         }
@@ -672,29 +676,28 @@ void dlgTable::OnAddConstr(wxNotifyEvent &ev)
         {
             dlgUnique unq(mainForm, lstColumns);
             unq.CenterOnParent();
-            unq.SetConnection(connection);
+            unq.SetDatabase(database);
             if (unq.Go(true) >= 0)
-                AppendListItem(lstConstraints, unq.GetName(), unq.GetDefinition(), PGICON_UNIQUE);
+                lstConstraints->AppendItem(PGICON_UNIQUE, unq.GetName(), unq.GetDefinition());
             break;
         }
         case 3: // Check
         {
             dlgCheck chk(mainForm);
             chk.CenterOnParent();
-            chk.SetConnection(connection);
+            chk.SetDatabase(database);
             if (chk.Go(true) >= 0)
-                AppendListItem(lstConstraints, chk.GetName(), chk.GetDefinition(), PGICON_CHECK);
+                lstConstraints->AppendItem(PGICON_CHECK, chk.GetName(), chk.GetDefinition());
             break;
         }
     }
-    wxNotifyEvent event;
-    OnChange(event);
+    CheckChange();
 }
 
 
-void dlgTable::OnRemoveConstr(wxNotifyEvent &ev)
+void dlgTable::OnRemoveConstr(wxCommandEvent &ev)
 {
-    int pos=GetListSelected(lstConstraints);
+    int pos=lstConstraints->GetSelection();
     if (pos < 0)
         return;
 
@@ -711,8 +714,8 @@ void dlgTable::OnRemoveConstr(wxNotifyEvent &ev)
     
     lstConstraints->DeleteItem(pos);
     btnRemoveConstr->Disable();
-    wxNotifyEvent event;
-    OnChange(event);
+
+    CheckChange();
 }
 
 
