@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: dlgTrigger.cpp,v 1.24 2004/11/09 13:41:41 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // dlgTrigger.cpp - PostgreSQL Trigger Property
@@ -26,23 +27,22 @@
 
 
 // pointer to controls
-#define chkRow          CTRL("chkRow",          wxCheckBox)
-#define cbFunction      CTRL("cbFunction",      wxComboBox)
-#define txtArguments    CTRL("txtArguments",    wxTextCtrl)
-#define rdbFires        CTRL("rdbFires",        wxRadioBox)
-#define chkInsert       CTRL("chkInsert",       wxCheckBox)
-#define chkUpdate       CTRL("chkUpdate",       wxCheckBox)
-#define chkDelete       CTRL("chkDelete",       wxCheckBox)
+#define chkRow          CTRL_CHECKBOX("chkRow")
+#define cbFunction      CTRL_COMBOBOX2("cbFunction")
+#define txtArguments    CTRL_TEXT("txtArguments")
+#define rdbFires        CTRL_RADIOBOX("rdbFires")
+#define chkInsert       CTRL_CHECKBOX("chkInsert")
+#define chkUpdate       CTRL_CHECKBOX("chkUpdate")
+#define chkDelete       CTRL_CHECKBOX("chkDelete")
 
 
 BEGIN_EVENT_TABLE(dlgTrigger, dlgProperty)
-    EVT_TEXT(XRCID("txtName"),                      dlgTrigger::OnChange)
-    EVT_CHECKBOX(XRCID("chkInsert"),                dlgTrigger::OnChange)
-    EVT_CHECKBOX(XRCID("chkUpdate"),                dlgTrigger::OnChange)
-    EVT_CHECKBOX(XRCID("chkDelete"),                dlgTrigger::OnChange)
-    EVT_TEXT(XRCID("cbFunction"),                   dlgTrigger::OnChange)
-    EVT_TEXT(XRCID("txtArguments"),                 dlgTrigger::OnChange)
-    EVT_TEXT(XRCID("txtComment"),                   dlgTrigger::OnChange)
+    EVT_CHECKBOX(XRCID("chkInsert"),                dlgProperty::OnChange)
+    EVT_CHECKBOX(XRCID("chkUpdate"),                dlgProperty::OnChange)
+    EVT_CHECKBOX(XRCID("chkDelete"),                dlgProperty::OnChange)
+    EVT_TEXT(XRCID("cbFunction"),                   dlgTrigger::OnChangeFunc)
+    EVT_COMBOBOX(XRCID("cbFunction"),               dlgProperty::OnChange)
+    EVT_TEXT(XRCID("txtArguments"),                 dlgProperty::OnChange)
 END_EVENT_TABLE();
 
 
@@ -53,8 +53,6 @@ dlgTrigger::dlgTrigger(frmMain *frame, pgTrigger *node, pgTable *parentNode)
     trigger=node;
     table=parentNode;
     wxASSERT(!table || table->GetType() == PG_TABLE);
-
-    txtOID->Disable();
 }
 
 
@@ -69,22 +67,20 @@ int dlgTrigger::Go(bool modal)
     if (trigger)
     {
         // edit mode
-        txtName->SetValue(trigger->GetName());
-
         chkRow->SetValue((trigger->GetTriggerType() & TRIGGER_TYPE_ROW) != 0);
         chkInsert->SetValue((trigger->GetTriggerType() & TRIGGER_TYPE_INSERT) != 0);
         chkUpdate->SetValue((trigger->GetTriggerType() & TRIGGER_TYPE_UPDATE) != 0);
         chkDelete->SetValue((trigger->GetTriggerType() & TRIGGER_TYPE_DELETE) != 0);
         rdbFires->SetSelection(trigger->GetTriggerType() & TRIGGER_TYPE_BEFORE ? 0 : 1);
         txtArguments->SetValue(trigger->GetArguments());
-        txtName->Disable();
+        if (!connection->BackendMinimumVersion(7, 4))
+            txtName->Disable();
         chkRow->Disable();
         cbFunction->Append(trigger->GetFunction());
         cbFunction->SetSelection(0);
         txtArguments->Disable();
         cbFunction->Disable();
         rdbFires->Disable();
-        txtComment->SetValue(trigger->GetComment());
         chkInsert->Disable();
         chkUpdate->Disable();
         chkDelete->Disable();
@@ -94,11 +90,12 @@ int dlgTrigger::Go(bool modal)
         // create mode
         wxString sysRestr;
         if (!settings->GetShowSystemObjects())
-            sysRestr = wxT(" AND (pronamespace=2200 OR pronamespace >") 
-				+ NumToStr(connection->GetLastSystemOID()) + wxT(")");
+            sysRestr = wxT("   AND ") + connection->SystemNamespaceRestriction(wxT("nspname"));
 
         pgSet *set=connection->ExecuteSet(
-            wxT("SELECT quote_ident(nspname) || '.' || quote_ident(proname) FROM pg_proc, pg_namespace WHERE pg_proc.pronamespace = pg_namespace.oid AND prorettype=") + NumToStr(PGOID_TYPE_TRIGGER) + sysRestr);
+            wxT("SELECT quote_ident(nspname) || '.' || quote_ident(proname)\n")
+            wxT("  FROM pg_proc, pg_namespace\n")
+            wxT(" WHERE pg_proc.pronamespace = pg_namespace.oid AND prorettype=") + NumToStr(PGOID_TYPE_TRIGGER) + sysRestr);
         if (set)
         {
             while (!set->Eof())
@@ -121,14 +118,17 @@ int dlgTrigger::Go(bool modal)
 wxString dlgTrigger::GetSql()
 {
     wxString sql;
+    wxString name=GetName();
 
     if (trigger)
     {
-        // nothing
+        if (name != trigger->GetName())
+            sql = wxT("ALTER TRIGGER ") + trigger->GetQuotedIdentifier() + wxT(" ON ") + table->GetQuotedFullIdentifier()
+                + wxT(" RENAME TO ") + qtIdent(name) + wxT(";\n");
     }
     else
     {
-        sql = wxT("CREATE TRIGGER ") + qtIdent(GetName());
+        sql = wxT("CREATE TRIGGER ") + qtIdent(name);
         if (rdbFires->GetSelection())
             sql += wxT(" AFTER");
         else
@@ -179,10 +179,17 @@ pgObject *dlgTrigger::CreateObject(pgCollection *collection)
 }
 
 
-void dlgTrigger::OnChange(wxNotifyEvent &ev)
+void dlgTrigger::OnChangeFunc(wxCommandEvent &ev)
+{
+    cbFunction->GuessSelection(ev);
+    CheckChange();
+}
+
+
+void dlgTrigger::CheckChange()
 {
     if (trigger)
-        btnOK->Enable(txtComment->GetValue() != trigger->GetComment());
+        EnableOK(txtComment->GetValue() != trigger->GetComment());
     else
     {
         wxString function=cbFunction->GetValue();
