@@ -1,10 +1,11 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: frmEditGrid.cpp,v 1.63 2004/11/18 11:47:27 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
-// frmQuery.cpp - SQL Query Box
+// frmEditGrid.cpp - Edit Grid Box
 //
 //////////////////////////////////////////////////////////////////////////
 
@@ -18,15 +19,17 @@
 #include "wxgridsel.h"
 #endif
 
-#include <wx/generic/gridctrl.h>
-
 // App headers
 #include "pgAdmin3.h"
 #include "pgDefs.h"
 #include "frmMain.h"
 #include "menu.h"
 
+#include <wx/generic/gridctrl.h>
+#include <wx/clipbrd.h>
+
 #include "frmEditGrid.h"
+#include "dlgEditGridOptions.h"
 #include "pgTable.h"
 #include "pgView.h"
 
@@ -37,30 +40,38 @@
 #include "images/readdata.xpm"
 #include "images/delete.xpm"
 #include "images/edit_undo.xpm"
+#include "images/sortfilter.xpm"
 #include "images/help.xpm"
+#include "images/clip_copy.xpm"
 
 
 
-BEGIN_EVENT_TABLE(frmEditGrid, wxFrame)
+BEGIN_EVENT_TABLE(frmEditGrid, pgFrame)
     EVT_MENU(MNU_REFRESH,   frmEditGrid::OnRefresh)
     EVT_MENU(MNU_DELETE,    frmEditGrid::OnDelete)
     EVT_MENU(MNU_SAVE,      frmEditGrid::OnSave)
     EVT_MENU(MNU_UNDO,      frmEditGrid::OnUndo)
+    EVT_MENU(MNU_OPTIONS,   frmEditGrid::OnOptions)
     EVT_MENU(MNU_HELP,      frmEditGrid::OnHelp)
+    EVT_MENU(MNU_COPY,      frmEditGrid::OnCopy)
     EVT_CLOSE(              frmEditGrid::OnClose)
     EVT_KEY_DOWN(           frmEditGrid::OnKey)
     EVT_GRID_RANGE_SELECT(frmEditGrid::OnGridSelectCells)
     EVT_GRID_SELECT_CELL(frmEditGrid::OnCellChange)
     EVT_GRID_EDITOR_SHOWN(frmEditGrid::OnEditorShown)
     EVT_GRID_LABEL_LEFT_DCLICK(frmEditGrid::OnLabelDoubleClick)
+    EVT_GRID_LABEL_RIGHT_CLICK(frmEditGrid::OnLabelRightClick)
 END_EVENT_TABLE()
 
 
-frmEditGrid::frmEditGrid(frmMain *form, const wxString& _title, pgConn *_conn, const wxPoint& pos, const wxSize& size, pgSchemaObject *obj)
-: wxFrame(NULL, -1, _title, pos, size)
+frmEditGrid::frmEditGrid(frmMain *form, const wxString& _title, pgConn *_conn, pgSchemaObject *obj)
+: pgFrame(NULL, _title)
 {
-    SetIcon(wxIcon(viewdata_xpm));
     wxLogInfo(wxT("Creating EditGrid"));
+    SetIcon(wxIcon(viewdata_xpm));
+    wxWindowBase::SetFont(settings->GetSystemFont());
+    dlgName = wxT("frmEditGrid");
+    RestorePosition(-1, -1, 600, 500, 200, 150);
     connection=_conn;
     mainForm=form;
     thread=0;
@@ -69,6 +80,7 @@ frmEditGrid::frmEditGrid(frmMain *form, const wxString& _title, pgConn *_conn, c
 
 
     CreateStatusBar();
+    SetStatusBarPane(-1);
 
     sqlGrid = new ctlSQLGrid(this, CTL_EDITGRID, wxDefaultPosition, wxDefaultSize);
     sqlGrid->SetSizer(new wxBoxSizer(wxVERTICAL));
@@ -88,24 +100,30 @@ frmEditGrid::frmEditGrid(frmMain *form, const wxString& _title, pgConn *_conn, c
     toolBar->AddTool(MNU_REFRESH, _("Refresh"), wxBitmap(readdata_xpm), _("Refresh"), wxITEM_NORMAL);
     toolBar->AddTool(MNU_UNDO, _("Undo"), wxBitmap(edit_undo_xpm), _("Undo change of data."), wxITEM_NORMAL);
     toolBar->AddSeparator();
+    toolBar->AddTool(MNU_COPY, _("Copy"), wxBitmap(clip_copy_xpm), _("Copy selected lines to clipboard"), wxITEM_NORMAL);
+    toolBar->AddSeparator();
     toolBar->AddTool(MNU_DELETE, _("Delete"), wxBitmap(delete_xpm), _("Delete selected lines."), wxITEM_NORMAL);
     toolBar->AddSeparator();
+    toolBar->AddTool(MNU_OPTIONS, _("Options"), wxBitmap(sortfilter_xpm), _("Sort/filter options."), wxITEM_NORMAL);
+    toolBar->AddSeparator();
     toolBar->AddTool(MNU_HELP, _("Help"), wxBitmap(help_xpm), _("Display help on SQL commands."));
 
     toolBar->Realize();
     toolBar->EnableTool(MNU_SAVE, false);
     toolBar->EnableTool(MNU_UNDO, false);
+    toolBar->EnableTool(MNU_COPY, false);
     toolBar->EnableTool(MNU_DELETE, false);
 
 
-    wxAcceleratorEntry entries[4];
+    wxAcceleratorEntry entries[5];
 
     entries[0].Set(wxACCEL_CTRL,                (int)'S',      MNU_SAVE);
     entries[1].Set(wxACCEL_NORMAL,              WXK_F5,        MNU_REFRESH);
     entries[2].Set(wxACCEL_CTRL,                (int)'Z',      MNU_UNDO);
     entries[3].Set(wxACCEL_NORMAL,              WXK_F1,        MNU_HELP);
+    entries[3].Set(wxACCEL_CTRL,                (int)'C',      MNU_COPY);
 
-    wxAcceleratorTable accel(4, entries);
+    wxAcceleratorTable accel(5, entries);
     SetAcceleratorTable(accel);
 
 
@@ -120,6 +138,8 @@ frmEditGrid::frmEditGrid(frmMain *form, const wxString& _title, pgConn *_conn, c
         orderBy = table->GetQuotedPrimaryKey();
         if (orderBy.IsEmpty() && hasOids)
             orderBy=wxT("oid");
+        if (!orderBy.IsEmpty())
+            orderBy += wxT(" ASC");
     }
     else if (obj->GetType() == PG_VIEW)
     {
@@ -131,7 +151,29 @@ frmEditGrid::frmEditGrid(frmMain *form, const wxString& _title, pgConn *_conn, c
     }
 }
 
+void frmEditGrid::SetSortCols(const wxString &cols) 
+{ 
+	if (orderBy != cols) { 
+		orderBy = cols; 
+		optionsChanged = true;
+	} 
+}
 
+void frmEditGrid::SetFilter(const wxString &filter) 
+{ 
+	if (rowFilter != filter) { 
+		rowFilter = filter; 
+		optionsChanged = true;
+	} 
+}
+
+void frmEditGrid::OnLabelRightClick(wxGridEvent& event)
+{
+    wxArrayInt rows=sqlGrid->GetSelectedRows();
+    if (rows.GetCount())
+    {
+    }
+}
 
 #define EXTRAEXTENT_HEIGHT 6
 #define EXTRAEXTENT_WIDTH  6
@@ -187,7 +229,7 @@ void frmEditGrid::OnLabelDoubleClick(wxGridEvent& event)
     {
         for (row=0 ; row < sqlGrid->GetNumberRows() ; row++)
         {
-            if (((sqlTable*)sqlGrid->GetTable())->CheckInCache(row))
+            if (sqlGrid->GetTable()->CheckInCache(row))
             {
                 extent = sqlGrid->GetBestSize(row, col).GetWidth();
                 if (extent > extentWant)
@@ -228,7 +270,7 @@ void frmEditGrid::OnLabelDoubleClick(wxGridEvent& event)
 
 void frmEditGrid::OnCellChange(wxGridEvent& event)
 {
-    sqlTable *table=(sqlTable*)sqlGrid->GetTable();
+    sqlTable *table=sqlGrid->GetTable();
     if (table)
     {
         if (table->LastRow() >= 0)
@@ -250,6 +292,29 @@ void frmEditGrid::OnCellChange(wxGridEvent& event)
 }
 
 
+void frmEditGrid::OnCopy(wxCommandEvent &ev)
+{
+    wxArrayInt rows=sqlGrid->GetSelectedRows();
+    size_t i;
+    if (rows.GetCount())
+    {
+        wxString str;
+        for (i=0 ; i < rows.GetCount() ; i++)
+        {
+            str.Append(sqlGrid->GetTable()->GetExportLine(rows.Item(i)));
+    
+            if (rows.GetCount() > 1)
+                str.Append(END_OF_LINE);
+        }
+        if (wxTheClipboard->Open())
+        {
+            wxTheClipboard->SetData(new wxTextDataObject(str));
+            wxTheClipboard->Close();
+        }
+    }
+    SetStatusText(wxString::Format(_("%d rows copied to clipboard."), rows.GetCount()));
+}
+
 
 void frmEditGrid::OnHelp(wxCommandEvent &ev)
 {
@@ -317,11 +382,7 @@ void frmEditGrid::OnKey(wxKeyEvent &event)
                         {
                             long from, to;
                             txt->GetSelection(&from, &to);
-#ifdef __WXMSW__
-                            txt->Replace(from, to, wxT("\r\n"));
-#else
-                            txt->Replace(from, to, wxT("\n"));
-#endif
+                            txt->Replace(from, to, END_OF_LINE);
                         }
                     }
                     edit->DecRef();
@@ -396,7 +457,7 @@ void frmEditGrid::OnClose(wxCloseEvent& event)
 void frmEditGrid::OnUndo(wxCommandEvent& event)
 {
     sqlGrid->DisableCellEditControl();
-    ((sqlTable*)sqlGrid->GetTable())->UndoLine(sqlGrid->GetGridCursorRow());
+    sqlGrid->GetTable()->UndoLine(sqlGrid->GetGridCursorRow());
     sqlGrid->ForceRefresh();
 }
 
@@ -413,27 +474,58 @@ void frmEditGrid::OnSave(wxCommandEvent& event)
     sqlGrid->HideCellEditControl();
     sqlGrid->SaveEditControlValue();
     sqlGrid->DisableCellEditControl();
-    ((sqlTable*)sqlGrid->GetTable())->StoreLine();
+    sqlGrid->GetTable()->StoreLine();
 
     toolBar->EnableTool(MNU_SAVE, false);
     toolBar->EnableTool(MNU_UNDO, false);
 }
 
+void frmEditGrid::OnOptions(wxCommandEvent& event)
+{
+	optionsChanged = false;
+    dlgEditGridOptions *winOptions = new dlgEditGridOptions(this, connection, tableName, sqlGrid);
+    winOptions->ShowModal();
 
+	if (optionsChanged) Go();
+}
+
+template < class T >
+int ArrayCmp(T *a, T *b)
+{
+	if (*a == *b)
+		return 0;
+
+	if (*a > *b)
+		return 1;
+	else
+		return -1;
+}
 
 void frmEditGrid::OnDelete(wxCommandEvent& event)
 {
+	wxMessageDialog msg(this, _("Are you sure you wish to delete the selected row(s)?"), _("Delete rows?"), wxYES_NO | wxICON_QUESTION);
+    if (msg.ShowModal() != wxID_YES)
+        return;
+
     sqlGrid->BeginBatch();
     wxArrayInt delrows=sqlGrid->GetSelectedRows();
     int i=delrows.GetCount();
 
+	// Sort the grid so we always delete last->first, otherwise we 
+	// could end up deleting anything because the array returned by 
+	// GetSelectedRows is in the order that rows were selected by
+	// the user.
+	delrows.Sort(ArrayCmp);
+
     // don't care a lot about optimizing here; doing it line by line
     // just as sqlTable::DeleteRows does
-    while (i--)
-        sqlGrid->DeleteRows(delrows.Item(i), 1);
+	while (i--)
+		sqlGrid->DeleteRows(delrows.Item(i), 1);
+
+
     sqlGrid->EndBatch();
 
-    SetStatusText(wxString::Format(_("%d rows."), ((sqlTable*)sqlGrid->GetTable())->GetNumberStoredRows()), 0);
+    SetStatusText(wxString::Format(_("%d rows."), sqlGrid->GetTable()->GetNumberStoredRows()), 0);
 }
 
 
@@ -455,6 +547,9 @@ void frmEditGrid::OnGridSelectCells(wxGridRangeSelectEvent& event)
         bool enable=rows.GetCount() > 0;
         if (enable)
         {
+            wxCommandEvent nullEvent;
+            OnSave(event);
+
             // check if a readonly line is selected
             int row, col;
             size_t i;
@@ -464,15 +559,18 @@ void frmEditGrid::OnGridSelectCells(wxGridRangeSelectEvent& event)
                 row = rows.Item(i);
                 bool lineEnabled=false;
 
-                if (row != sqlGrid->GetNumberRows()-1)
+                if (row == sqlGrid->GetNumberRows()-1)
                 {
-                    for (col = 0 ; col < sqlGrid->GetNumberCols() ; col++)
+                    // the (*) line may not be deleted/copied
+                    enable=false;
+                    break;
+                }
+                for (col = 0 ; col < sqlGrid->GetNumberCols() ; col++)
+                {
+                    if (!sqlGrid->IsReadOnly(row, col))
                     {
-                        if (!sqlGrid->IsReadOnly(row, col))
-                        {
-                            lineEnabled=true;
-                            break;
-                        }
+                        lineEnabled=true;
+                        break;
                     }
                 }
 
@@ -483,28 +581,52 @@ void frmEditGrid::OnGridSelectCells(wxGridRangeSelectEvent& event)
                 }
             }
         }
-        toolBar->EnableTool(MNU_DELETE,  enable);
+        toolBar->EnableTool(MNU_DELETE, enable);
+        toolBar->EnableTool(MNU_COPY,  enable);
     }
     event.Skip();
 }
 
 
-void frmEditGrid::Go()
+void frmEditGrid::ShowForm(bool filter)
 {
+	bool abort = false;
+
     if (relkind == 'r' || relkind == 'v')
-        Show(true);
+	{
+		if (filter) 
+		{
+			dlgEditGridOptions *winOptions = new dlgEditGridOptions(this, connection, tableName, sqlGrid);
+			abort = !(winOptions->ShowModal());
+		}
+		if (abort) {
+			Close();
+			Destroy();
+		} else {
+            Show(true);
+		    Go();
+		}
+	}
     else
     {
-        wxLogError(_("No Table or view."));
+        wxLogError(__("No Table or view."));
         Close();
         Destroy();
     }
+}
+
+void frmEditGrid::Go()
+{
     SetStatusText(_("Refreshing data, please wait."), 0);
 
     wxString qry=wxT("SELECT ");
     if (hasOids)
-        qry += wxT("Oid, ");
+        qry += wxT("oid, ");
     qry += wxT("* FROM ") + tableName;
+    if (!rowFilter.IsEmpty())
+    {
+        qry += wxT(" WHERE ") + rowFilter;
+    }
     if (!orderBy.IsEmpty())
     {
         qry += wxT(" ORDER BY ") + orderBy;
@@ -522,7 +644,7 @@ void frmEditGrid::Go()
     while (thread && thread->IsRunning())
     {
         wxYield();
-        wxUsleep(10);
+        wxMilliSleep(10);
     }
     if (!thread)
         return;
@@ -554,10 +676,7 @@ frmEditGrid::~frmEditGrid()
 {
     wxLogInfo(wxT("Destroying SQL EditGrid"));
     mainForm->RemoveFrame(this);
-    settings->Write(wxT("frmEditGrid/Width"), GetSize().x);
-    settings->Write(wxT("frmEditGrid/Height"), GetSize().y);
-    settings->Write(wxT("frmEditGrid/Left"), GetPosition().x);
-    settings->Write(wxT("frmEditGrid/Top"), GetPosition().y);
+
     if (connection)
         delete connection;
 }
@@ -593,7 +712,7 @@ ctlSQLGrid::ctlSQLGrid(wxFrame *parent, wxWindowID id, const wxPoint& pos, const
 void ctlSQLGrid::ResizeEditor(int row, int col)
 {
 
-    if (((sqlTable*)GetTable())->needsResizing(col))
+    if (GetTable()->needsResizing(col))
     {
         wxGridCellAttr* attr = GetCellAttr(row, col);
         wxGridCellRenderer* renderer = attr->GetRenderer(this, row, col);
@@ -716,6 +835,45 @@ bool ctlSQLGrid::SetTable(wxGridTableBase *table, bool takeOwnership)
 
 
 
+wxArrayInt ctlSQLGrid::GetSelectedRows() const
+{
+    wxArrayInt rows, rows2;
+
+    wxGridCellCoordsArray tl=GetSelectionBlockTopLeft(), br=GetSelectionBlockBottomRight();
+
+    int maxCol=((ctlSQLGrid*)this)->GetNumberCols() -1;
+    size_t i;
+    for (i=0 ; i < tl.GetCount() ; i++)
+    {
+        wxGridCellCoords c1=tl.Item(i), c2=br.Item(i);
+        if (c1.GetCol() != 0 || c2.GetCol() != maxCol)
+            return rows2;
+
+        int j;
+        for (j=c1.GetRow() ; j <= c2.GetRow() ; j++)
+            rows.Add(j);
+    }
+
+    rows2=wxGrid::GetSelectedRows();
+
+    rows.Sort(ArrayCmp);
+    rows2.Sort(ArrayCmp);
+
+    size_t i2=0, cellRowMax=rows.GetCount();
+
+    for (i=0 ; i < rows2.GetCount() ; i++)
+    {
+        int row=rows2.Item(i);
+        while (i2 < cellRowMax && rows.Item(i2) < row)
+            i2++;
+        if (i2 == cellRowMax || row != rows.Item(i2))
+            rows.Add(row);
+    }
+
+    return rows;
+}
+
+
 class sqlGridTextEditor : public wxGridCellTextEditor
 {
 public:
@@ -967,7 +1125,7 @@ void sqlGridNumericEditor::Create(wxWindow* parent, wxWindowID id, wxEvtHandler*
 
 
 
-sqlTable::sqlTable(pgConn *conn, pgQueryThread *_thread, const wxString& tabName, const Oid _relid, bool _hasOid, const wxString& _pkCols, char _relkind)
+sqlTable::sqlTable(pgConn *conn, pgQueryThread *_thread, const wxString& tabName, const OID _relid, bool _hasOid, const wxString& _pkCols, char _relkind)
 {
     connection=conn;
     primaryKeyColNumbers = _pkCols;
@@ -995,16 +1153,18 @@ sqlTable::sqlTable(pgConn *conn, pgQueryThread *_thread, const wxString& tabName
 
 
     pgSet *colSet=connection->ExecuteSet(
-        wxT("SELECT nspname, relname, t.typname, attname, COALESCE(b.oid, t.oid) AS basetype, atthasdef, adsrc,\n")
+        wxT("SELECT n.nspname AS nspname, relname, t.typname, nt.nspname AS typnspname, ")
+               wxT("attname, COALESCE(b.oid, t.oid) AS basetype, atthasdef, adsrc,\n")
         wxT("       CASE WHEN t.typbasetype::oid=0 THEN att.atttypmod else t.typtypmod END AS typmod,\n")
         wxT("       CASE WHEN t.typbasetype::oid=0 THEN att.attlen else t.typlen END AS typlen\n")
         wxT("  FROM pg_attribute att\n")
         wxT("  JOIN pg_type t ON t.oid=att.atttypid\n")
+        wxT("  JOIN pg_namespace nt ON nt.oid=t.typnamespace\n")
         wxT("  JOIN pg_class c ON c.oid=attrelid\n")
         wxT("  JOIN pg_namespace n ON n.oid=relnamespace\n")
         wxT("  LEFT OUTER JOIN pg_type b ON b.oid=t.typbasetype\n")
         wxT("  LEFT OUTER JOIN pg_attrdef def ON adrelid=attrelid AND adnum=attnum\n")
-        wxT(" WHERE attnum > 0 AND NOT attisdropped AND attrelid=") + NumToStr((long)relid) + wxT("::oid\n")
+        wxT(" WHERE attnum > 0 AND NOT attisdropped AND attrelid=") + NumToStr(relid) + wxT("::oid\n")
         wxT(" ORDER BY attnum"));
 
 
@@ -1017,6 +1177,7 @@ sqlTable::sqlTable(pgConn *conn, pgQueryThread *_thread, const wxString& tabName
             columns[0].name = wxT("oid");
             columns[0].numeric = true;
             columns[0].attr->SetReadOnly(true);
+            columns[0].type = PGTYPCLASS_NUMERIC;
         }
 
         for (i=(hasOids ? 1 : 0) ; i < nCols ; i++)
@@ -1025,6 +1186,7 @@ sqlTable::sqlTable(pgConn *conn, pgQueryThread *_thread, const wxString& tabName
 
             columns[i].name = colSet->GetVal(wxT("attname"));
             columns[i].typeName = colSet->GetVal(wxT("typname"));
+            columns[i].typeNspName = colSet->GetVal(wxT("typnspname"));
 
             columns[i].type = (Oid)colSet->GetOid(wxT("basetype"));
             if ((columns[i].type == PGOID_TYPE_INT4 || columns[i].type == PGOID_TYPE_INT8)
@@ -1129,6 +1291,9 @@ sqlTable::sqlTable(pgConn *conn, pgQueryThread *_thread, const wxString& tabName
     }
     else
     {
+        // um, we never should reach here because this would mean 
+        // that datatypes are unreadable.
+        // *if* we reach here, namespace info is missing.
         for (i=0 ; i < nCols ; i++)
         {
             columns[i].typeName = thread->DataSet()->ColType(i);
@@ -1183,6 +1348,42 @@ int sqlTable::GetNumberStoredRows()
 }
 
 
+wxString sqlTable::GetExportLine(int row)
+{
+    wxString str;
+    cacheLine *line = GetLine(row);
+    if (line)
+    {
+        int col;
+        for (col=0 ; col < nCols ; col++)
+        {
+            if (col)
+                str.Append(settings->GetExportColSeparator());
+            bool needQuote = settings->GetExportQuoting() > 1;
+
+            // find out if string
+            switch (columns[col].type)
+            {
+                case PGTYPCLASS_NUMERIC:
+                case PGTYPCLASS_BOOL:
+                    break;
+                default:
+                    needQuote=true;
+                    break;
+            }
+            if (needQuote)
+                str.Append(settings->GetExportQuoteChar());
+
+            str.Append(line->cols[col]);
+        
+            if (needQuote)
+                str.Append(settings->GetExportQuoteChar());
+        }
+    }
+    return str;
+}
+
+
 wxString sqlTable::GetColLabelValue(int col)
 {
     wxString label=columns[col].name + wxT("\n");
@@ -1269,7 +1470,8 @@ wxString sqlTable::MakeKey(cacheLine *line)
             if (!where.IsEmpty())
                 where += wxT(" AND ");
             where += qtIdent(columns[cn-1].name) + wxT(" = ") 
-                  + qtString(colval) + wxT("::") + qtIdent(columns[cn-1].typeName);
+                  + qtString(colval) + wxT("::") + qtIdent(columns[cn-1].typeNspName)
+                                     + wxT(".") + qtIdent(columns[cn-1].typeName);
         }
     }
     return where;
@@ -1651,7 +1853,7 @@ wxString sqlCellAttr::Quote(const wxString& value)
     else
         str=qtString(value);
 
-    return str + wxT("::") + typeName;
+    return str + wxT("::") + qtIdent(typeNspName) + wxT(".") + qtIdent(typeName);
 }
 
 
