@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: frmMain.cpp,v 1.112 2004/11/09 14:15:43 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // frmMain.cpp - The main form
@@ -11,6 +12,12 @@
 //
 //////////////////////////////////////////////////////////////////////////
 
+#include "pgAdmin3.h"
+
+#ifndef WIN32
+#include <unistd.h>
+#endif
+
 // wxWindows headers
 #include <wx/wx.h>
 #include <wx/splitter.h>
@@ -23,7 +30,6 @@
 #include <wx/imaglist.h>
 
 // App headers
-#include "pgAdmin3.h"
 #include "misc.h"
 #include "menu.h"
 #include "frmMain.h"
@@ -34,7 +40,6 @@
 #include "pgServer.h"
 #include "pgObject.h"
 #include "pgCollection.h"
-#include "frmQueryBuilder.h"
 #include "frmHelp.h"
 
 
@@ -42,10 +47,13 @@
 WX_DEFINE_LIST(windowList);
 
 // Icons
+#include "images/pgAdmin3.xpm"
+#include "images/elephant32.xpm"
 #include "images/aggregate.xpm"
 #include "images/baddatabase.xpm"
 #include "images/check.xpm"
 #include "images/closeddatabase.xpm"
+#include "images/tablespace.xpm"
 #include "images/cast.xpm"
 #include "images/conversion.xpm"
 #include "images/column.xpm"
@@ -62,7 +70,6 @@ WX_DEFINE_LIST(windowList);
 #include "images/namespace.xpm"
 #include "images/operator.xpm"
 #include "images/operatorclass.xpm"
-#include "images/pgAdmin3.xpm"
 #include "images/properties.xpm"
 #include "images/property.xpm"
 #include "images/public.xpm"
@@ -82,24 +89,64 @@ WX_DEFINE_LIST(windowList);
 #include "images/vacuum.xpm"
 #include "images/view.xpm"
 #include "images/viewdata.xpm"
+#include "images/viewfiltereddata.xpm"
 #include "images/triggerfunction.xpm"
 #include "images/constraints.xpm"
 #include "images/primarykey.xpm"
 #include "images/unique.xpm"
 #include "images/help2.xpm"
+#include "images/agent.xpm"
+#include "images/job.xpm"
+#include "images/jobdisabled.xpm"
+#include "images/step.xpm"
+#include "images/schedule.xpm"
 
 
+#if wxDIALOG_UNIT_COMPATIBILITY
+#error wxWindows must be compiled with wxDIALOG_UNIT_COMPATIBILITY=0!
+#endif
 
-frmMain::frmMain(const wxString& title, const wxPoint& pos, const wxSize& size)
-: wxFrame((wxFrame *)NULL, -1, title, pos, size)
+
+extern wxString backupExecutable, restoreExecutable;
+
+frmMain::frmMain(const wxString& title)
+: pgFrame((wxFrame *)NULL, title)
 {
+    msgLevel=0;
 
-	// Current database
-	m_database = NULL;
+    dlgName = wxT("frmMain");
+    RestorePosition(50, 50, 750, 550, 300, 200);
+
+    wxWindowBase::SetFont(settings->GetSystemFont());
+
+    {
+        wxLogInfo(wxT("Using fontmetrics %d/%d, %d Point"), GetCharWidth(), GetCharHeight(), GetFont().GetPointSize());
+        wxLogInfo(wxT("Native Description '%s'"), GetFont().GetNativeFontInfoDesc().c_str());
+        wxWindowDC dc(this);
+        dc.SetFont(GetFont());
+
+        wxCoord w, h, d, e;
+
+        dc.GetTextExtent(wxT("M"), &w, &h, &d, &e);
+        wxLogInfo(wxT("Draw size of 'M': w=%d, h=%d, descent %d, external lead %d."), w, h, d, e);
+
+        dc.GetTextExtent(wxT("g"), &w, &h, &d, &e);
+        wxLogInfo(wxT("Draw size of 'g': w=%d, h=%d, descent %d, external lead %d."), w, h, d, e);
+
+        dc.GetTextExtent(wxT("Mg"), &w, &h, &d, &e);
+        wxLogInfo(wxT("Draw size of 'Mg': w=%d, h=%d, descent %d, external lead %d."), w, h, d, e);
+    }
+
+    // Current database
     denyCollapseItem=wxTreeItemId();
+    currentObject = 0;
+    m_database = 0;
 
     // Icon
-    SetIcon(wxIcon(pgAdmin3_xpm));
+    wxIconBundle icons;
+    icons.AddIcon(wxIcon(pgAdmin3_xpm));
+    icons.AddIcon(wxIcon(elephant32_xpm));
+    SetIcons(icons);
 
     // Build menus
     menuBar = new wxMenuBar();
@@ -131,16 +178,19 @@ frmMain::frmMain(const wxString& title, const wxPoint& pos, const wxSize& size)
 
     // Tools Menu
     toolsMenu = new wxMenu();
-    toolsMenu->Append(MNU_CONNECT, _("&Connect..."),              _("Connect to the selected server."));
+    toolsMenu->Append(MNU_STARTSERVICE, _("Start service"),       _("Start PostgreSQL Service"));
+    toolsMenu->Append(MNU_STOPSERVICE, _("Stop service"),         _("Stop PostgreSQL Service"));
+    toolsMenu->Append(MNU_CONNECT, _("&Connect"),                 _("Connect to the selected server."));
     toolsMenu->Append(MNU_DISCONNECT, _("Disconnec&t"),           _("Disconnect from the selected server."));
     toolsMenu->AppendSeparator();
     toolsMenu->Append(MNU_SQL, _("&Query tool"),                  _("Execute arbitrary SQL queries."));
-#ifdef __DEBUG_QUERYBUILDER__
-    toolsMenu->Append(MNU_QUERYBUILDER, _("Query &builder"),      _("Start the query builder."));
-#endif
 	toolsMenu->Append(MNU_VIEWDATA, _("View &Data"),              _("View the data in the selected object."));
+	toolsMenu->Append(MNU_VIEWFILTEREDDATA, _("View F&iltered Data"), _("Apply a filter and view the data in the selected object."));
     toolsMenu->Append(MNU_MAINTENANCE, _("&Maintenance"),         _("Maintain the current database or table."));
-    toolsMenu->Append(MNU_RELOAD, _("Re&load module"),            _("Reload library module which implements this function."));
+    toolsMenu->Append(MNU_BACKUP, _("&Backup"),                   _("Creates a backup of the current database to a local file"));
+    toolsMenu->Append(MNU_RESTORE, _("&Restore"),                 _("Restores a backup from a local file"));
+//    toolsMenu->Append(MNU_INDEXCHECK, _("&FK Index check"),       _("Checks existence of foreign key indexes"));
+    toolsMenu->Append(MNU_GRANTWIZARD, _("&Grant Wizard"),        _("Grants rights to multiple objects"));
     toolsMenu->Append(MNU_STATUS, _("&Server Status"),            _("Displays the current database status."));
     menuBar->Append(toolsMenu, _("&Tools"));
 
@@ -149,6 +199,7 @@ frmMain::frmMain(const wxString& title, const wxPoint& pos, const wxSize& size)
     viewMenu->Append(MNU_SYSTEMOBJECTS, _("&System objects"),     _("Show or hide system objects."), wxITEM_CHECK);
     viewMenu->AppendSeparator();
     viewMenu->Append(MNU_REFRESH, _("Re&fresh\tF5"),              _("Refresh the selected object."));
+    viewMenu->Append(MNU_COUNT, _("&Count"),                      _("Count rows in the selected object."));
     menuBar->Append(viewMenu, _("&Display"));
 
     // Help Menu
@@ -157,7 +208,7 @@ frmMain::frmMain(const wxString& title, const wxPoint& pos, const wxSize& size)
     helpMenu->Append(MNU_FAQ, _("pgAdmin III &FAQ"),              _("Frequently asked questions about pgAdmin III."));
     helpMenu->Append(MNU_HELP, _("&SQL Help\tF1"),                _("Display help on SQL commands."));
     helpMenu->Append(MNU_PGSQLHELP, _("&PostgreSQL Help"),        _("Display help on PostgreSQL database system."));
-    helpMenu->Append(MNU_TIPOFTHEDAY, _("&Tip of the day..."),    _("Show a tip of the day."));
+    helpMenu->Append(MNU_TIPOFTHEDAY, _("&Tip of the day"),       _("Show a tip of the day."));
     helpMenu->AppendSeparator();
     helpMenu->Append(MNU_BUGREPORT, _("&Bugreport"),              _("How to send a bugreport to the pgAdmin Development Team."));
     helpMenu->Append(MNU_ABOUT, _("&About..."),                   _("Show about dialog."));
@@ -166,28 +217,13 @@ frmMain::frmMain(const wxString& title, const wxPoint& pos, const wxSize& size)
 #endif 
     menuBar->Append(helpMenu, _("&Help"));
 
-    treeContextMenu = new wxMenu();
-    // Tree Context Menu
     newContextMenu = new wxMenu();
-    treeContextMenu->Append(MNU_REFRESH, _("Re&fresh\tF5"),       _("Refresh the selected object."));
-    treeContextMenu->Append(MNU_RELOAD, _("Re&load module"),      _("Reload library module which implements this function."));
-    treeContextMenu->Append(MNU_NEWOBJECT, _("New &Object"), newContextMenu, _("Create a new object."));
-    treeContextMenu->AppendSeparator();
-	treeContextMenu->Append(MNU_VIEWDATA, _("View &Data"),        _("View the data in the selected object."));
-    treeContextMenu->Append(MNU_MAINTENANCE, _("&Maintenance"),    _("Maintain the current database or table."));
-    treeContextMenu->Append(MNU_CONNECT, _("&Connect..."),        _("Connect to the selected server."));
-    treeContextMenu->Append(MNU_DISCONNECT, _("&Disconnect"),     _("Disconnect from the selected server."));
-    treeContextMenu->AppendSeparator();
-    treeContextMenu->Append(MNU_CREATE, _("&Create"),             _("Create a new object of the same type as the selected object."));
-    treeContextMenu->Append(MNU_DROP, _("&Delete/Drop"),  	      _("Delete/Drop the selected object."));
-    treeContextMenu->Append(MNU_PROPERTIES, _("&Properties"),     _("Display/edit the properties of the selected object."));
-
+    treeContextMenu = 0;
 
     // Add the Menubar and set some options
     SetMenuBar(menuBar);
 
     editMenu->Enable(MNU_NEWOBJECT, false);
-    treeContextMenu->Enable(MNU_NEWOBJECT, false);
     fileMenu->Enable(MNU_PASSWORD, false);
     viewMenu->Check(MNU_SYSTEMOBJECTS, settings->GetShowSystemObjects());
 
@@ -195,9 +231,9 @@ frmMain::frmMain(const wxString& title, const wxPoint& pos, const wxSize& size)
     statusBar = CreateStatusBar(3);
     int iWidths[3] = {0, -1, 100};
     SetStatusWidths(3, iWidths);
-    SetStatusText(wxT(""), 0);
-    SetStatusText(_("Ready."), 1);
-    SetStatusText(_("0 Secs"), 2);
+    statusBar->SetStatusText(wxT(""), 0);
+    statusBar->SetStatusText(_("Ready."), 1);
+    statusBar->SetStatusText(_("0 Secs"), 2);
 
     wxAcceleratorEntry entries[4];
     entries[0].Set(wxACCEL_NORMAL, WXK_F5, MNU_REFRESH);
@@ -220,13 +256,14 @@ frmMain::frmMain(const wxString& title, const wxPoint& pos, const wxSize& size)
     toolBar->AddSeparator();
     toolBar->AddTool(MNU_SQL, _("Query tool"), wxBitmap(sql_xpm), _("Execute arbitrary SQL queries."), wxITEM_NORMAL);
     toolBar->AddTool(MNU_VIEWDATA, _("View Data"), wxBitmap(viewdata_xpm), _("View the data in the selected object."), wxITEM_NORMAL);
+    toolBar->AddTool(MNU_VIEWFILTEREDDATA, _("View Filtered Data"), wxBitmap(viewfiltereddata_xpm), _("Apply a filter and view the data in the selected object."), wxITEM_NORMAL);
     toolBar->AddTool(MNU_MAINTENANCE, _("Maintenance"), wxBitmap(vacuum_xpm), _("Maintain the current database or table."), wxITEM_NORMAL);
     toolBar->AddSeparator();
     toolBar->AddTool(MNU_HELP, _("SQL Help"), wxBitmap(help2_xpm), _("Display help on SQL commands."));
 
     // Display the bar and configure buttons. 
     toolBar->Realize();
-    SetButtons(FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE);
+    SetButtons();
     toolBar->EnableTool(MNU_STOP, FALSE);
     
     // Setup the vertical splitter & treeview
@@ -242,11 +279,15 @@ frmMain::frmMain(const wxString& title, const wxPoint& pos, const wxSize& size)
     vertical->SetMinimumPaneSize(50);
 
     // Setup the horizontal splitter for the listview & sql pane
-    wxNotebook* listViews = new wxNotebook(horizontal, -1, wxDefaultPosition, wxDefaultSize, wxNB_BOTTOM);
-    properties = new wxListCtrl(listViews, CTL_PROPVIEW, wxDefaultPosition, wxDefaultSize, wxLC_REPORT | wxSIMPLE_BORDER);
-    statistics = new wxListCtrl(listViews, CTL_STATVIEW, wxDefaultPosition, wxDefaultSize, wxLC_REPORT | wxSIMPLE_BORDER);
-    listViews->AddPage(properties, _("Properties"));
-    listViews->AddPage(statistics, _("Statistics"));
+    listViews = new wxNotebook(horizontal, CTL_NOTEBOOK, wxDefaultPosition, wxDefaultSize, wxNB_BOTTOM);
+    properties = new ctlListView(listViews, CTL_PROPVIEW, wxDefaultPosition, wxDefaultSize, wxSIMPLE_BORDER);
+    statistics = new ctlListView(listViews, CTL_STATVIEW, wxDefaultPosition, wxDefaultSize, wxSIMPLE_BORDER);
+    dependsOn = new ctlListView(listViews, CTL_DEPVIEW, wxDefaultPosition, wxDefaultSize, wxSIMPLE_BORDER);
+    referencedBy = new ctlListView(listViews, CTL_REFVIEW, wxDefaultPosition, wxDefaultSize, wxSIMPLE_BORDER);
+    listViews->AddPage(properties, _("Properties"));        // NBP_PROPERTIES
+    listViews->AddPage(statistics, _("Statistics"));        // NBP_STATISTICS
+    listViews->AddPage(dependsOn, _("Depends on"));         // NBP_DEPENDSON
+    listViews->AddPage(referencedBy, _("Referenced by"));   // NBP_REFERENCEDBY
     sqlPane = new ctlSQLBox(horizontal, CTL_SQLPANE, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE | wxSIMPLE_BORDER | wxTE_READONLY | wxTE_RICH2);
 
     splitpos=settings->Read(wxT("frmMain/SplitHorizontal"), 300);
@@ -258,7 +299,7 @@ frmMain::frmMain(const wxString& title, const wxPoint& pos, const wxSize& size)
     horizontal->SetMinimumPaneSize(50);
 
     //Setup the global imagelist
-	images = new wxImageList(16, 16, true, 35);
+	images = new wxImageList(16, 16, true, 50);
     images->Add(wxIcon(property_xpm));
     images->Add(wxIcon(statistics_xpm));
     images->Add(wxIcon(servers_xpm));
@@ -267,6 +308,7 @@ frmMain::frmMain(const wxString& title, const wxPoint& pos, const wxSize& size)
     images->Add(wxIcon(database_xpm));
     images->Add(wxIcon(language_xpm));
     images->Add(wxIcon(namespace_xpm));
+    images->Add(wxIcon(tablespace_xpm));
     images->Add(wxIcon(aggregate_xpm));
     images->Add(wxIcon(function_xpm));
     images->Add(wxIcon(operator_xpm));
@@ -295,6 +337,13 @@ frmMain::frmMain(const wxString& title, const wxPoint& pos, const wxSize& size)
     images->Add(wxIcon(unique_xpm));
     images->Add(wxIcon(public_xpm));
 
+    // agent, job, jobdisabled, step, schedule
+    images->Add(wxIcon(agent_xpm));
+    images->Add(wxIcon(job_xpm));
+    images->Add(wxIcon(jobdisabled_xpm));
+    images->Add(wxIcon(step_xpm));
+    images->Add(wxIcon(schedule_xpm));
+
     browser->SetImageList(images);
 
     // Add the root node
@@ -303,18 +352,20 @@ frmMain::frmMain(const wxString& title, const wxPoint& pos, const wxSize& size)
 
     properties->SetImageList(images, wxIMAGE_LIST_SMALL);
     // Add the property view columns
-    properties->InsertColumn(0, _("Properties"), wxLIST_FORMAT_LEFT, 500);
+    properties->AddColumn(_("Properties"), 500);
     properties->InsertItem(0, _("No properties are available for the current selection"), PGICON_PROPERTY);
 
 
     statistics->SetImageList(images, wxIMAGE_LIST_SMALL);
     // Add the statistics view columns & set the colour
-    statistics->InsertColumn(0, _("Statistics"), wxLIST_FORMAT_LEFT, 500);
+    statistics->AddColumn(_("Statistics"), 500);
     statistics->InsertItem(0, _("No statistics are available for the current selection"), PGICON_STATISTICS);
 
     wxColour background;
     background = wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE);
     statistics->SetBackgroundColour(background);
+    dependsOn->SetBackgroundColour(background);
+    referencedBy->SetBackgroundColour(background);
     sqlPane->SetBackgroundColour(background);
 
     // Load servers
@@ -327,29 +378,26 @@ frmMain::~frmMain()
 {
     StoreServers();
 
-    settings->Write(wxT("frmMain/Width"), GetSize().x);
-    settings->Write(wxT("frmMain/Height"), GetSize().y);
-    settings->Write(wxT("frmMain/Left"), GetPosition().x);
-    settings->Write(wxT("frmMain/Top"), GetPosition().y);
+    SavePosition();
     settings->Write(wxT("frmMain/SplitHorizontal"), horizontal->GetSashPosition());
     settings->Write(wxT("frmMain/SplitVertical"), vertical->GetSashPosition());
 
     // Clear the treeview
     browser->DeleteAllItems();
 
-    delete treeContextMenu;
+    if (treeContextMenu)
+        delete treeContextMenu;
 	delete images;
 }
 
 
 void frmMain::Refresh(pgObject *data)
 {
-    StartMsg(wxString::Format(_("Refreshing %s..."), data->GetTypeName().c_str()));
+    StartMsg(wxString::Format(_("Refreshing %s"), data->GetTranslatedTypeName().c_str()));
     browser->Freeze();
 
     wxTreeItemId currentItem=data->GetId();
-
-    browser->DeleteChildren(data->GetId());
+    browser->DeleteChildren(currentItem);
 
 	// refresh information about the object
 
@@ -362,6 +410,9 @@ void frmMain::Refresh(pgObject *data)
         wxLogInfo(wxT("Deleting ") + data->GetTypeName() + wxT(" ") 
             + data->GetQuotedFullIdentifier() + wxT(" for Refresh"));
 
+        if (data == currentObject)
+            currentObject = newData;
+
         if (newData)
         {
             wxLogInfo(wxT("Replacing with new Node ") + newData->GetTypeName() + wxT(" ") 
@@ -373,31 +424,80 @@ void frmMain::Refresh(pgObject *data)
         else
         {
             wxLogInfo(wxT("No object to replace: vanished after refresh."));
-            browser->SelectItem(browser->GetItemParent(currentItem));
-            browser->Delete(currentItem);
+            wxTreeItemId delItem=currentItem;
+            currentItem=browser->GetItemParent(currentItem);
+            browser->SelectItem(currentItem);
+            browser->Delete(delItem);
         }
     }
-	execSelChange(currentItem);
+    if (currentItem)
+        execSelChange(currentItem, currentItem == browser->GetSelection());
     browser->Thaw();
     EndMsg();
 }
 
 
-pgObject *frmMain::GetSelectedObject()
+
+void frmMain::OnPageChange(wxNotebookEvent& event)
 {
     wxTreeItemId item=browser->GetSelection();
+	if (!item)
+		return;
     pgObject *data = (pgObject*)browser->GetItemData(item);
 
-    /* When it isn't connected. */
-    if (data == NULL)
-        return 0;
-	
-    if (FindFocus() == properties && data->IsCollection())
+    if (!data)
+        return;
+
+    switch (event.GetSelection())
     {
-        return ((pgCollection*)data)->FindChild(browser, properties->GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED));
+        case NBP_STATISTICS:
+        {
+            statistics->Freeze();
+            data->ShowStatistics(this, statistics);
+            statistics->Thaw();
+            break;
+        }
+        case NBP_DEPENDSON:
+        {
+            dependsOn->Freeze();
+            data->ShowDependsOn(this, dependsOn);
+            dependsOn->Thaw();
+            break;
+        }
+        case NBP_REFERENCEDBY:
+        {
+            referencedBy->Freeze();
+            data->ShowReferencedBy(this, referencedBy);
+            referencedBy->Thaw();
+            break;
+        }
+        default:
+            break;
     }
-    else
-        return data;
+}
+
+
+ctlListView *frmMain::GetStatistics()
+{
+    if (listViews->GetSelection() == NBP_STATISTICS)
+        return statistics;
+    return 0;
+}
+
+
+ctlListView *frmMain::GetDependsOn()
+{
+    if (listViews->GetSelection() == NBP_DEPENDSON)
+        return dependsOn;
+    return 0;
+}
+
+
+ctlListView *frmMain::GetReferencedBy()
+{
+    if (listViews->GetSelection() == NBP_REFERENCEDBY)
+        return referencedBy;
+    return 0;
 }
 
 
@@ -406,6 +506,101 @@ void frmMain::RemoveFrame(wxWindow *frame)
     frames.DeleteObject(frame);
 }
 
+
+bool frmMain::checkAlive()
+{
+    bool userInformed = false;
+    bool closeIt;
+
+    wxCookieType cookie;
+    wxTreeItemId serverItem=browser->GetFirstChild(servers, cookie);
+    while (serverItem)
+    {
+        pgServer *server=(pgServer*)browser->GetItemData(serverItem);
+
+        if (server && server->GetType() == PG_SERVER && server->connection())
+        {
+            if (server->connection()->IsAlive())
+            {
+                wxCookieType cookie2;
+                wxTreeItemId item = browser->GetFirstChild(serverItem, cookie2);
+                while (item)
+                {
+                    pgObject *obj=(pgObject*)browser->GetItemData(item);
+                    if (obj && obj->GetType() == PG_DATABASES)
+                    {
+                        wxCookieType cookie3;
+                        item = browser->GetFirstChild(obj->GetId(), cookie3);
+                        while (item)
+                        {
+                            pgDatabase *db=(pgDatabase*)browser->GetItemData(item);
+                            if (db && db->GetType() == PG_DATABASE)
+                            {
+                                pgConn *conn=db->GetConnection();
+                                if (conn)
+                                {
+                                    if (!conn->IsAlive() && conn->GetStatus() == PGCONN_BROKEN)
+                                    {
+                                        conn->Close();
+                                        if (!userInformed)
+                                        {
+                                            wxMessageDialog dlg(this, _("Close database browser? If you abort, the object browser will not show accurate data."),
+                                            wxString::Format(_("Connection to database %s lost."), db->GetName().c_str()), 
+                                                wxICON_EXCLAMATION|wxYES_NO|wxYES_DEFAULT);
+
+                                            closeIt = (dlg.ShowModal() == wxID_YES);
+                                            userInformed = true;
+                                        }
+                                        if (closeIt)
+                                        {
+                                            browser->DeleteChildren(db->GetId());
+                                            browser->SetItemImage(db->GetId(), PGICON_CLOSEDDATABASE, wxTreeItemIcon_Selected);
+                                            browser->SetItemImage(db->GetId(), PGICON_CLOSEDDATABASE, wxTreeItemIcon_Selected);
+                                            db->Disconnect();
+                                        }
+                                    }
+                                }
+                            }
+                            item = browser->GetNextChild(obj->GetId(), cookie3);
+                        }
+                    }
+                    item = browser->GetNextChild(serverItem, cookie2);
+                }
+            }
+            else
+            {
+                if (server->connection()->GetStatus() == PGCONN_BROKEN)
+                {
+                    server->connection()->Close();
+                    if (!userInformed)
+                    {
+                        wxMessageDialog dlg(this, _("Close server browser? If you abort, the object browser will not show accurate data."),
+                            wxString::Format(_("Connection to server %s lost."), server->GetName().c_str()), 
+                            wxICON_EXCLAMATION|wxYES_NO|wxYES_DEFAULT);
+
+                        closeIt = (dlg.ShowModal() == wxID_YES);
+                        userInformed = true;
+                    }
+                    if (closeIt)
+                    {
+
+                        browser->SelectItem(serverItem);
+                        browser->SetItemImage(serverItem, PGICON_SERVERBAD, wxTreeItemIcon_Normal);
+                        browser->SetItemImage(serverItem, PGICON_SERVERBAD, wxTreeItemIcon_Selected);
+                        server->Disconnect();
+                        execSelChange(serverItem, true);
+                        browser->DeleteChildren(serverItem);
+                    }
+                }
+            }
+        }
+
+        serverItem = browser->GetNextChild(servers, cookie);
+    }
+    return userInformed;
+}
+
+
 wxTreeItemId frmMain::RestoreEnvironment(pgServer *server)
 {
     wxTreeItemId item, lastItem;
@@ -428,7 +623,7 @@ wxTreeItemId frmMain::RestoreEnvironment(pgServer *server)
         return item;
 
     // found DATABASES item
-    data->ShowTree(this, browser, 0, 0, 0);
+    data->ShowTree(this, browser, 0, 0);
     lastItem=item;
 
     item = browser->GetFirstChild(lastItem, cookie);
@@ -444,7 +639,7 @@ wxTreeItemId frmMain::RestoreEnvironment(pgServer *server)
         return lastItem;
 
     // found last DATABASE 
-    data->ShowTree(this, browser, 0, 0, 0);
+    data->ShowTree(this, browser, 0, 0);
     lastItem = item;
 
     wxString lastSchema=server->GetLastSchema();
@@ -464,7 +659,7 @@ wxTreeItemId frmMain::RestoreEnvironment(pgServer *server)
         return lastItem;
 
     // found SCHEMAS item
-    data->ShowTree(this, browser, 0, 0, 0);
+    data->ShowTree(this, browser, 0, 0);
     lastItem=item;
 
     item = browser->GetFirstChild(lastItem, cookie);
@@ -532,57 +727,71 @@ void frmMain::StoreServers()
     wxLogInfo(wxT("Storing listed servers for later..."));
 
     // Store the currently listed servers for later retrieval.
-
-    // Write the individual servers
-    // Iterate through all the child nodes of the Servers node
     wxCookieType cookie;
     wxString key;
     pgObject *data;
     pgServer *server;
     int numServers = 0;
 
+	// Get the hostname for later...
+	char buf[255];
+	gethostname(buf, 255); 
+    wxString hostname = wxString(buf, wxConvUTF8);
+
+    // Write the individual servers
+    // Iterate through all the child nodes of the Servers node
     wxTreeItemId item = browser->GetFirstChild(servers, cookie);
     while (item) {
+
         data = (pgObject *)browser->GetItemData(item);
         if (data->GetType() == PG_SERVER) {
-            // We have a sever, so cast the object and save the settings
-            ++numServers;
-            server = (pgServer *)data;
 
-            // Hostname
-            key.Printf(wxT("Servers/Server%d"), numServers);
-            settings->Write(key, server->GetName());
+			// Cast the object, and check if it was autodiscovered before saving.
+			server = (pgServer *)data;
 
-            key.Printf(wxT("Servers/Description%d"), numServers);
-            settings->Write(key, server->GetDescription());
+			// We have an 'added' server, so save it
+			++numServers;
 
-            // Port
-            key.Printf(wxT("Servers/Port%d"), numServers);
-            settings->Write(key, server->GetPort());
+			// Hostname
+			key.Printf(wxT("Servers/Server%d"), numServers);
+		    settings->Write(key, server->GetName());
 
-            // Trusted
-            key.Printf(wxT("Servers/Trusted%d"), numServers);
-            settings->Write(key, BoolToStr(server->GetTrusted()));
+			// Comment
+		    key.Printf(wxT("Servers/Description%d"), numServers);
+	        settings->Write(key, server->GetDescription());
 
-            // Database
-            key.Printf(wxT("Servers/Database%d"), numServers);
-            settings->Write(key, server->GetDatabase());
+			// Service ID
+		    key.Printf(wxT("Servers/ServiceID%d"), numServers);
+	        settings->Write(key, server->GetServiceID());
 
-            // Username
-            key.Printf(wxT("Servers/Username%d"), numServers);
-            settings->Write(key, server->GetUsername());
+			// Port
+			key.Printf(wxT("Servers/Port%d"), numServers);
+		    settings->Write(key, server->GetPort());
 
-            // last Database
-            key.Printf(wxT("Servers/LastDatabase%d"), numServers);
-            settings->Write(key, server->GetLastDatabase());
+			// Trusted
+			key.Printf(wxT("Servers/Trusted%d"), numServers);
+	        settings->Write(key, BoolToStr(server->GetTrusted()));
 
-            // last Schema
-            key.Printf(wxT("Servers/LastSchema%d"), numServers);
-            settings->Write(key, server->GetLastSchema());
-            
-            key.Printf(wxT("Servers/SSL%d"), numServers);
-            settings->Write(key, server->GetSSL());
-        }
+			// Database
+			key.Printf(wxT("Servers/Database%d"), numServers);
+	        settings->Write(key, server->GetDatabaseName());
+
+			// Username
+			key.Printf(wxT("Servers/Username%d"), numServers);
+	        settings->Write(key, server->GetUsername());
+
+			// last Database
+			key.Printf(wxT("Servers/LastDatabase%d"), numServers);
+			settings->Write(key, server->GetLastDatabase());
+
+			// last Schema
+			key.Printf(wxT("Servers/LastSchema%d"), numServers);
+			settings->Write(key, server->GetLastSchema());
+	        
+			// SSL
+			key.Printf(wxT("Servers/SSL%d"), numServers);
+			settings->Write(key, server->GetSSL());
+		}
 
         // Get the next item
         item = browser->GetNextChild(servers, cookie);
@@ -590,28 +799,38 @@ void frmMain::StoreServers()
 
     // Write the server count
     settings->Write(wxT("Servers/Count"), numServers);
-
     wxLogInfo(wxT("Stored %d servers."), numServers);
 }
 
+
 void frmMain::RetrieveServers()
 {
     // Retrieve previously stored servers
     wxLogInfo(wxT("Reloading servers..."));
 
-    int numServers;
-    settings->Read(wxT("Servers/Count"), &numServers, 0);
+    long numServers=settings->Read(wxT("Servers/Count"), 0L);
 
-    int loop, port, ssl=0;
-    wxString key, servername, description, database, username, lastDatabase, lastSchema, trusted;
+    long loop, port, ssl=0;
+    wxString key, servername, description, database, username, lastDatabase, lastSchema, trusted, serviceID;
     pgServer *server;
 
+    wxArrayString servicedServers;
+
+	// Get the hostname for later...
+	char buf[255];
+	gethostname(buf, 255); 
+    wxString hostname = wxString(buf, wxConvUTF8);
+
     for (loop = 1; loop <= numServers; ++loop) {
         
         // Server
         key.Printf(wxT("Servers/Server%d"), loop);
         settings->Read(key, &servername, wxT(""));
 
+        // service location
+        key.Printf(wxT("Servers/ServiceID%d"), loop);
+        settings->Read(key, &serviceID, wxT(""));
+
         // Comment
         key.Printf(wxT("Servers/Description%d"), loop);
         settings->Read(key, &description, wxT(""));
@@ -647,28 +866,127 @@ void frmMain::RetrieveServers()
 #endif
 
         // Add the Server node
-        server = new pgServer(servername, database, username, port, StrToBool(trusted), ssl);
-        server->iSetDescription(description);
+        server = new pgServer(servername, description, database, username, port, StrToBool(trusted), ssl);
         server->iSetLastDatabase(lastDatabase);
         server->iSetLastSchema(lastSchema);
+        server->iSetServiceID(serviceID);
+		server->iSetDiscovered(false);
         browser->AppendItem(servers, server->GetFullName(), PGICON_SERVERBAD, -1, server);
+
+
+#ifdef WIN32
+        int bspos = serviceID.Find('\\');
+        if (bspos >= 0)
+        {
+            if (serviceID.Left(2) != wxT(".\\") && !serviceID.Matches(wxGetHostName() + wxT("\\*")))
+                serviceID = wxEmptyString;
+        }
+        if (!serviceID.IsEmpty())
+            servicedServers.Add(serviceID);
+#endif
+
     }
 
+#ifdef WIN32
+
+	// Add local servers. Will currently only work on Win32 with >= BETA3 
+	// of the Win32 PostgreSQL installer.
+	wxRegKey *pgKey = new wxRegKey(wxT("HKEY_LOCAL_MACHINE\\Software\\PostgreSQL\\Services"));
+
+	if (pgKey->Exists())
+	{
+
+		wxString svcName, temp;
+		long cookie = 0;
+		long *tmpport = 0;
+		bool flag = false;
+
+		flag = pgKey->GetFirstKey(svcName, cookie);
+
+		while (flag != false)
+		{
+            if (servicedServers.Index(svcName, false) < 0)
+            {
+			    key.Printf(wxT("HKEY_LOCAL_MACHINE\\Software\\PostgreSQL\\Services\\%s"), svcName);
+			    wxRegKey *svcKey = new wxRegKey(key);
+
+                servername = wxT("localhost");
+                database = wxT("template1");
+			    svcKey->QueryValue(wxT("Display Name"), description);
+			    svcKey->QueryValue(wxT("Database Superuser"), username);
+                svcKey->QueryValue(wxT("Port"), &port);
+
+			    // Add the Server node
+			    server = new pgServer(servername, description, database, username, port, false, 0);
+			    server->iSetDiscovered(true);
+			    server->iSetServiceID(svcName);
+			    browser->AppendItem(servers, server->GetFullName(), PGICON_SERVERBAD, -1, server);
+            }
+			// Get the next one...
+			flag = pgKey->GetNextKey(svcName, cookie);
+		}
+	}
+#endif //WIN32
+
     // Reset the Servers node text
     wxString label;
     label.Printf(_("Servers (%d)"), browser->GetChildrenCount(servers, FALSE));
     browser->SetItemText(servers, label);
+
 }
 
 
-void frmMain::SetButtons(bool refresh, bool create, bool drop, bool properties, bool sql, bool viewData, bool maintenance)
+void frmMain::SetButtons(pgObject *obj)
 {
+    bool refresh=false,
+         create=false,
+         drop=false,
+         properties=false,
+         sql=false,
+         viewData=false,
+         maintenance=false,
+         backup=false,
+         restore=false,
+         status=false;
+
+    if (obj)
+    {
+        refresh=true;
+        create=obj->CanCreate();
+        drop=obj->CanDrop();
+        properties=obj->CanEdit();
+        viewData=obj->CanView();
+        maintenance=obj->CanMaintenance();
+        backup=obj->CanBackup();
+        restore=obj->CanRestore();
+        status=obj->GetServer() != 0;
+
+        switch (obj->GetType())
+        {
+            case PG_SERVERS:
+            case PG_SERVER:
+            case PG_DATABASES:
+            case PG_TABLESPACES:
+            case PG_TABLESPACE:
+            case PG_GROUPS:
+            case PG_GROUP:
+            case PG_USERS:
+            case PG_USER:
+                sql=false;
+                break;
+            default:
+                sql=true;
+                break;
+        }
+    }
+
     toolBar->EnableTool(MNU_REFRESH, refresh);
     toolBar->EnableTool(MNU_CREATE, create);
     toolBar->EnableTool(MNU_DROP, drop);
     toolBar->EnableTool(MNU_PROPERTIES, properties);
     toolBar->EnableTool(MNU_SQL, sql);
     toolBar->EnableTool(MNU_VIEWDATA, viewData);
+    toolBar->EnableTool(MNU_VIEWFILTEREDDATA, viewData);
     toolBar->EnableTool(MNU_MAINTENANCE, maintenance);
 
 	// Handle the menus associated with the buttons
@@ -678,23 +996,59 @@ void frmMain::SetButtons(bool refresh, bool create, bool drop, bool properties,
 	toolsMenu->Enable(MNU_CONNECT, false);
 	toolsMenu->Enable(MNU_DISCONNECT, false);
 	toolsMenu->Enable(MNU_SQL, sql);
-#ifdef __DEBUG_QUERYBUILDER__
-	toolsMenu->Enable(MNU_QUERYBUILDER, sql);
-#endif
 	toolsMenu->Enable(MNU_MAINTENANCE, maintenance);
-	toolsMenu->Enable(MNU_STATUS, sql);
+	toolsMenu->Enable(MNU_BACKUP, backup && !backupExecutable.IsNull());
+	toolsMenu->Enable(MNU_RESTORE, restore && !restoreExecutable.IsNull());
+//    toolsMenu->Enable(MNU_INDEXCHECK, false);
+    toolsMenu->Enable(MNU_GRANTWIZARD, false);
+	toolsMenu->Enable(MNU_STATUS, status);
 	toolsMenu->Enable(MNU_VIEWDATA, viewData);
+	toolsMenu->Enable(MNU_VIEWFILTEREDDATA, viewData);
+    toolsMenu->Enable(MNU_STARTSERVICE, false);
+    toolsMenu->Enable(MNU_STOPSERVICE, false);
 	viewMenu->Enable(MNU_REFRESH, refresh);
-    toolsMenu->Enable(MNU_RELOAD, false);
-
-	treeContextMenu->Enable(MNU_CREATE, create);
-	treeContextMenu->Enable(MNU_DROP, drop);
-	treeContextMenu->Enable(MNU_CONNECT, false);
-	treeContextMenu->Enable(MNU_DISCONNECT, false);
-	treeContextMenu->Enable(MNU_REFRESH, refresh);
-	treeContextMenu->Enable(MNU_PROPERTIES, properties);
-	treeContextMenu->Enable(MNU_MAINTENANCE, maintenance);
-	treeContextMenu->Enable(MNU_VIEWDATA, viewData);
-    treeContextMenu->Enable(MNU_RELOAD, false);
+	viewMenu->Enable(MNU_COUNT, false);
 }
 
+
+
+
+void frmMain::StartMsg(const wxString& msg)
+{
+   if (msgLevel++)
+        return;
+
+    timermsg.Printf(wxT("%s..."), msg.c_str());
+    wxBeginBusyCursor();
+    stopwatch.Start(0);
+    wxLogStatus(timermsg);
+    statusBar->SetStatusText(timermsg, 1);
+    statusBar->SetStatusText(wxT(""), 2);
+}
+
+
+void frmMain::EndMsg()
+{
+    msgLevel--;
+
+    if (!msgLevel)
+    {
+        // Get the execution time & display it
+        float timeval = stopwatch.Time();
+        wxString time;
+        time.Printf(_("%.2f secs"), (timeval/1000));
+        statusBar->SetStatusText(time, 2);
+
+        // Display the 'Done' message
+        statusBar->SetStatusText(timermsg + _(" Done."), 1);
+        wxLogStatus(wxT("%s (%s)"), timermsg.c_str(), time.c_str());
+        wxEndBusyCursor();
+    }
+}
+
+
+void frmMain::SetStatusText(const wxString &msg)
+{
+    statusBar->SetStatusText(msg, 1);
+    statusBar->SetStatusText(wxEmptyString, 2);
+}
