@@ -1,7 +1,8 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: frmQuery.cpp,v 1.78 2004/11/26 11:15:21 dpage Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // frmQuery.cpp - SQL Query Box
@@ -19,6 +20,8 @@
 #include "frmHelp.h"
 #include "menu.h"
 
+#include <wx/clipbrd.h>
+
 // Icons
 #include "images/sql.xpm"
 // Bitmaps
@@ -32,28 +35,20 @@
 #include "images/edit_undo.xpm"
 #include "images/edit_redo.xpm"
 #include "images/query_execute.xpm"
+#include "images/query_execfile.xpm"
 #include "images/query_explain.xpm"
 #include "images/query_cancel.xpm"
 #include "images/help.xpm"
 
 
 
-BEGIN_EVENT_TABLE(frmQuery, wxFrame)
+BEGIN_EVENT_TABLE(frmQuery, pgFrame)
     EVT_CLOSE(                      frmQuery::OnClose)
+    EVT_SET_FOCUS(                  frmQuery::OnSetFocus)
     EVT_MENU(MNU_OPEN,              frmQuery::OnOpen)
     EVT_MENU(MNU_SAVE,              frmQuery::OnSave)
     EVT_MENU(MNU_SAVEAS,            frmQuery::OnSaveAs)
     EVT_MENU(MNU_EXPORT,            frmQuery::OnExport)
-    EVT_MENU(MNU_RECENT+1,          frmQuery::OnRecent)
-    EVT_MENU(MNU_RECENT+2,          frmQuery::OnRecent)
-    EVT_MENU(MNU_RECENT+3,          frmQuery::OnRecent)
-    EVT_MENU(MNU_RECENT+4,          frmQuery::OnRecent)
-    EVT_MENU(MNU_RECENT+5,          frmQuery::OnRecent)
-    EVT_MENU(MNU_RECENT+6,          frmQuery::OnRecent)
-    EVT_MENU(MNU_RECENT+7,          frmQuery::OnRecent)
-    EVT_MENU(MNU_RECENT+8,          frmQuery::OnRecent)
-    EVT_MENU(MNU_RECENT+9,          frmQuery::OnRecent)
-    EVT_MENU(MNU_EXIT,              frmQuery::OnExit)
     EVT_MENU(MNU_CUT,               frmQuery::OnCut)
     EVT_MENU(MNU_COPY,              frmQuery::OnCopy)
     EVT_MENU(MNU_PASTE,             frmQuery::OnPaste)
@@ -62,6 +57,7 @@ BEGIN_EVENT_TABLE(frmQuery, wxFrame)
     EVT_MENU(MNU_UNDO,              frmQuery::OnUndo)
     EVT_MENU(MNU_REDO,              frmQuery::OnRedo)
     EVT_MENU(MNU_EXECUTE,           frmQuery::OnExecute)
+    EVT_MENU(MNU_EXECFILE,          frmQuery::OnExecFile)
     EVT_MENU(MNU_EXPLAIN,           frmQuery::OnExplain)
     EVT_MENU(MNU_CANCEL,            frmQuery::OnCancel)
     EVT_MENU(MNU_CONTENTS,          frmQuery::OnContents)
@@ -69,23 +65,25 @@ BEGIN_EVENT_TABLE(frmQuery, wxFrame)
     EVT_MENU(MNU_CLEARHISTORY,      frmQuery::OnClearHistory)
     EVT_MENU(MNU_SAVEHISTORY,       frmQuery::OnSaveHistory)
     EVT_ACTIVATE(                   frmQuery::OnActivate)
-#ifdef __WXGTK__
-    EVT_KEY_DOWN(                   frmQuery::OnKeyDown)
-#endif
-    EVT_STC_MODIFIED(CTL_SQLQUERY,    frmQuery::OnChange)
+    EVT_STC_MODIFIED(CTL_SQLQUERY,  frmQuery::OnChangeStc)
 END_EVENT_TABLE()
 
 
 
 
-frmQuery::frmQuery(frmMain *form, const wxString& _title, pgConn *_conn, const wxPoint& pos, const wxSize& size, const wxString& query)
-: wxFrame(NULL, -1, _title, pos, size)
+frmQuery::frmQuery(frmMain *form, const wxString& _title, pgConn *_conn, const wxString& query)
+: pgFrame(NULL, _title)
 {
     mainForm=form;
     title = _title;
     conn=_conn;
 
+    dlgName = wxT("frmQuery");
+    recentKey = wxT("RecentFiles");
+    RestorePosition(100, 100, 600, 500, 200, 150);
+
     SetIcon(wxIcon(sql_xpm));
+    wxWindowBase::SetFont(settings->GetSystemFont());
     wxLogInfo(wxT("Creating SQL Query box"));
     menuBar = new wxMenuBar();
 
@@ -116,6 +114,7 @@ frmQuery::frmQuery(frmMain *form, const wxString& _title, pgConn *_conn, const w
 
     queryMenu = new wxMenu();
     queryMenu->Append(MNU_EXECUTE, _("&Execute\tF5"), _("Execute query"));
+    queryMenu->Append(MNU_EXECFILE, _("Execute to file"), _("Execute query, write result to file"));
     queryMenu->Append(MNU_EXPLAIN, _("E&xplain\tF7"), _("Explain query"));
 
     wxMenu *eo=new wxMenu();
@@ -140,25 +139,27 @@ frmQuery::frmQuery(frmMain *form, const wxString& _title, pgConn *_conn, const w
     queryMenu->Check(MNU_VERBOSE, settings->GetExplainVerbose());
     queryMenu->Check(MNU_ANALYZE, settings->GetExplainAnalyze());
 
-    updateRecentFiles();
+    UpdateRecentFiles();
 
-    wxAcceleratorEntry entries[7];
+    wxAcceleratorEntry entries[8];
 
-    entries[0].Set(wxACCEL_CTRL,                (int)'O',      MNU_OPEN);
-    entries[1].Set(wxACCEL_CTRL,                (int)'S',      MNU_SAVE);
-    entries[2].Set(wxACCEL_CTRL,                (int)'F',      MNU_FIND);
-    entries[3].Set(wxACCEL_NORMAL,              WXK_F5,        MNU_EXECUTE);
-    entries[4].Set(wxACCEL_NORMAL,              WXK_F7,        MNU_EXPLAIN);
-    entries[5].Set(wxACCEL_ALT,                 WXK_PAUSE,     MNU_CANCEL);
-    entries[6].Set(wxACCEL_NORMAL,              WXK_F1,        MNU_HELP);
+    entries[0].Set(wxACCEL_CTRL,                (int)'E',      MNU_EXECUTE);
+    entries[1].Set(wxACCEL_CTRL,                (int)'O',      MNU_OPEN);
+    entries[2].Set(wxACCEL_CTRL,                (int)'S',      MNU_SAVE);
+    entries[3].Set(wxACCEL_CTRL,                (int)'F',      MNU_FIND);
+    entries[4].Set(wxACCEL_NORMAL,              WXK_F5,        MNU_EXECUTE);
+    entries[5].Set(wxACCEL_NORMAL,              WXK_F7,        MNU_EXPLAIN);
+    entries[6].Set(wxACCEL_ALT,                 WXK_PAUSE,     MNU_CANCEL);
+    entries[7].Set(wxACCEL_NORMAL,              WXK_F1,        MNU_HELP);
 
-    wxAcceleratorTable accel(7, entries);
+    wxAcceleratorTable accel(8, entries);
     SetAcceleratorTable(accel);
 
     queryMenu->Enable(MNU_CANCEL, false);
 
     int iWidths[4] = {0, -1, 110, 110};
     statusBar=CreateStatusBar(4);
+    SetStatusBarPane(-1);
     SetStatusWidths(4, iWidths);
     SetStatusText(_("ready"), STATUSPOS_MSGS);
 
@@ -180,6 +181,7 @@ frmQuery::frmQuery(frmMain *form, const wxString& _title, pgConn *_conn, const w
     toolBar->AddTool(MNU_FIND, _("Find"), wxBitmap(edit_find_xpm), _("Find text"), wxITEM_NORMAL);
     toolBar->AddSeparator();
     toolBar->AddTool(MNU_EXECUTE, _("Execute"), wxBitmap(query_execute_xpm), _("Execute query"), wxITEM_NORMAL);
+    toolBar->AddTool(MNU_EXECFILE, _("Execute to file"), wxBitmap(query_execfile_xpm), _("Execute query, write result to file"), wxITEM_NORMAL);
     toolBar->AddTool(MNU_EXPLAIN, _("Explain"), wxBitmap(query_explain_xpm), _("Explain query"), wxITEM_NORMAL);
     toolBar->AddTool(MNU_CANCEL, _("Cancel"), wxBitmap(query_cancel_xpm), _("Cancel query"), wxITEM_NORMAL);
     toolBar->AddSeparator();
@@ -190,6 +192,7 @@ frmQuery::frmQuery(frmMain *form, const wxString& _title, pgConn *_conn, const w
     horizontal->SetMinimumPaneSize(50);
 
     sqlQuery = new ctlSQLBox(horizontal, CTL_SQLQUERY, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE | wxSIMPLE_BORDER | wxTE_RICH2);
+    sqlQuery->SetMarginWidth(1, 16);
 
     output = new wxNotebook(horizontal, -1, wxDefaultPosition, wxDefaultSize, wxNB_BOTTOM);
     sqlResult = new ctlSQLResult(output, conn, CTL_SQLRESULT, wxDefaultPosition, wxDefaultSize);
@@ -200,6 +203,11 @@ frmQuery::frmQuery(frmMain *form, const wxString& _title, pgConn *_conn, const w
     output->AddPage(msgResult, _("Messages"));
     output->AddPage(msgHistory, _("History"));
 
+    sqlQuery->Connect(wxID_ANY, wxEVT_SET_FOCUS,(wxObjectEventFunction)(wxEventFunction)(wxCommandEventFunction)&frmQuery::OnFocus);
+    sqlResult->Connect(wxID_ANY, wxEVT_SET_FOCUS, (wxObjectEventFunction)(wxEventFunction)(wxCommandEventFunction)&frmQuery::OnFocus);
+    msgResult->Connect(wxID_ANY, wxEVT_SET_FOCUS, (wxObjectEventFunction)(wxEventFunction)(wxCommandEventFunction)&frmQuery::OnFocus);
+    msgHistory->Connect(wxID_ANY, wxEVT_SET_FOCUS, (wxObjectEventFunction)(wxEventFunction)(wxCommandEventFunction)&frmQuery::OnFocus);
+
     int splitpos=settings->Read(wxT("frmQuery/Split"), 250);
     if (splitpos < 50)
         splitpos = 50;
@@ -208,6 +216,8 @@ frmQuery::frmQuery(frmMain *form, const wxString& _title, pgConn *_conn, const w
     horizontal->SplitHorizontally(sqlQuery, output, splitpos);
 
     sqlQuery->SetText(query);
+    sqlQuery->Colourise(0, query.Length());
+
     changed = !query.IsNull() && settings->GetStickySql();
     if (changed)
         setExtendedTitle();
@@ -225,89 +235,29 @@ frmQuery::frmQuery(frmMain *form, const wxString& _title, pgConn *_conn, const w
 frmQuery::~frmQuery()
 {
     wxLogInfo(wxT("Destroying SQL Query box"));
+
+    sqlQuery->Disconnect(wxID_ANY, wxEVT_SET_FOCUS,(wxObjectEventFunction)(wxEventFunction)(wxCommandEventFunction)&frmQuery::OnFocus);
+    sqlResult->Disconnect(wxID_ANY, wxEVT_SET_FOCUS, (wxObjectEventFunction)(wxEventFunction)(wxCommandEventFunction)&frmQuery::OnFocus);
+    msgResult->Disconnect(wxID_ANY, wxEVT_SET_FOCUS, (wxObjectEventFunction)(wxEventFunction)(wxCommandEventFunction)&frmQuery::OnFocus);
+    msgHistory->Disconnect(wxID_ANY, wxEVT_SET_FOCUS, (wxObjectEventFunction)(wxEventFunction)(wxCommandEventFunction)&frmQuery::OnFocus);
+
     mainForm->RemoveFrame(this);
-    settings->Write(wxT("frmQuery/Width"), GetSize().x);
-    settings->Write(wxT("frmQuery/Height"), GetSize().y);
-    settings->Write(wxT("frmQuery/Left"), GetPosition().x);
-    settings->Write(wxT("frmQuery/Top"), GetPosition().y);
+
     settings->Write(wxT("frmQuery/Split"), horizontal->GetSashPosition());
     settings->SetExplainAnalyze(queryMenu->IsChecked(MNU_ANALYZE));
     settings->SetExplainVerbose(queryMenu->IsChecked(MNU_VERBOSE));
+
+    sqlResult->Abort(); // to make sure conn is unused
     if (conn)
         delete conn;
 }
 
 
-
-void frmQuery::updateRecentFiles()
-{
-    wxString lastFiles[10]; // 0 will be unused for convenience
-    int i, maxFiles=9;
-    int recentIndex=maxFiles;
-
-    for (i=1 ; i <= maxFiles ; i++)
-    {
-        lastFiles[i] = settings->Read(wxT("RecentFiles/") + wxString::Format(wxT("%d"), i), wxT(""));
-        if (!lastPath.IsNull() && lastPath.IsSameAs(lastFiles[i], wxARE_FILENAMES_CASE_SENSITIVE))
-            recentIndex=i;
-    }
-    while (i <= maxFiles)
-        lastFiles[i++] = wxT("");
-
-    if (recentIndex > 1 && !lastPath.IsNull())
-    {
-        for (i=recentIndex ; i > 1 ; i--)
-            lastFiles[i] = lastFiles[i-1];
-        lastFiles[1] = lastPath;
-    }
-
-    i=recentFileMenu->GetMenuItemCount();
-    while (i)
-    {
-        wxMenuItem *item = recentFileMenu->Remove(MNU_RECENT+i);
-        if (item)
-            delete item;
-        i--;
-    }
-
-    for (i=1 ; i <= maxFiles ; i++)
-    {
-        settings->Write(wxT("RecentFiles/") + wxString::Format(wxT("%d"), i), lastFiles[i]);
-
-
-        if (!lastFiles[i].IsNull())
-            recentFileMenu->Append(MNU_RECENT+i, wxT("&") + wxString::Format(wxT("%d"), i) + wxT("  ") + lastFiles[i]);
-    }
-}
-
-
 void frmQuery::OnActivate(wxActivateEvent& event)
 {
     if (event.GetActive())
         updateMenu();
-}
-
-
-void frmQuery::OnRecent(wxCommandEvent& event)
-{
-    int fileNo=event.GetId() - MNU_RECENT;
-    lastPath = settings->Read(wxT("RecentFiles/") + wxString::Format(wxT("%d"), fileNo), wxT(""));
-
-    if (!lastPath.IsNull())
-    {
-        int dirsep;
-        dirsep = lastPath.Find(wxFILE_SEP_PATH, true);
-        lastDir = lastPath.Mid(0, dirsep);
-        lastFilename = lastPath.Mid(dirsep+1);
-        openLastFile();
-    }
-}
-
-
-void frmQuery::OnKeyDown(wxKeyEvent& event)
-{
-    event.m_metaDown=false;
-    event.Skip();
+	event.Skip();
 }
 
 
@@ -324,12 +274,6 @@ void frmQuery::Go()
 }
 
 
-void frmQuery::OnExit(wxCommandEvent& event)
-{
-    Close();
-}
-
-
 typedef struct __sqltokenhelp
 {
     wxChar *token;
@@ -479,6 +423,14 @@ void frmQuery::OnSaveHistory(wxCommandEvent& event)
 
 }
 
+
+void frmQuery::OnSetFocus(wxFocusEvent& event)
+{
+	sqlQuery->SetFocus();
+	event.Skip();
+}
+
+
 void frmQuery::OnClearHistory(wxCommandEvent& event)
 {
     queryMenu->Enable(MNU_SAVEHISTORY, false);
@@ -486,26 +438,98 @@ void frmQuery::OnClearHistory(wxCommandEvent& event)
     msgHistory->Clear();
 }
 
+
+void frmQuery::OnFocus(wxFocusEvent& ev)
+{
+    if (wxDynamicCast(this, wxFrame))
+        updateMenu(ev.GetEventObject());
+    else
+    {
+        frmQuery *wnd=(frmQuery*)GetParent();
+        if (wnd)
+            wnd->OnFocus(ev);
+    }
+    ev.Skip();
+}
+
+
 void frmQuery::OnCut(wxCommandEvent& ev)
 {
-    sqlQuery->Cut();
-    updateMenu();
+    if (currentControl() == sqlQuery)
+    {
+        sqlQuery->Cut();
+        updateMenu();
+    }
 }
 
+
+wxWindow *frmQuery::currentControl()
+{
+    wxWindow *wnd=FindFocus();
+    if (wnd == output)
+    {
+        switch (output->GetSelection())
+        {
+            case 0: wnd = sqlResult;    break;
+            case 1: wnd = msgResult;    break;
+            case 2: wnd = msgHistory;   break;
+        }
+    }
+    return wnd;
+}
+
+
 void frmQuery::OnCopy(wxCommandEvent& ev)
 {
-    sqlQuery->Copy();
+    wxWindow *wnd=currentControl();
+
+    if (wnd == sqlQuery)
+        sqlQuery->Copy();
+    else if (wnd == msgResult)
+        msgResult->Copy();
+    else if (wnd == msgHistory)
+        msgHistory->Copy();
+    else if (wnd == sqlResult && sqlResult->GetSelectedItemCount() > 0)
+    {
+        wxString str;
+        int row=-1;
+        while (true)
+        {
+            row = sqlResult->GetNextItem(row, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
+            if (row < 0)
+                break;
+            
+            str.Append(sqlResult->GetExportLine(row));
+            if (sqlResult->GetSelectedItemCount() > 1)
+                str.Append(END_OF_LINE);
+        }
+
+        if (wxTheClipboard->Open())
+        {
+            wxTheClipboard->SetData(new wxTextDataObject(str));
+            wxTheClipboard->Close();
+        }
+
+    }
     updateMenu();
 }
 
 void frmQuery::OnPaste(wxCommandEvent& ev)
 {
-    sqlQuery->Paste();
+	if (currentControl() == sqlQuery)
+	  sqlQuery->Paste();
 }
 
 void frmQuery::OnClear(wxCommandEvent& ev)
 {
-    sqlQuery->ClearAll();
+    wxWindow *wnd=currentControl();
+
+    if (wnd == sqlQuery)
+        sqlQuery->ClearAll();
+    else if (wnd == msgResult)
+        msgResult->Clear();
+    else if (wnd == msgHistory)
+        msgHistory->Clear();
 }
 
 void frmQuery::OnFind(wxCommandEvent& ev)
@@ -538,23 +562,54 @@ void frmQuery::setExtendedTitle()
 }
 
 
-void frmQuery::updateMenu()
+void frmQuery::updateMenu(wxObject *obj)
 {
-    bool canUndo=sqlQuery->CanUndo();
+    bool canCut=false;
+    bool canPaste=false;
+    bool canUndo=false;
+    bool canRedo=false;
+    bool canClear=false;
+    bool canFind=false;
+
+
+    if (!obj || obj == sqlQuery)
+    {
+        canUndo=sqlQuery->CanUndo();
+        canRedo=sqlQuery->CanRedo();
+        canPaste=sqlQuery->CanPaste();
+
+        canCut = true;
+        canClear = true;
+        canFind = true;
+    }
+    else if (obj == msgResult || obj == msgHistory)
+    {
+        canClear = true;
+    }
+
+
     toolBar->EnableTool(MNU_UNDO, canUndo);
     editMenu->Enable(MNU_UNDO, canUndo);
 
-    bool canRedo=sqlQuery->CanRedo();
     toolBar->EnableTool(MNU_REDO, canRedo);
     editMenu->Enable(MNU_REDO, canRedo);
 
-    bool canPaste=sqlQuery->CanPaste();
     toolBar->EnableTool(MNU_PASTE, canPaste);
     editMenu->Enable(MNU_PASTE, canPaste);
+
+    toolBar->EnableTool(MNU_CUT, canCut);
+    editMenu->Enable(MNU_CUT, canCut);
+
+    toolBar->EnableTool(MNU_CLEAR, canClear);
+    editMenu->Enable(MNU_CLEAR, canClear);
+
+    toolBar->EnableTool(MNU_FIND, canFind);
+    editMenu->Enable(MNU_FIND, canFind);
 }
 
 
-void frmQuery::OnClose(wxCloseEvent& event)
+
+bool frmQuery::CheckChanged(bool canVeto)
 {
     if (changed && settings->GetAskSaveConfirmation())
     {
@@ -563,31 +618,53 @@ void frmQuery::OnClose(wxCloseEvent& event)
             fn = wxT(" in file ") + lastPath;
         wxMessageDialog msg(this, wxString::Format(_("The text %s has changed.\nDo you want to save changes?"), fn.c_str()), _("pgAdmin III Query"), 
                     wxYES_NO|wxNO_DEFAULT|wxICON_EXCLAMATION|
-                    (event.CanVeto() ? wxCANCEL : 0));
+                    (canVeto ? wxCANCEL : 0));
 
     	wxCommandEvent noEvent;
         switch (msg.ShowModal())
         {
             case wxID_YES:
                 if (lastPath.IsNull())
-                {
                     OnSaveAs(noEvent);
-                    if (changed && event.CanVeto())
-                        event.Veto();
-                }
                 else
                     OnSave(noEvent);
-                break;
+
+                return changed;
+
             case wxID_CANCEL:
-                event.Veto();
-                return;
+                return true;
         }
     }
+    return false;
+}
+
+
+void frmQuery::OnClose(wxCloseEvent& event)
+{
+    if (CheckChanged(event.CanVeto()) && event.CanVeto())
+    {
+        event.Veto();
+        return;
+    }
+
+    Hide();
+
+    if (queryMenu->IsEnabled(MNU_CANCEL))
+    {
+        wxCommandEvent ev;
+        OnCancel(ev);
+    }
+
+    while (sqlResult->RunStatus() == CTLSQL_RUNNING)
+    {
+        wxLogInfo(wxT("SQL Query box: Waiting for query to abort"));
+        wxSleep(1);
+    }
     Destroy();
 }
 
 
-void frmQuery::OnChange(wxNotifyEvent& event)
+void frmQuery::OnChangeStc(wxStyledTextEvent& event)
 {
     if (!changed)
     {
@@ -598,41 +675,33 @@ void frmQuery::OnChange(wxNotifyEvent& event)
 }
 
 
-void frmQuery::openLastFile()
+void frmQuery::OpenLastFile()
 {
-    wxString str=FileRead(lastPath, this, lastFileFormat ? 1 : 0);
+    wxString str=FileRead(lastPath);
     if (!str.IsEmpty())
     {
         sqlQuery->SetText(str);
+        sqlQuery->Colourise(0, str.Length());
         wxYield();  // needed to process sqlQuery modify event
         changed = false;
         setExtendedTitle();
-        updateRecentFiles();
+        UpdateRecentFiles();
     }
 }
         
 void frmQuery::OnOpen(wxCommandEvent& event)
 {
+    if (CheckChanged(false))
+        return;
+
     wxFileDialog dlg(this, _("Open query file"), lastDir, wxT(""), 
-        _("Query files (*.sql)|*.sql|UTF-8 query files (*.usql)|*.usql|All files (*.*)|*.*"), wxOPEN|wxHIDE_READONLY);
+        _("Query files (*.sql)|*.sql|All files (*.*)|*.*"), wxOPEN);
     if (dlg.ShowModal() == wxID_OK)
     {
         lastFilename=dlg.GetFilename();
         lastDir = dlg.GetDirectory();
         lastPath = dlg.GetPath();
-        switch (dlg.GetFilterIndex())
-        {
-            case 0: 
-                lastFileFormat = false;
-                break;
-            case 1:
-                lastFileFormat = true;
-                break;
-            default:
-                lastFileFormat = settings->GetUnicodeFile();
-                break;
-        }
-        openLastFile();
+        OpenLastFile();
     }
 }
 
@@ -644,18 +713,19 @@ void frmQuery::OnSave(wxCommandEvent& event)
         return;
     }
 
-    if (FileWrite(lastPath, sqlQuery->GetText(), lastFileFormat ? 1 : 0))
+    if (FileWrite(lastPath, sqlQuery->GetText(), -1))
     {
         changed=false;
         setExtendedTitle();
-        updateRecentFiles();
+        UpdateRecentFiles();
     }
 }
 
+
 void frmQuery::OnSaveAs(wxCommandEvent& event)
 {
     wxFileDialog *dlg=new wxFileDialog(this, _("Save query file as"), lastDir, lastFilename, 
-        _("Query files (*.sql)|*.sql|UTF-8 query files (*.usql)|*.usql|All files (*.*)|*.*"), wxSAVE|wxOVERWRITE_PROMPT);
+        _("Query files (*.sql)|*.sql|UTF-8 query files (*.sql)|*.sql|All files (*.*)|*.*"), wxSAVE|wxOVERWRITE_PROMPT);
     if (dlg->ShowModal() == wxID_OK)
     {
         lastFilename=dlg->GetFilename();
@@ -674,7 +744,13 @@ void frmQuery::OnSaveAs(wxCommandEvent& event)
                 break;
         }
 
-        OnSave(event);
+        if (FileWrite(lastPath, sqlQuery->GetText(), lastFileFormat ? 1 : 0))
+        {
+            changed=false;
+            setExtendedTitle();
+            UpdateRecentFiles();
+            fileMenu->Enable(MNU_RECENT, (recentFileMenu->GetMenuItemCount() > 0));
+        }
     }
     delete dlg;
 }
@@ -738,15 +814,32 @@ void frmQuery::OnExecute(wxCommandEvent& event)
     sqlQuery->SetFocus();
 }
 
+
+void frmQuery::OnExecFile(wxCommandEvent &event)
+{
+    wxString query=sqlQuery->GetSelectedText();
+    if (query.IsNull())
+        query = sqlQuery->GetText();
+
+    if (query.IsNull())
+        return;
+    execQuery(query, 0, false, 0, true);
+    sqlQuery->SetFocus();
+}
+
+
 void frmQuery::setTools(const bool running)
 {
     toolBar->EnableTool(MNU_EXECUTE, !running);
+    toolBar->EnableTool(MNU_EXECFILE, !running);
     toolBar->EnableTool(MNU_EXPLAIN, !running);
     toolBar->EnableTool(MNU_CANCEL, running);
     queryMenu->Enable(MNU_EXECUTE, !running);
+    queryMenu->Enable(MNU_EXECFILE, !running);
     queryMenu->Enable(MNU_EXPLAIN, !running);
     queryMenu->Enable(MNU_CANCEL, running);
     fileMenu->Enable(MNU_EXPORT, sqlResult->CanExport());
+    fileMenu->Enable(MNU_RECENT, (recentFileMenu->GetMenuItemCount() > 0));
 }
 
 
@@ -763,7 +856,7 @@ void frmQuery::showMessage(const wxString& msg, const wxString &msgShort)
     SetStatusText(str, STATUSPOS_MSGS);
 }
 
-void frmQuery::execQuery(const wxString &query, int resultToRetrieve, bool singleResult, const int queryOffset)
+void frmQuery::execQuery(const wxString &query, int resultToRetrieve, bool singleResult, const int queryOffset, bool toFile)
 {
     long rowsReadTotal=0;
     setTools(true);
@@ -794,13 +887,14 @@ void frmQuery::execQuery(const wxString &query, int resultToRetrieve, bool singl
         wxYield();
 
         wxString str;
+        wxLongLong elapsedQuery;
         wxLongLong startTimeQuery=wxGetLocalTimeMillis();
         while (sqlResult->RunStatus() == CTLSQL_RUNNING)
         {
             elapsedQuery=wxGetLocalTimeMillis() - startTimeQuery;
             SetStatusText(elapsedQuery.ToString() + wxT(" ms"), STATUSPOS_SECS);
             wxYield();
-            wxUsleep(10);
+            wxMilliSleep(10);
             str=sqlResult->GetMessagesAndClear();
             if (!str.IsEmpty())
             {
@@ -822,9 +916,25 @@ void frmQuery::execQuery(const wxString &query, int resultToRetrieve, bool singl
             output->SetSelection(1);
             if (sqlResult->RunStatus() == PGRES_COMMAND_OK)
             {
-                showMessage(wxString::Format(_("Query returned successfully with no result in %s ms."),
-                    elapsedQuery.ToString().c_str()), _("OK."));
-//                wxMessageBox(illegal string for translation...("Query returned successfully with no result in ") + qTime, _("Query Results"), wxICON_INFORMATION | wxOK);
+                int insertedCount = sqlResult->InsertedCount();
+                OID insertedOid = sqlResult->InsertedOid();
+                if (insertedCount < 0)
+                {
+                    showMessage(wxString::Format(_("Query returned successfully with no result in %s ms."),
+                        elapsedQuery.ToString().c_str()), _("OK."));
+                }
+                else if (insertedCount == 1 && insertedOid)
+                {
+                    showMessage(wxString::Format(_("Query returned successfully: one row with OID %d inserted, %s ms execution time."),
+                        insertedOid, elapsedQuery.ToString().c_str()), 
+                        wxString::Format(_("One Row with OID %d inserted."), insertedOid));
+                }
+                else
+                {
+                    showMessage(wxString::Format(_("Query returned successfully: %d rows affected, %s ms execution time."),
+                        insertedCount, elapsedQuery.ToString().c_str()), 
+                        wxString::Format(_("%d rows affected."), insertedCount));
+                }
             }
             else
             {
@@ -866,105 +976,129 @@ void frmQuery::execQuery(const wxString &query, int resultToRetrieve, bool singl
             output->SetSelection(0);
             long rowsTotal=sqlResult->NumRows();
 
-            if (singleResult)
+            if (toFile)
             {
-                rowsReadTotal=sqlResult->RetrieveOne();
-                showMessage(wxString::Format(_("%d rows retrieved."), rowsReadTotal), _("OK."));
+                SetStatusText(wxString::Format(_("%d rows."), rowsTotal), STATUSPOS_ROWS);
+
+                if (rowsTotal)
+                {
+                    SetStatusText(_("Writing data."), STATUSPOS_MSGS);
+
+                    toolBar->EnableTool(MNU_CANCEL, false);
+                    queryMenu->Enable(MNU_CANCEL, false);
+                    SetCursor(*wxHOURGLASS_CURSOR);
+
+                    if (sqlResult->Export())
+                        SetStatusText(_("Data written to file."), STATUSPOS_MSGS);
+                    else
+                        SetStatusText(_("Data export aborted."), STATUSPOS_MSGS);
+                    SetCursor(wxNullCursor);
+                }
+                else
+                    SetStatusText(_("No data to export."), STATUSPOS_MSGS);
             }
             else
             {
-                SetStatusText(wxString::Format(_("Retrieving data: %d rows."), rowsTotal), STATUSPOS_MSGS);
-                wxYield();
-
-                long maxRows=settings->GetMaxRows();
-
-                if (!maxRows)
-                    maxRows = rowsTotal;
-                if (rowsTotal > maxRows)
+                if (singleResult)
                 {
-                    wxMessageDialog msg(this, wxString::Format(
-                            _("The maximum of %ld rows is exceeded (total %ld)."), maxRows, rowsTotal) +
-                            _("\nRetrieve all rows anyway?"), _("Limit exceeded"), 
-                                wxYES_NO|wxCANCEL|wxNO_DEFAULT|wxICON_EXCLAMATION);
-                    switch (msg.ShowModal())
-                    {
-                        case wxID_YES:
-                            maxRows = rowsTotal;
-                            break;
-                        case wxID_CANCEL:
-                            maxRows = 0;
-                            break;
-                    }
+                    rowsReadTotal=sqlResult->RetrieveOne();
+                    showMessage(wxString::Format(_("%d rows retrieved."), rowsReadTotal), _("OK."));
                 }
-                wxLongLong startTimeRetrieve=wxGetLocalTimeMillis();
-                wxLongLong elapsed;
-                elapsedRetrieve=0;
-		bool resultFreezed=false;
-                
-                while (!aborted && rowsReadTotal < maxRows)
-                {
-                    // Rows will be retrieved in chunks, the first being smaller to have an early screen update
-                    // later, screen update is disabled to speed up retrieval
-                    long chunk;
-                    if (!rowsReadTotal) chunk=20;
-                    else                chunk=100;
-
-                    if (chunk > maxRows-rowsReadTotal)
-                        chunk = maxRows-rowsReadTotal;
-
-                    long rowsRead=sqlResult->Retrieve(chunk);
-                    if (!rowsRead)
-                        break;
-
-                    elapsed = wxGetLocalTimeMillis() - startTimeRetrieve;
-
-                    if (!rowsReadTotal)
-		            {
-                        wxYield();
-			            if (rowsRead < maxRows)
-				    {
-					resultFreezed=true;
-					sqlResult->Freeze();
-				    }
-		            }
-                    rowsReadTotal += rowsRead;
-
-                    if (elapsed > elapsedRetrieve +100)
-                    {
-                        elapsedRetrieve=elapsed;
-                        SetStatusText(elapsedQuery.ToString() + wxT("+") + elapsedRetrieve.ToString() + wxT(" ms"), STATUSPOS_SECS);
-                        wxYield();
-                    }
-                }
-		if (resultFreezed)
-		    sqlResult->Thaw();
-
-                elapsedRetrieve=wxGetLocalTimeMillis() - startTimeRetrieve;
-                SetStatusText(elapsedQuery.ToString() + wxT("+") + elapsedRetrieve.ToString() + wxT(" ms"), STATUSPOS_SECS);
-
-                str= _("Total query runtime: ") + elapsedQuery.ToString() + wxT(" ms.\n") +
-                     _("Data retrieval runtime: ") + elapsedRetrieve.ToString() + wxT(" ms.\n");
-                msgResult->AppendText(str);
-                msgHistory->AppendText(str);
-
-                if (rowsReadTotal == sqlResult->NumRows())
-                    showMessage(wxString::Format(_("%ld rows retrieved."), rowsReadTotal), _("OK."));
                 else
                 {
-                    wxString nr;
-                    nr.Printf(_("%ld  rows not retrieved."), rowsTotal - rowsReadTotal);
-                    showMessage(wxString::Format(_("Total %ld rows.\n"), rowsTotal) + nr, nr);
-                }
-                msgHistory->AppendText(wxT("\n"));
+                    SetStatusText(wxString::Format(_("Retrieving data: %d rows."), rowsTotal), STATUSPOS_MSGS);
+                    wxYield();
 
+                    long maxRows=settings->GetMaxRows();
+
+                    if (!maxRows)
+                        maxRows = rowsTotal;
+                    if (rowsTotal > maxRows)
+
+                    {
+                        wxMessageDialog msg(this, wxString::Format(
+                                _("The maximum of %ld rows is exceeded (total %ld)."), maxRows, rowsTotal) +
+                                _("\nRetrieve all rows anyway?"), _("Limit exceeded"), 
+                                    wxYES_NO|wxCANCEL|wxNO_DEFAULT|wxICON_EXCLAMATION);
+                        switch (msg.ShowModal())
+                        {
+                            case wxID_YES:
+                                maxRows = rowsTotal;
+                                break;
+                            case wxID_CANCEL:
+                                maxRows = 0;
+                                break;
+                        }
+                    }
+                    wxLongLong startTimeRetrieve=wxGetLocalTimeMillis();
+                    wxLongLong elapsed;
+                    elapsedRetrieve=0;
+                    bool resultFreezed=false;
+                
+                    while (!aborted && rowsReadTotal < maxRows)
+                    {
+                        // Rows will be retrieved in chunks, the first being smaller to have an early screen update
+                        // later, screen update is disabled to speed up retrieval
+                        long chunk;
+                        if (!rowsReadTotal) chunk=20;
+                        else                chunk=100;
+
+                        if (chunk > maxRows-rowsReadTotal)
+                            chunk = maxRows-rowsReadTotal;
+
+                        long rowsRead=sqlResult->Retrieve(chunk);
+                        if (!rowsRead)
+                            break;
+
+                        elapsed = wxGetLocalTimeMillis() - startTimeRetrieve;
+
+                        if (!rowsReadTotal)
+		                {
+                            wxYield();
+			                if (rowsRead < maxRows)
+    				        {
+        					    resultFreezed=true;
+	        				    sqlResult->Freeze();
+			    	        }
+		                }
+                        rowsReadTotal += rowsRead;
+
+                        if (elapsed > elapsedRetrieve +100)
+                        {
+                            elapsedRetrieve=elapsed;
+                            SetStatusText(elapsedQuery.ToString() + wxT("+") + elapsedRetrieve.ToString() + wxT(" ms"), STATUSPOS_SECS);
+                            wxYield();
+                        }
+                    }
+                    if (resultFreezed)
+                        sqlResult->Thaw();
+
+                    elapsedRetrieve=wxGetLocalTimeMillis() - startTimeRetrieve;
+                    SetStatusText(elapsedQuery.ToString() + wxT("+") + elapsedRetrieve.ToString() + wxT(" ms"), STATUSPOS_SECS);
+
+                    str= _("Total query runtime: ") + elapsedQuery.ToString() + wxT(" ms.\n") +
+                         _("Data retrieval runtime: ") + elapsedRetrieve.ToString() + wxT(" ms.\n");
+                    msgResult->AppendText(str);
+                    msgHistory->AppendText(str);
+
+                    if (rowsReadTotal == sqlResult->NumRows())
+                        showMessage(wxString::Format(_("%ld rows retrieved."), rowsReadTotal), _("OK."));
+                    else
+                    {
+                        wxString nr;
+                        nr.Printf(_("%ld  rows not retrieved."), rowsTotal - rowsReadTotal);
+                        showMessage(wxString::Format(_("Total %ld rows.\n"), rowsTotal) + nr, nr);
+                    }
+                    msgHistory->AppendText(wxT("\n"));
+
+                }
+                if (rowsTotal == rowsReadTotal)
+                    SetStatusText(wxString::Format(_("%d rows."), rowsTotal), STATUSPOS_ROWS);
+                else
+                    SetStatusText(wxString::Format(_("%ld of %ld rows"), rowsReadTotal, rowsTotal), STATUSPOS_ROWS);
             }
-            if (rowsTotal == rowsReadTotal)
-                SetStatusText(wxString::Format(_("%d rows."), rowsTotal), STATUSPOS_ROWS);
-            else
-                SetStatusText(wxString::Format(_("%ld of %ld rows"), rowsReadTotal, rowsTotal), STATUSPOS_ROWS);
         }
     }
-
     setTools(false);
     if (rowsReadTotal)
     {
