@@ -1,20 +1,28 @@
 //////////////////////////////////////////////////////////////////////////
 //
 // pgAdmin III - PostgreSQL Tools
-// Copyright (C) 2002 - 2003, The pgAdmin Development Team
+// RCS-ID:      $Id: misc.cpp,v 1.62 2004/10/15 10:43:45 andreas Exp $
+// Copyright (C) 2002 - 2004, The pgAdmin Development Team
 // This software is released under the Artistic Licence
 //
 // misc.cpp - Miscellaneous Utilities
 //
 //////////////////////////////////////////////////////////////////////////
 
+#include "pgAdmin3.h"
+
 // wxWindows headers
 #include <wx/wx.h>
 #include <wx/app.h>
 #include <wx/timer.h>
 #include <wx/xrc/xmlres.h>
 #include <wx/file.h>
+#include <wx/textbuf.h>
 #include <wx/help.h>
+#include <wx/fontenc.h>
+
+#include "utffile.h"
+#include <locale.h>
 
 #ifdef __WXMSW__
 #include <wx/msw/helpchm.h>
@@ -26,14 +34,16 @@
 // App headers
 #include "misc.h"
 #include "menu.h"
-#include "pgAdmin3.h"
 #include "frmMain.h"
 #include "frmHelp.h"
 
-
 extern "C"
 {
+#define YYSTYPE_IS_DECLARED
+#define DECIMAL DECIMAL_P
+  typedef int YYSTYPE;
 #include "parser/keywords.h"
+#include "parser/parse.h"
 }
 
 // we dont have an appropriate wxLongLong method
@@ -53,47 +63,6 @@ extern "C"
 
 
 
-// Global Vars - yuch!
-wxStopWatch stopwatch;
-wxString timermsg;
-long msgLevel=0;
-
-void StartMsg(const wxString& msg)
-{
-    extern frmMain *winMain;
-
-    if (msgLevel++)
-        return;
-
-    timermsg.Printf(wxT("%s..."), msg.c_str());
-    wxBeginBusyCursor();
-    stopwatch.Start(0);
-    wxLogStatus(timermsg);
-    winMain->statusBar->SetStatusText(timermsg, 1);
-    winMain->statusBar->SetStatusText(wxT(""), 2);
-}
-
-void EndMsg()
-{
-    extern frmMain *winMain;
-
-    msgLevel--;
-
-    if (!msgLevel)
-    {
-        // Get the execution time & display it
-        float timeval = stopwatch.Time();
-        wxString time;
-        time.Printf(_("%.2f secs"), (timeval/1000));
-        winMain->statusBar->SetStatusText(time, 2);
-
-        // Display the 'Done' message
-        winMain->statusBar->SetStatusText(timermsg + _(" Done."), 1);
-        wxLogStatus(wxT("%s (%s)"), timermsg.c_str(), time.c_str());
-        wxEndBusyCursor();
-    }
-}
-
 // Conversions
 
 
@@ -105,7 +74,7 @@ wxString BoolToYesNo(bool value)
 
 wxString BoolToStr(bool value)
 {
-    return value ? wxT("Yes") : wxT("No");
+    return value ? wxT("true") : wxT("false");
 }
 
 
@@ -138,7 +107,7 @@ wxString NumToStr(long value)
 wxString NumToStr(OID value)
 {
     wxString result;
-    result.Printf(wxT("%u"), (long)value);
+    result.Printf(wxT("%lu"), (long)value);
     return result;
 }
 
@@ -151,9 +120,10 @@ long StrToLong(const wxString& value)
 
 OID StrToOid(const wxString& value)
 {
-    return (OID)atol(value.ToAscii());
+    return (OID)strtoul(value.ToAscii(), 0, 10);
 }
 
+
 wxString NumToStr(double value)
 {
     wxString result;
@@ -179,7 +149,12 @@ wxString NumToStr(double value)
 
 double StrToDouble(const wxString& value)
 {
-    return strtod(value.ToAscii(), 0);
+    wxCharBuffer buf = value.ToAscii();
+    char *p=strchr(buf, '.');
+    if (p)
+        *p = localeconv()->decimal_point[0];
+
+    return strtod(buf, 0);
 }
 
 
@@ -189,20 +164,31 @@ wxULongLong StrToLongLong(const wxString &value)
 }
 
 
-wxString GetListText(wxListCtrl *lst, long row, long col)
+wxString DateToAnsiStr(const wxDateTime &datetime)
 {
-    wxListItem item;
-    item.SetId(row);
-    item.SetColumn(col);
-    item.SetMask(wxLIST_MASK_TEXT);
-    lst->GetItem(item);
-    return item.GetText();
+    if (!datetime.IsValid())
+        return wxEmptyString;
+
+    return datetime.FormatISODate() + wxT(" ") + datetime.FormatISOTime();
 }
 
 
-long GetListSelected(wxListCtrl *lst)
+wxString DateToStr(const wxDateTime &datetime)
 {
-    return lst->GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
+    if (!datetime.IsValid())
+        return wxEmptyString;
+
+    return datetime.FormatDate() + wxT(" ") + datetime.FormatTime();
+}
+
+
+wxDateTime StrToDateTime(const wxString &value)
+{
+    wxDateTime dt;
+    /* This hasn't just been used. ( Is not infinity ) */
+    if ( !value.IsEmpty() )
+    dt.ParseDateTime(value);
+    return dt;
 }
 
 
@@ -247,7 +233,7 @@ wxString qtString(const wxString& value)
 }
 
 
-wxString qtDocumentHere(const wxString &value)
+wxString qtStringDollar(const wxString &value)
 {
     wxString qtDefault=wxT("BODY");
     wxString qt=qtDefault;
@@ -265,72 +251,169 @@ wxString qtDocumentHere(const wxString &value)
 }
 
 
-wxString qtIdent(const wxString& value)
+void FillKeywords(wxString &str)
 {
-    wxString result = value;
-    
-    if (result.Length() == 0)
-        return result;
+    // unfortunately, the keyword list is static. 
+    // If the first or the last word change, these both need to get updated.
+    const ScanKeyword *keyword=ScanKeywordLookup("abort");
+    const ScanKeyword *last=ScanKeywordLookup("zone");
 
-    int pos = 0;
+    wxASSERT(keyword && last);
 
-#if 0
-    if (ScanKeywordLookup(value.ToAscii()))
-    {
-        result.Append(wxT("\""));
-        result.Prepend(wxT("\""));
-        return result;
-    }
-#endif
+    str = wxString::FromAscii(keyword->name);
+
+    while (keyword++ < last)
+        str += wxT(" ") + wxString::FromAscii(keyword->name);
+}
+
+
+static bool needsQuoting(wxString& value, bool forTypes)
+{
     // Replace Double Quotes
-    result.Replace(wxT("\""), wxT("\"\""));
-	
+    if (value.Replace(wxT("\""), wxT("\"\"")) > 0)
+        return true;
+
     // Is it a number?
-    if (result.IsNumber()) {
-        result.Append(wxT("\""));
-        result.Prepend(wxT("\""));
-        return result;
-    } else {
-        while (pos < (int)result.length()) {
-            if (!((result.GetChar(pos) >= '0') && (result.GetChar(pos) <= '9')) && 
-                !((result.GetChar(pos)  >= 'a') && (result.GetChar(pos)  <= 'z')) && 
-                !(result.GetChar(pos)  == '_')){
-            
-                result.Append(wxT("\""));
-                result.Prepend(wxT("\""));
-                return result;	
+    if (value.IsNumber()) 
+        return true;
+    else
+    {
+        int pos = 0;
+        while (pos < (int)value.length())
+        {
+            wxChar c=value.GetChar(pos);
+            if (!(c >= '0' && c <= '9') && 
+                !(c >= 'a' && c  <= 'z') && 
+                !(c == '_'))
+            {
+                return true;
             }
             pos++;
         }
-    }	
-    return result;
+    }
+
+    // is it a keyword?
+    const ScanKeyword *sk=ScanKeywordLookup(value.ToAscii());
+    if (sk)
+    {
+        if (forTypes)
+        {
+            switch (sk->value)
+            {
+                case ANY:
+                case BIGINT:
+                case BIT:
+                case BOOLEAN_P:
+                case CHAR_P:
+                case CHARACTER:
+                case DECIMAL:
+                case DOUBLE_P:
+                case FLOAT_P:
+                case INT_P:
+                case INTEGER:
+                case INTERVAL:
+                case NUMERIC:
+                case SET:
+                case SMALLINT:
+                case TIME:
+                case TIMESTAMP:
+                case TRIGGER:
+                case VARCHAR:
+                    break;
+                default:
+                    return true;
+            }
+        }
+        else
+            return true;
+    }
+
+    return false;
 }
 
-// Keith 2003.03.11
-// We need an identifier validation function
-bool IsValidIdentifier(wxString ident)
+
+wxString qtTypeIdent(const wxString& value)
 {
-    // compiler complains if not unsigned
-	unsigned int len = ident.length();
-	if (!len)
-		return false;
+    if (value.Length() == 0)
+        return value;
 
-	wxString first = 
-		wxT("_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ");
-	wxString second = 
-		wxT("_0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ");
+    wxString result = value;
 
-	if (!first.Contains(ident.Left(1)))
-		return false;
+    if (needsQuoting(result, true))
+        return wxT("\"") + result + wxT("\"");
+    else
+        return result;
+}
 
-	unsigned int si;
-	for ( si = 1; si < len; si++)
-	{
-	    if (!second.Contains(ident.Mid(si, 1)))
-		    return false;
-	}
 
-	return true;
+wxString qtIdent(const wxString& value)
+{
+    if (value.Length() == 0)
+        return value;
+
+    wxString result = value;
+
+    if (needsQuoting(result, false))
+        return wxT("\"") + result + wxT("\"");
+    else
+        return result;
+}
+
+
+wxString qtStrip(const wxString &str)
+{
+    if (str.Left(2) == wxT("\\\""))
+        return str.Mid(2, str.Length()-4);
+    else
+        return str;
+}
+
+
+void FillArray(wxArrayString &array, const wxString &list)
+{
+    const wxChar *cp=list.c_str();
+
+    wxString str;
+    bool quote=false;
+    
+    while (*cp)
+    {
+        switch (*cp)
+        {
+            case '\\':
+            {
+                if (cp[1]== '"')
+                {
+                    cp++;
+                    str.Append(*cp);
+                }
+                break;
+            }
+            case '"':
+            case '\'':
+            {
+                quote = !quote;
+                break;
+            }
+            case ',':
+            {
+                if (!quote)
+                {
+                    array.Add(str);
+                    str=wxEmptyString;
+                    break;
+                }
+            }
+            default:
+            {
+                str.Append(*cp);
+                break;
+            }
+        }
+        cp++;
+    }
+    if (!str.IsEmpty())
+        array.Add(str);
 }
 
 
@@ -383,74 +466,50 @@ wxString queryTokenizer::GetNextToken()
 }
 
 
-wxString FileRead(const wxString &filename, wxWindow *errParent, int format)
+wxString FileRead(const wxString &filename, int format)
 {
     wxString str;
 
-    wxFile file(filename);
+    wxFontEncoding encoding;
+    if (format > 0)
+        encoding = wxFONTENCODING_UTF8;
+    else
+        encoding=wxFONTENCODING_DEFAULT;
+
+    wxUtfFile file(filename, wxFile::read, encoding);
+
     if (file.IsOpened())
-    {
-        int len=file.Length();
-        char *buf=new char[len+1];
-        memset(buf, 0, len+1);
-        file.Read(buf, len);
-        file.Close();
+        file.Read(str);
 
-#if wxUSE_UNICODE
-        if (format < 0)
-            format = (settings->GetUnicodeFile() ? 1 : 0);
- 
-        size_t nLen;
-        if (format)
-            nLen = wxConvUTF8.MB2WC(0, buf, 0);
-        else
-            nLen = wxConvLibc.MB2WC(0, buf, 0);
-
-        if (nLen == (size_t) -1)
-        {
-            // Format error
-            if (errParent)
-                wxMessageDialog (errParent, _("File cannot be opened with this format.\nPlease chose another format."), 
-                _("FATAL"), wxOK|wxCENTRE|wxICON_ERROR).ShowModal();
-        }
-        else
-        {
-            str=wxString(' ', nLen);
-            if (format)
-                wxConvUTF8.MB2WC((wxChar*)str.c_str(), buf, nLen);
-            else
-                wxConvLibc.MB2WC((wxChar*)str.c_str(), buf, nLen);
-            str.Replace(wxT("\r"), wxT(""));
-        }
-#else
-        str=buf;
-        str.Replace(wxT("\r"), wxT(""));
-#endif
-        delete[] buf;
-    }
-    return str;
+    return wxTextBuffer::Translate(str, wxTextFileType_Unix);   // we want only \n
 }
 
 
 bool FileWrite(const wxString &filename, const wxString &data, int format)
 {
-    wxFile file(filename, wxFile::write);
-    if (file.IsOpened())
-    {
-        wxString buf=data;
-#ifdef __WIN32__
-        buf.Replace(wxT("\n"), wxT("\r\n"));
-#endif
-        if (format < 0)
-            format = settings->GetUnicodeFile() ? 1 : 0;
+    wxFontEncoding encoding = wxFONTENCODING_DEFAULT;
+    wxUtfFile file;
 
-        if (format == 1)
-            file.Write(buf, wxConvUTF8);
-        else
-            file.Write(buf, wxConvLibc);
-        file.Close();
-        return true;
+    if (format < 0)
+    {
+        if (wxFile::Access(filename, wxFile::read))
+        {
+            file.Open(filename);
+            encoding = file.GetEncoding();
+            file.Close();
+        }
+        if (encoding == wxFONTENCODING_DEFAULT)
+            encoding = settings->GetUnicodeFile() ? wxFONTENCODING_UTF8 : wxFONTENCODING_SYSTEM;
     }
+    else
+        encoding = format ? wxFONTENCODING_UTF8 : wxFONTENCODING_SYSTEM;
+
+
+    file.Open(filename, wxFile::write, wxS_DEFAULT, encoding);
+
+    if (file.IsOpened())
+        return file.Write(wxTextBuffer::Translate(data));
+
     return false;
 }
 
@@ -464,29 +523,35 @@ void DisplayHelp(wxWindow *wnd, const wxString &helpTopic, char **icon)
     if (firstCall)
     {
         firstCall=false;
-        wxString helpfile=docPath + wxT("/") + settings->GetCanonicalLanguage() + wxT("/pgadmin3");
+        wxString helpdir=docPath + wxT("/") + settings->GetCanonicalLanguage();
 
-        if (!wxFile::Exists(helpfile + wxT(".hhp")) &&
-#ifdef __WXMSW__
-            !wxFile::Exists(helpfile + wxT(".chm")) &&
+        if (!wxFile::Exists(helpdir + wxT("/pgadmin3..hhp")) &&
+#if defined(__WXMSW__) || wxUSE_LIBMSPACK
+            !wxFile::Exists(helpdir + wxT("/pgadmin3.chm")) &&
 #endif
-            !wxFile::Exists(helpfile + wxT(".zip")))
-            helpfile=docPath + wxT("/en_US/pgadmin3");
+            !wxFile::Exists(helpdir + wxT("/pgadmin3.zip")))
+            helpdir=docPath + wxT("/en_US");
 
 #ifdef __WXMSW__
 #ifndef __WXDEBUG__
-        if (wxFile::Exists(helpfile + wxT(".chm")))
+       if (wxFile::Exists(helpdir + wxT("/pgadmin3.chm")))
         {
             helpCtl=new wxCHMHelpController();
-            helpCtl->Initialize(helpfile);
+            helpCtl->Initialize(helpdir + wxT("/pgadmin3"));
         }
         else
 #endif
 #endif
-        if (wxFile::Exists(helpfile + wxT(".hhp")) || wxFile::Exists(helpfile + wxT(".zip")))
+#if wxUSE_LIBMSPACK
+        if (wxFile::Exists(helpdir + wxT("/pgadmin3.chm")) ||
+            wxFile::Exists(helpdir + wxT("/pgadmin3.hhp")) || wxFile::Exists(helpdir + wxT("/pgadmin3.zip")))
+#else
+        if (wxFile::Exists(helpdir + wxT("/pgadmin3.hhp")) || wxFile::Exists(helpdir + wxT("/pgadmin3.zip")))
+#endif
         {
             helpCtl=new wxHtmlHelpController();
-            helpCtl->Initialize(helpfile);
+			((wxHtmlHelpController*)helpCtl)->SetTempDir(helpdir);
+            helpCtl->Initialize(helpdir + wxT("/pgadmin3"));
         }
     }
 
@@ -509,39 +574,31 @@ void DisplayHelp(wxWindow *wnd, const wxString &helpTopic, char **icon)
 
 void DisplaySqlHelp(wxWindow *wnd, const wxString &helpTopic, char **icon)
 {
-    if (settings->GetSqlHelpSite().length() != 0) 
+    if (helpTopic.Left(8) == wxT("pgadmin/"))
+        DisplayHelp(wnd, helpTopic.Mid(8), icon);
+    else if (settings->GetSqlHelpSite().length() != 0) 
         frmHelp::LoadSqlDoc(wnd, helpTopic  + wxT(".html"));
     else
         DisplayHelp(wnd, wxT("pg/") + helpTopic, icon);
 }
 
 
-
-BEGIN_EVENT_TABLE(DialogWithHelp, wxDialog)
-    EVT_MENU(MNU_HELP,                  DialogWithHelp::OnHelp)
-    EVT_BUTTON(XRCID("btnHelp"),        DialogWithHelp::OnHelp)
-END_EVENT_TABLE();
-
-
-DialogWithHelp::DialogWithHelp(frmMain *frame) : wxDialog()
+#ifndef WIN32
+wxString ExecProcess(const wxString &cmd)
 {
-    mainForm = frame;
+    wxString res;
+	FILE *f=popen(cmd.ToAscii(), "r");
 
-    wxAcceleratorEntry entries[2];
-    entries[0].Set(wxACCEL_NORMAL, WXK_F1, MNU_HELP);
-// this is for GTK because Meta (usually Numlock) is interpreted like Alt
-// there are too many controls to reset m_Meta in all of them
-    entries[1].Set(wxACCEL_ALT, WXK_F1, MNU_HELP);
-    wxAcceleratorTable accel(2, entries);
-
-    SetAcceleratorTable(accel);
-}
-
-
-void DialogWithHelp::OnHelp(wxCommandEvent& ev)
-{
-    wxString page=GetHelpPage();
-
-    if (!page.IsEmpty())
-        DisplaySqlHelp(this, page);
+	if (f)
+	{
+		char buffer[1024];
+		int cnt;
+		while ((cnt = fread(buffer, 1, 1024, f)) > 0)
+		{
+			res += wxString::FromAscii(buffer);
+		}
+		pclose(f);
+	}
+	return res;
 }
+#endif
